#include <algorithm>
#include <iostream>
#include <numeric>
#include <vector>
#include <queue>
#include <set>

typedef long long llong;
const int MAXN = 800 + 10;
const llong INF = 1e18;

int n, m;
int p, G;
llong w[MAXN];
llong used[MAXN][MAXN];
llong money[MAXN][MAXN];
std::vector <std::pair <int,int>> g[MAXN];

struct DijkstraElement
{
    int node;
    int max;
    llong used;
    llong money;

    inline friend bool operator < (const DijkstraElement &a, const DijkstraElement &b)
    {
        return a.used > b.used || (a.used == b.used && a.money < b.money);
    }
};

bool bl[MAXN][MAXN];
std::priority_queue <DijkstraElement> pq;
void solve()
{
    for (int i = 1 ; i <= n ; ++i)
    {
        for (int j = 1 ; j <= n ; ++j)
        {
            used[i][j] = 1e18;
            money[i][j] = -1e18;
            bl[i][j] = false;
        }
    }

    used[1][1] = 0;
    money[1][1] = p;
    pq.push({1, 1, 0, money[1][1]});
    while (!pq.empty())
    {
        auto [node, max, currUsed, currMoney] = pq.top();
        pq.pop();

        if (bl[node][max]) continue;
        bl[node][max] = true;

        for (const auto &[next, t] : g[node])
        {
            llong add = std::max(0LL, (t - currMoney) / w[max] + (((t - currMoney) % w[max]) > 0));
            llong newUsed = currUsed + add;
            llong newMoney = currMoney + add * w[max] - t;
            int newMax = max;
            if (w[max] < w[next])
            {
                newMax = next;
            }

            if (used[next][newMax] > newUsed || (used[next][newMax] == newUsed && money[next][newMax] < newMoney))
            {
                used[next][newMax] = newUsed;
                money[next][newMax] = newMoney;
                pq.push({next, newMax, newUsed, newMoney});
            } 
        }
    }
    
    llong ans = 1e18;
    for (int i = 1 ; i <= n ; ++i)
    {
        if (money[n][i] >= 0)
        {
            ans = std::min(ans, used[n][i]);
        }
    }
    
    if (ans == 1e18) std::cout << -1 << '\n';
    else std::cout << ans << '\n';
}

void input()
{
    std::cin >> n >> m >> p >> G;
    for (int i = 1 ; i <= n ; ++i) std::cin >> w[i];
    for (int i = 1 ; i <= m ; ++i)
    {
        int u, v, t;
        std::cin >> u >> v >> t;
        g[u].emplace_back(v, t);
    }
}

void fastIO()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIO();
    input();
    solve();

    return 0;
}


