use std::io::{self, prelude::*};

struct Tree {
    n: usize,
    s: Vec<usize>
}

impl Tree {
    fn new(n: usize) -> Self {
        Tree {
            n,
            s: vec![0; 2 * n]
        }
    }

    fn update(&mut self, mut p: usize, val: usize) {
        p += self.n;
        self.s[p] = val;
        while p > 1 {
            self.s[p >> 1] = std::cmp::max(self.s[p], self.s[p ^ 1]);
            p >>= 1;
        }
    }

    fn query(&self, mut l: usize, mut r: usize) -> usize {
        let mut res = 0;
        l += self.n; r += self.n;
        while l < r {
            if (l&1) == 1 {
                res = res.max(self.s[l]);
                l += 1;
            }

            if (r&1) == 1 {
                r -= 1;
                res = res.max(self.s[r]);
            }
            l >>= 1;
            r >>= 1;
        }
        res
    }
}

fn solve<R: BufRead, W: Write>(scan: &mut Scanner<R>, w: &mut W) -> io::Result<()> {
    let n = scan.token();
    let albs = (0..n)
        .map(|_| {
            (0..scan.token())
                .map(|_| scan.token())
                .collect::<Vec<i32>>()
        })
        .collect::<Vec<_>>();

    let mut albs = albs
        .into_iter()
        .map(|alb| {
            alb.into_iter().fold(vec![], |mut acc, x| {
                if acc.is_empty() || acc.last() < Some(&x) {
                    acc.push(x);
                }
                acc
            })
        })
        .collect::<Vec<_>>();
    albs.sort_unstable_by_key(|a| a.last().cloned());

    let ks: Vec<_> = albs.iter().map(|v| v.len()).collect();
    let mut dp: Vec<_> = ks.iter().map(|&k| vec![0; k + 1]).collect();
    let mut mxdp = vec![0; n];
    dp[0]
        .iter_mut()
        .enumerate()
        .for_each(|(i, x)| *x = ks[0] - i);
    mxdp[0] = ks[0];
    let mut tree = Tree::new(n);
    tree.update(0, mxdp[0]);
    for i in 1..n {
        dp[i][ks[i]] = mxdp[i - 1];
        // dp[i][0] = ks[i];
        for start in 0..ks[i] {
            let bound = albs[..i].partition_point(|x| x.last() < albs[i].get(start));
            dp[i][start] = ks[i] - start + tree.query(0, bound);
        }
        mxdp[i] = *dp[i].iter().max().unwrap();
        tree.update(i, mxdp[i]);
    }

    writeln!(w, "{}", mxdp.iter().max().unwrap())
}

fn main() -> io::Result<()> {
    let mut scan = Scanner::new(io::BufReader::new(io::stdin().lock()));
    let mut w = io::BufWriter::new(io::stdout().lock());
    solve(&mut scan, &mut w)
}

struct Scanner<R> {
    reader: R,
    buf: Vec<u8>,
    buf_iter: std::str::SplitAsciiWhitespace<'static>,
}

impl<R: BufRead> Scanner<R> {
    fn new(reader: R) -> Self {
        Self {
            reader,
            buf: vec![],
            buf_iter: "".split_ascii_whitespace(),
        }
    }

    fn token<T: std::str::FromStr>(&mut self) -> T {
        loop {
            if let Some(token) = self.buf_iter.next() {
                return token.parse().ok().expect("Failed parse");
            }
            self.buf.clear();
            self.reader
                .read_until(b'\n', &mut self.buf)
                .expect("Failed read");
            self.buf_iter = unsafe {
                let slice = std::str::from_utf8_unchecked(&self.buf);
                std::mem::transmute(slice.split_ascii_whitespace())
            };
        }
    }
}
