#include <iostream>
#include <vector>
#include <set>
#include <algorithm>
#include <unordered_map>
#include <numeric>

#if !defined LOCAL && !defined DEBUG
#define USE_FAST_IO
#endif

#ifdef USE_FAST_IO
#define endl '\n'
#endif

using namespace std;

const int INF = 1'000'000'007;

struct SegmentTree {
    vector<int> tree;
    int sz;

    void update(int v, int l, int r, int idx, int value) {
        if (l + 1 == r) {
            tree[v] = max(tree[v], value);
            return;
        }
        int m = (l + r) / 2;
        if (idx < m) {
            update(2 * v + 1, l, m, idx, value);
        } else {
            update(2 * v + 2, m, r, idx, value);
        }
        tree[v] = max(tree[2 * v + 1], tree[2 * v + 2]);
    }

    int get(int v, int l, int r, int ql, int qr) {
        if (ql >= r || l >= qr) {
            return -INF;
        }
        if (ql <= l && r <= qr) {
            return tree[v];
        }
        int m = (l + r) / 2;
        return max(get(2 * v + 1, l, m, ql, qr), get(2 * v + 2, m, r, ql, qr));
    }

    void update(int idx, int value) {
        update(0, 0, sz, idx, value);
    }

    int get(int ql, int qr) {
        return get(0, 0, sz, ql, qr);
    }

    SegmentTree(int n) {
        tree.assign(4 * n + 5, -INF);
        sz = n;
    }
};

const int MAX_C = 200228;

int main() {
#ifdef USE_FAST_IO
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int n; cin >> n;
    vector<vector<int>> albums;
    vector<vector<int>> album_coords;
    vector<vector<int>> album_values;
    vector<int> maxs;
    for (int i = 0; i < n; i++) {
        int k; cin >> k;
        vector<int> album(k);
        vector<int> coords;
        for (int &x : album) {
            cin >> x;
            coords.push_back(x);
        }
        sort(coords.begin(), coords.end());
        coords.resize(unique(coords.begin(), coords.end()) - coords.begin());
        unordered_map<int, int> wrap;
        int sz = coords.size();
        for (int i = 0; i < sz; i++) {
            wrap[coords[i]] = i;
        }
        int pr_max = 0;
        int initial = 0;
        vector<int> pr_descent(sz + 1);
        for (int i = 0; i < k; i++) {
            if (album[i] > pr_max) {
                initial++;
                pr_descent[wrap[album[i]] + 1]++;
            }
            pr_max = max(pr_max, album[i]);
        }
        vector<int> values(sz);
        for (int i = 0; i < sz; i++) {
            initial -= pr_descent[i];
            values[i] = initial;
        }
        albums.emplace_back(album);
        album_coords.emplace_back(coords);
        album_values.emplace_back(values);
        maxs.emplace_back(pr_max);
    }
    vector<int> order(n);
    iota(order.begin(), order.end(), 0);
    sort(order.begin(), order.end(), [&](int i, int j) -> bool {
            return maxs[i] < maxs[j];
            });
    SegmentTree st(MAX_C);
    st.update(0, 0);
    for (int i : order) {
        const vector<int> &coords = album_coords[i];
        const vector<int> &values = album_values[i];
        int k = coords.size();
        int ans = st.get(0, MAX_C);
        for (int i = 0; i < k; i++) {
            ans = max(ans, values[i] + st.get(0, coords[i]));
        }
        st.update(maxs[i], ans);
    }
    cout << st.get(0, MAX_C) << endl;
}
