#include "bits/stdc++.h"

using namespace std;
using ll = long long;
using ld = long double;
#define all(x) (x).begin(), (x).end()
#define sz(x) (int)(x).size()

struct Fenwick {
	int n;
	vector<int> t;
	Fenwick(int n = 0) : n(n) {
		t.resize(n + 5, 0);
	}

	void add(int i, int x) {
		for (i++; i < n; i += i & -i)
			t[i] += x;
	}

	int sum(int r) {
		int res = 0;
		for (r++; r; r -= r & -r)
			res += t[r];
		return res;
	}

	int sum(int l, int r) { // [l; r]
		return sum(r) - sum(l - 1);
	}
};


struct segtree {
	int size;
	struct node {
		int mx;
		node(int x = 0) : mx(x) {}
	};
	vector<node> t;
	segtree(int n) {
		size = 1;
		while (size < n)
			size *= 2;
		t.assign(2 * size, 0);
	}

	int get_max(int l, int r, int x, int lx, int rx) {
		if (rx <= l || lx >= r) {
			return 0;
		}
		if (l <= lx && rx <= r) {
			return t[x].mx;
		}

		int mx = (lx + rx) / 2;
		int a = get_max(l, r, 2 * x, lx, mx);
		int b = get_max(l, r, 2 * x + 1, mx, rx);
		return max(a, b);
	}
	int get_max(int l, int r) {
		return get_max(l, r, 1, 0, size);
	}

	void set(int pos, int value, int x, int lx, int rx) {
		if (rx - lx == 1) {
			t[x].mx = value;
			return;
		}

		int mx = (lx + rx) / 2;
		if (pos < mx)
			set(pos, value, 2 * x, lx, mx);
		else
			set(pos, value, 2 * x + 1, mx, rx);
		t[x] = max(t[2 * x].mx, t[2 * x + 1].mx);
	}
	void set(int pos, int value) {
		set(pos, value, 1, 0, size);
	}
};

void solve_case() {
	int n; cin >> n;
	vector<vector<int>> V(n);
	for (auto& v : V) {
		int k; cin >> k;
		v.resize(k);
		for (auto& x : v)
			cin >> x;
	}

	int MAX_A = 2e5 + 100;
	Fenwick f(MAX_A);
	vector<int> past(MAX_A, -1);

	vector<int> dp(MAX_A);

	vector<vector<pair<int, int>>> g(MAX_A);
	for (int k = 0; k < n; k++) {
		auto& a = V[k];
		vector<int> mx(sz(a));
		mx[0] = a[0];
		for (int i = 1; i < sz(a); i++)
			mx[i] = max(mx[i - 1], a[i]);

		vector<int> len(sz(a), 0);
		for (int i = sz(a) - 1; i >= 0; i--) {
			int l = a[i], r = MAX_A + 2;
			while (r - l > 1) {
				int m = (l + r) / 2;
				int cnt = f.sum(a[i] + 1, m);

				if (cnt)
					r = m;
				else
					l = m;
			}

			int res = 0;
			if (r != MAX_A + 2) {
				res = len[past[r]];
			}

			len[i] = res + 1;
			past[a[i]] = i;
			f.add(a[i], 1);
		}

		for (int i = 0; i < sz(a); i++) {
			f.add(a[i], -1);
			past[a[i]] = -1;
		}

		int MX = *max_element(all(a));
		for (int i = 0; i < sz(a); i++) {
			if (!i || mx[i] != mx[i - 1]) {
				g[mx[i]].emplace_back(MX + 1, len[i]);
			}
		}
	}

	segtree st(MAX_A);
	for (int i = 0; i < MAX_A; i++) {
		int have = st.get_max(0, i + 1);
		for (auto& [v, w] : g[i]) {
			int next = have + w;

			int had = st.get_max(v, v + 1);
			if (next > had) {
				st.set(v, next);
			}
		}
	}

	cout << st.get_max(0, MAX_A) << "\n";
}

signed main() {
	cin.tie(0)->sync_with_stdio(0);
#ifdef LOCAL
	(void)freopen("../input.txt", "r", stdin);
#endif

	int T = 1;
	//cin >> T;
	while (T--) {
		solve_case();
	}

	return 0;
}
