#include <iostream>
#include <vector>

using namespace std;
using ll = long long;

vector<vector<ll>> als[200001];

struct SegTree {
    vector<ll> tree;
    ll sz;

    explicit SegTree(ll sz) : sz(sz) {
        tree.resize(sz << 2);
    }

    void set(ll pos, ll val) {
        _set(0, 0, sz - 1, pos, val);
    }

    void _set(ll root, ll l, ll r, ll pos, ll val) {
        if (l == r) {
            tree[root] = max(tree[root], val);
            return;
        }

        ll m = (l + r) >> 1;
        ll lRoot = (root << 1) + 1;
        ll rRoot = (root << 1) + 2;
        if (pos <= m) {
            _set(lRoot, l, m, pos, val);
        } else {
            _set(rRoot, m + 1, r, pos, val);
        }

        tree[root] = max(tree[lRoot], tree[rRoot]);
    }

    ll getMax(ll l, ll r) {
        return _getMax(0, 0, sz - 1, l, r);
    }

    ll _getMax(ll root, ll l, ll r, ll sL, ll sR) {
        if (sL <= l && r <= sR) {
            return tree[root];
        }

        if (l > sR || r < sL) {
            return 0;
        }

        ll m = (l + r) >> 1;
        ll lRoot = (root << 1) + 1;
        ll rRoot = (root << 1) + 2;
        return max(_getMax(lRoot, l, m, sL, sR), _getMax(rRoot, m + 1, r, sL, sR));
    }
};

void resolve() {
    ll n;
    cin >> n;
    for (ll i = 0; i < n; ++i) {
        ll sz;
        cin >> sz;
        vector<ll> cur;
        for (ll j = 0; j < sz; ++j) {
            ll x;
            cin >> x;
            if (!j || cur.back() < x) {
                cur.push_back(x);
            }
        }

        als[cur.back()].push_back(cur);
    }

    SegTree dpTree(200001);
    for (auto &al: als) {
        for (const auto &lst: al) {
            ll sz = ll(lst.size());
            for (ll i = 0; i < sz; ++i) {
                dpTree.set(lst.back(), dpTree.getMax(0, lst[i] - 1) + sz - i);
            }
        }
    }

    cout << dpTree.tree[0] << '\n';
}

int32_t main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    resolve();
    return 0;
}
