//#define _GLIBCXX_DEBUG

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <cassert>
#include <functional>
#include <set>
#include <algorithm>
#include <map>
#include <random>
#include <chrono>

using namespace std;

struct EC{
    int id;
    int64_t p[2];
};


struct STNode{
    uint64_t L, R;
    int64_t maximum;
    STNode* left_child = nullptr;
    STNode* right_child = nullptr;

    STNode(uint64_t _l, uint64_t _r, const vector<EC>& arr, int fgf) : L(_l), R(_r){
        if (L == R){
            maximum = arr[L].p[fgf];
        } else {
            uint64_t m = (L + R) / 2;
            left_child = new STNode(L, m, arr, fgf);
            right_child = new STNode(m + 1, R, arr, fgf);
            maximum = max(left_child->maximum, right_child->maximum);
        }
    }

    int64_t id_of_nearest_to_right_greater_num(int64_t value){
        if (maximum <= value){
            return -1;
        }
        if (L == R){
            return L;
        }
        if (right_child->maximum > value){
            return right_child->id_of_nearest_to_right_greater_num(value);
        }
        return left_child->id_of_nearest_to_right_greater_num(value);
    }

    int64_t get_max(uint64_t A, uint64_t B){
        if (R < A || B < L){
            return -1;
        }
        if (A <= L && R <= B){
            return maximum;
        }
        return max(left_child->get_max(A, B), right_child->get_max(A, B));
    }
};

struct RandShit{
    std::uniform_int_distribution<uint64_t> ri_dist;
    std::default_random_engine r_eng;
    RandShit() : ri_dist(0, -1), r_eng(std::chrono::steady_clock::now().time_since_epoch().count()){}
    uint64_t get(){
        return ri_dist(r_eng);
    }
};

struct KurevoNode;
int64_t kur_get_kc_in_sbt(KurevoNode* node);
struct KurevoNode{
    int64_t key;
    int64_t kc;
    uint64_t priority;

    int64_t kc_in_sbt;

    KurevoNode* left_child = nullptr;
    KurevoNode* right_child = nullptr;

//    KurevoNode(int64_t _key, int64_t _kc, RandShit& rsh) : key(_key), kc(_kc), kc_in_sbt(_kc),
//                priority(rsh.get()){}
    KurevoNode(int64_t _key, int64_t _kc, RandShit& rsh) : key(_key), kc(_kc), kc_in_sbt(_kc),
                                                           priority((rand() << 16) + rand()){}

    void kur_set_left(KurevoNode* new_right){
        left_child = new_right;
        kc_in_sbt = kur_get_kc_in_sbt(left_child) + kc + kur_get_kc_in_sbt(right_child);
    }

    void kur_set_right(KurevoNode* new_left){
        right_child = new_left;
        kc_in_sbt = kur_get_kc_in_sbt(left_child) + kc + kur_get_kc_in_sbt(right_child);
    }
};

int64_t kur_get_kc_in_sbt(KurevoNode* node){
    return node ? node->kc_in_sbt : 0;
}

KurevoNode* kur_merge(KurevoNode* X, KurevoNode* Y){
    if (!X){return Y;}
    if (!Y){return X;}
    if (X->priority > Y->priority){
        X->kur_set_right(kur_merge(X->right_child, Y));
        return X;
    } else {
        Y->kur_set_left(kur_merge(X, Y->left_child));
        return Y;
    }
}

pair<KurevoNode*, KurevoNode*> kur_split(KurevoNode* node, int64_t K, bool eq_to_left){
    if (!node){return {nullptr, nullptr};}
    if (node->key < K){
        auto [L, R] = kur_split(node->right_child, K, eq_to_left);
        node->kur_set_right(L);
        return {node, R};
    } else if (node->key > K){
        auto [L, R] = kur_split(node->left_child, K, eq_to_left);
        node->kur_set_left(R);
        return {L, node};
    }
    if (eq_to_left){
        auto rch = node->right_child;
        node->kur_set_right(nullptr);
        return {node, rch};
    } else {
        auto lch = node->left_child;
        node->kur_set_left(nullptr);
        return {lch, node};
    }
}

void kur_add_val(KurevoNode*& node, int64_t V, RandShit& rsh){
    auto [P_A, P_BC] = kur_split(node, V, false);
    auto [P_B, P_C] = kur_split(P_BC, V, true);
    if (P_B){
        P_B->kc++;
        P_B->kc_in_sbt++;
    } else {
        P_B = new KurevoNode(V, 1, rsh);
    }
    node = kur_merge(P_A, kur_merge(P_B, P_C));
}

KurevoNode* mx_in_t(KurevoNode* node){
    if (!node){
        return nullptr;
    }
    if (!node->right_child){
        return node;
    }
    return mx_in_t(node->right_child);
}

int64_t max_with_limit_orn(KurevoNode*& node, int64_t L){
    auto [P_A, P_B] = kur_split(node, L, true);
    int64_t ans = -1;
    KurevoNode* lor = mx_in_t(P_A);
    if (lor){
        ans = lor->key;
    }
    node = kur_merge(P_A, P_B);
    return ans;
}

void kur_DfsPrint(KurevoNode* node, int l){
    if (l == 0){
        printf("Obkurka\n");
    }
    printf("%s", string(l * 4, ' ').c_str());
    if (l == 10){
        printf("Too deep(\n");
        return;
    }
    if (!node){
        printf("Null\n");
    } else {
        kur_DfsPrint(node->left_child, l + 1);
        printf("%ld %ld\n", node->key, node->kc);
        kur_DfsPrint(node->right_child, l + 1);
    }
}

int main(){
    RandShit randShit;
    uint64_t n;
    cin >> n;
    vector<EC> arr(n);
    for (int i = 0; i < n; i++){
        arr[i].id = i;
        cin >> arr[i].p[0] >> arr[i].p[1];
    }
    vector<EC> sfA = arr;
    sort(sfA.begin(), sfA.end(), [](const EC& X, const EC& Y) -> bool{
        return X.p[0] < Y.p[0];
    });
    STNode searchBinA{0, n - 1, sfA, 1};
    vector<EC> sfB = arr;
    sort(sfB.begin(), sfB.end(), [](const EC& X, const EC& Y) -> bool{
        return X.p[1] < Y.p[1];
    });
    STNode searchAinB{0, n - 1, sfB, 0};
    int64_t infinity = 99999999999999;
    int64_t best_D = infinity;
    auto cfit = [&n, &best_D, &randShit](int sgf, const vector<EC>& sfX, STNode& searchYinX){
        int Agf = 1 - sgf;
        KurevoNode* on_left_Y = nullptr;
        for (int i = 0; i < n; i++){
//            kur_DfsPrint(on_left_Y, 0);
            int64_t sgf_price_mx = sfX[i].p[sgf];
            int64_t mfly = max_with_limit_orn(on_left_Y, sgf_price_mx);
            int64_t mfny = (i == n - 1) ? -1 : searchYinX.get_max(i + 1, n - 1);
            int64_t best_pos_and_must_Y = max(mfly, mfny);
            if (best_pos_and_must_Y >= 0 && best_pos_and_must_Y <= sgf_price_mx){
                best_D = min(best_D, sgf_price_mx - best_pos_and_must_Y);
            }
            kur_add_val(on_left_Y, sfX[i].p[Agf], randShit);
        }
    };
    cfit(0, sfA, searchBinA);
    cfit(1, sfB, searchAinB);
    assert(best_D < infinity);
    cout << best_D << "\n";
}
