#include <iostream>
#include <vector>
#include <algorithm>
#include <unordered_map>
#include <random>
#include <chrono>
#include <unordered_set>

using namespace std;
using pii = pair<int, int>;

struct Node {
    vector<int> a;
    int sz;
    Node (int x) {
        a.push_back(x);
        sz = 1;
    }
    Node() {
        sz = 1;
    }
};

struct Tree {
    vector<Node> t;
    vector<int> orig;
    Tree(vector<int>& a) {
        orig = a;
        t.resize(4 * orig.size());
        build(0, 0, a.size());
    }

    vector<int> merge(vector<int>& a, vector<int>& b) {
        int l = 0;
        int r = 0;
        vector<int> res;
        while (l < a.size() || r < b.size()) {
            if (l == a.size()) {
                while (r < b.size()) {
                    res.push_back(b[r]);
                    ++r;
                }
            } else if (r == b.size()) {
                while (l < a.size()) {
                    res.push_back(a[l]);
                    ++l;
                }
            } else if (a[l] < b[r]) {
                res.push_back(a[l]);
                l++;
            } else {
                res.push_back(b[r]);
                r++;
            }
        }
        return res;
    }

    void build(int v, int l, int r) {
        if (l + 1 == r) {
            t[v] = Node(orig[l]);
            t[v].sz = 1;
        } else {
            int m = (l + r) / 2;
            build(2 * v + 1, l, m);
            build(2 * v + 2, m, r);
            t[v].a = merge(t[2 * v + 1].a, t[2 * v + 2].a);
            t[v].sz = t[2 * v + 1].sz + t[2 * v + 2].sz;
        }
    }

    vector<int> get(int v, int l, int r, int ql, int qr) {
        if (r <= ql || qr <= l) {
            return {};
        }
        if (ql <= l && r <= qr) {
            return t[v].a;
        }
        int m = (l + r) / 2;
        vector<int> f = get(2 * v + 1, l, m, ql, qr);
        vector<int> s = get(2 * v + 2, m, r, ql, qr);
        return merge(f, s);
    }
    vector<int> get(int ql, int qr) {
        return get(0, 0, orig.size(), ql, qr);
    }
};

int find(vector<int>& a, int x) {
    int l = 0;
    int r = a.size();
    int predl = -1;
    int predr = -1;
    while (r - l > 1 ) {
        predl = l;
        predr = r;
        int m1 = (2 * l + r) / 3;
        int m2 = (l + 2 * r) / 3;
        int res1 = abs(x - a[m1]);
        int res2 = abs(x - a[m2]);
        if (res1 <= res2) {
            r = m2;
        } else {
            l = m1;
        }
        if (predl == l && predr == r) break;
        predl = l;
        predr = r;
    }
    if (r == a.size()) {
        return l;
    }
    if (abs(x - a[l]) <= abs(x - a[r])) {
        return l;
    } else {
        return r;
    }
}

int main() {
    int n, x, y;
    cin >> n;
    vector<pii> a;
    for (int i = 0; i < n; ++i) {
        cin >> x >> y;
        a.emplace_back(x, y);
    }
    vector<int> possible;
    for (int i = 0; i < n; ++i) {
        possible.push_back(i);
    }
    sort(a.rbegin(), a.rend());
    vector<int> nums;
    for (pii i : a) {
        nums.push_back(i.second);
    }
    Tree tree(nums);
    unordered_set<int> cannot;
    int ans = 1e9;
    int cur = 0;
    for (int i = 0; i < n ;++i) {
        x = a[i].first;
        if (i == n - 1) {
            ans = min(ans, abs(x - cur));
        }
        else {
            if (i > 0) {
                cur = max(cur, a[i - 1].second);
            }
            vector<int> use = tree.get(i + 1, n);
            int ind = find(use, x);
            ans = min(ans, abs(x - max(cur, use[ind])));
        }
    }
    cout << ans << endl;
    return 0;
}