#include <iostream>
#include <algorithm>
#include <set>
#include <vector>
#include <map>
#include <random>


using namespace std;

mt19937 rnd;

struct Node {
    long key;
    long prior;
    long count;
    Node *l;
    Node *r;

    Node(long _key) {
        key = _key;
        prior = rnd();
        count = 1;
        l = r = nullptr;
    }
};

Node *merge(Node *a, Node *b) {
    if (a == nullptr)
        return b;
    if (b == nullptr)
        return a;
    if (a->prior > b->prior) {
        a->r = merge(a->r, b);
        return a;
    } else {
        b->l = merge(a, b->l);
        return b;
    }
}

pair<Node *, Node *> split(Node *t, long x) {
    if (t == nullptr)
        return {nullptr, nullptr};
    if (t->key >= x) {
        auto [l1, r1] = split(t->l, x);
        t->l = r1;
        return {l1, t};
    } else {
        auto [l1, r1] = split(t->r, x);
        t->r = l1;
        return {t, r1};
    }
}

void erase(Node *&t, long x) {
    auto [l1, r1] = split(t, x);
    auto [l2, r2] = split(r1, x + 1);
    if (l2 != nullptr) {
        --l2->count;
        if (!l2->count)
            l2 = nullptr;
    }
    t = merge(l1, merge(l2, r2));
}

void insert(Node *&t, long x) {
    auto [l1, r1] = split(t, x);
    auto [l2, r2] = split(r1, x + 1);
    if (l2 == nullptr) {
        l2 = new Node(x);
    } else {
        ++l2->count;
    }
    t = merge(l1, merge(l2, r2));
}

long get_ans(Node *&t, long m, long a) {
    auto [l1, r1] = split(t, m + 1);
    auto [l2, r2] = split(r1, a);
    auto curr1 = l2, curr2 = r2;
    while (curr1 != nullptr && curr1->r != nullptr)
        curr1 = curr1->r;
    while (curr2 != nullptr && curr2->l != nullptr)
        curr2 = curr2->l;
    long ans = 1e9 + 7;
    if (curr1 != nullptr)
        ans = min(ans, abs(a - curr1->key));
    if (curr2 != nullptr)
        ans = min(ans, abs(a - curr2->key));
    t = merge(l1, merge(l2, r2));
    return ans;
}


int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    long n;
    cin >> n;
    vector<pair<long, long>> data1(n);
    vector<long> maxx(n);
    for (long i = 0; i < n; ++i) {
        cin >> data1[i].first >> data1[i].second;
    }
    sort(data1.begin(), data1.end());
    maxx.back() = data1.back().second;
    for (long i = n - 2; i >= 0; --i) {
        maxx[i] = max(maxx[i + 1], data1[i].second);
    }
    maxx.push_back(-1);
    long answer = abs(data1[0].first - maxx[1]);
    long k = 1;
    Node *root = new Node(data1[0].second);
    for (long i = 1; i < n; ++i) {
        long curr = data1[i].first;
        while (k < n && data1[k].first == curr) {
            insert(root, data1[k].second);
            ++k;
        }
        erase(root, data1[i].second);
        if (k != n) {
            answer = min(answer, abs(maxx[k] - curr));
        }
        answer = min(answer, get_ans(root, maxx[k], curr));
        insert(root, data1[i].second);
    }
    cout << answer << endl;
    return 0;
}
