// C++ includes used for precompiling -*- C++ -*-

// Copyright (C) 2003-2022 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or (at your option)
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Under Section 7 of GPL version 3, you are granted additional
// permissions described in the GCC Runtime Library Exception, version
// 3.1, as published by the Free Software Foundation.

// You should have received a copy of the GNU General Public License and
// a copy of the GCC Runtime Library Exception along with this program;
// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
// <http://www.gnu.org/licenses/>.

/** @file stdc++.h
 *  This is an implementation file for a precompiled header.
 */

 // 17.4.1.2 Headers

 // C
#ifndef _GLIBCXX_NO_ASSERT
#include <cassert>
#endif
#include <cctype>
#include <cerrno>
#include <cfloat>
#include <ciso646>
#include <climits>
#include <clocale>
#include <cmath>
#include <csetjmp>
#include <csignal>
#include <cstdarg>
#include <cstddef>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>
#include <cwchar>
#include <cwctype>

#if __cplusplus >= 201103L
#include <ccomplex>
#include <cfenv>
#include <cinttypes>
#include <cstdalign>
#include <cstdbool>
#include <cstdint>
#include <ctgmath>
#include <cuchar>
#endif

// C++
#include <algorithm>
#include <bitset>
#include <complex>
#include <deque>
#include <exception>
#include <fstream>
#include <functional>
#include <iomanip>
#include <ios>
#include <iosfwd>
#include <iostream>
#include <istream>
#include <iterator>
#include <limits>
#include <list>
#include <locale>
#include <map>
#include <memory>
#include <new>
#include <numeric>
#include <ostream>
#include <queue>
#include <set>
#include <sstream>
#include <stack>
#include <stdexcept>
#include <streambuf>
#include <string>
#include <typeinfo>
#include <utility>
#include <valarray>
#include <vector>

#if __cplusplus >= 201103L
#include <array>
#include <atomic>
#include <chrono>
#include <codecvt>
#include <condition_variable>
#include <forward_list>
#include <future>
#include <initializer_list>
#include <mutex>
#include <random>
#include <ratio>
#include <regex>
#include <scoped_allocator>
#include <system_error>
#include <thread>
#include <tuple>
#include <typeindex>
#include <type_traits>
#include <unordered_map>
#include <unordered_set>
#endif

#if __cplusplus >= 201402L
#include <shared_mutex>
#endif

#if __cplusplus >= 201703L
#include <any>
#include <charconv>
// #include <execution>
#include <filesystem>
#include <optional>
#include <memory_resource>
#include <string_view>
#include <variant>
#endif

#if __cplusplus >= 202002L
#include <barrier>
#include <bit>
#include <compare>
#include <concepts>
#if __cpp_impl_coroutine
# include <coroutine>
#endif
#include <latch>
#include <numbers>
#include <ranges>
#include <span>
#include <stop_token>
#include <semaphore>
#include <source_location>
#include <syncstream>
#include <version>
#endif

#if __cplusplus > 202002L
#include <expected>
#include <spanstream>
#if __has_include(<stacktrace>)
# include <stacktrace>
#endif
#include <stdatomic.h>
#endif

using namespace std;

#ifdef OG_Matveychick1
bool local = 1;
#else
bool local = 0;
#endif

#define for0(i,n) for(int i=0;i<n;i++)
#define for1(i,n) for(int i=1;i<n;i++)
#define fi first
#define se second
#define con continue
#define ret return
#define rfor(i,n) for (int i=n-1;i>=0;i--)
#define be(x) x.begin()
#define en(x) x.end()
#define all(x) be(x),en(x)
#define pb push_back
#define pob pop_back
#define ins insert
#define bs binary_search
#define lb lower_bound
#define ub upper_bound
#define sz(x) x.size()
#define ba(x) x.back()



typedef long long ll;
typedef vector<ll> vll;
typedef set<ll> sll;
typedef string str;
typedef vector<string> vs;
typedef pair<ll, ll> pll;
typedef vector<pll> vpll;
typedef vector<vll> vvll;
typedef long double ld;
typedef vector<vpll> vvpll;


void solve();

ll rnd(ll l, ll r)
{
	ret rand() % (r - l + 1) + l;
}

ll bin_pow(ll n, ll x, ll mod)
{
	ll r = 1;
	ll um = n;
	while (x)
	{
		if (x & 1)
		{
			r *= um;
			r %= mod;
		}
		x >>= 1;
		um *= um;
		um %= mod;
	}
	ret r;
}

ll mtf(ll a, ll b, ll mod)
{
	ret a* bin_pow(b, mod - 2, mod) % mod;
}

int main()
{
	srand(time(0));
	if (local)
	{
		freopen("input.txt", "r", stdin);
		freopen("output.txt", "w", stdout);
	}
	int t = 1;
	//cin >> t;
	while (t--)
	{
		solve();
	}
}

bool f(pair<pll, ll> a, pair<pll, ll> b)
{
	if (a.fi.first == b.first.fi)
	{
		ret a.first.second < b.first.second;
	}
	ret a.first.first > b.first.first;
}

void solve()
{
	ll n;
	cin >> n;
	vpll a(n);
	for0(i, n)
	{
		cin >> a[i].fi >> a[i].se;
	}
	sort(all(a));
	a.resize(unique(all(a)) - be(a));
	n = sz(a);
	vector<pair<pll, ll>> al(2 * n);
	for0(i, n)
	{
		al[i*2] = { {a[i].second,a[i].fi},2 };
		al[i*2+1] = { {a[i].fi,a[i].se},1 };
	}
	sort(all(al), f);
	multiset<ll> ost1, ost2, l1, l2;
	for0(i, sz(a))
	{
		ost1.insert(a[i].fi);
		ost2.insert(a[i].se);
	}
	ll ans = 1e18;
	for0(i, sz(al))
	{
		if (al[i].second == 2)
		{
			if (l2.find(al[i].first.fi) != en(l2))
			{
				break;
			}
			ans = min(ans, abs(al[i].fi.fi - max((sz(l1) > 0 ? *--en(l1) : 0), (sz(ost1) > 0 ? *--en(ost1) : 0))));
			l1.insert(al[i].first.se);
			ost1.erase(ost1.find(al[i].first.se));
		}
		else
		{
			if (l1.find(al[i].first.fi) != en(l1))
			{
				break;
			}
			ans = min(ans, abs(al[i].fi.fi - max((sz(l2) > 0 ? *--en(l2) : 0), (sz(ost2) > 0 ? *--en(ost2) : 0))));
			l2.insert(al[i].first.se);
			ost2.erase(ost2.find(al[i].first.se));
		}
	}
	cout << ans;
}