#include <iostream>
#include <vector>
#include <unordered_map>
#include <algorithm>
#include <cassert>

using namespace std;

unordered_map<int, int64_t> query_cache;

vector<int> answer;

int64_t desired_sum(int l, int r) {
    return r * 1ll * (r +  1) / 2 - (l - 1) * 1ll * l / 2;
}

int64_t query(int p) {
    if (p == 0) {
        return 0;
    }
    if (query_cache.find(p) == query_cache.end()) {
        cout << "? " << p << endl;
        int64_t x; cin >> x;
        if (x == -1) {
            exit(1);
        }
        query_cache[p] = x;
    }
    return query_cache[p];
}

int cherrypick(int l, int r, int64_t s) {
    assert(r <= 2 * l);
    int64_t delta = desired_sum(l, r) - s;
    if (delta <= r) {
        return delta;
    }
    return -1;
}

void solve_cherrypick(int l, int r, int64_t sl, int64_t sr, vector<int> last_queries) {
    if (l > r) {
        return;
    }
    if (l == r) {
        if (sl == sr) {
            answer.push_back(l);
        }
        return;
    }
    if (sr - sl == desired_sum(l, r)) {
        return;
    }
    while (!last_queries.empty() && last_queries.back() < i) {
        last_queries.pop_back();
    }
    for (int i : last_queries) {
        int cherry = cherrypick(l, i, query(i) - sl);
        if (cherry > 0) {
            answer.push_back(cherry);
            solve_cherrypick(i + 1, r, query(i), sr, last_queries);
            return;
        }
    }
    int lo = (last_queries.empty() ? l - 1 : last_queries[0]);
    int hi = r + 1;
    last_queries = {};
    while (hi - lo > 1) {
        int mid = (lo + hi) / 2;
        int cherry = cherrypick(l, mid, query(mid) - sl);
        if (cherry > 0) {
            answer.push_back(cherry);
        }
        if (cherry >= 0) {
            solve_cherrypick(mid + 1, r, query(mid), sr, last_queries);
            // solve_cherrypick(mid + 1, r, query(mid), sr, {});
            return;
        }
        last_queries.push_back(mid);
        hi = mid;
    }
}

void solve(int l, int r, int64_t sl, int64_t sr) {
    if (l > r) {
        return;
    }
    if (l == r) {
        if (sl == sr) {
            answer.push_back(l);
        }
        return;
    }
    if (sr - sl == desired_sum(l, r)) {
        return;
    }
    int m = (l + r) / 2;
    int64_t sm = query(m - 1);
    solve_cherrypick(m, r, sm, sr, {});
    solve(l, m - 1, sl, sm);
}

int main() {
    int t; cin >> t;
    while (t--) {
        query_cache.clear();
        answer.clear();
        int n, k; cin >> n >> k;
        solve(1, n, 0, query(n));
        sort(answer.begin(), answer.end());
        cout << "! ";
        for (int i : answer) {
            cout << i << ' ';
        }
        cout << endl;
        int result; cin >> result;
        if (result == -1) {
            return 1;
        }
    }
}
