#include <iostream>
#include <vector>
#include <algorithm>
#include <unordered_map>
#include <set>

#if !defined LOCAL && !defined DEBUG
#define USE_FAST_IO
#endif

#ifdef USE_FAST_IO
#define endl '\n'
#endif

using namespace std;

struct Edge {
    int to, weight;
    explicit Edge() = default;
    Edge(int to_, int weight_) : to(to_), weight(weight_) {

    }
};

using Graph = vector<vector<Edge>>;

void solve_line(Graph &graph, vector<int> &earnings, int p) {
    int n = graph.size();
    int64_t ans = 0;
    int64_t cur = p;
    int earning = 0;
    for (int i = 0; i < n - 1; i++) {
        earning = max(earning, earnings[i]);
        int next_weight = graph[i][0].weight;
        int64_t needed = max((int64_t) 0, next_weight - cur);
        int64_t delta = (needed + earning - 1) / earning;
        ans += delta;
        cur += delta * earning;
        cur -= next_weight;
    }
    cout << ans << endl;
}

const int64_t INF_LL = (1LL << 57);

struct Distance {
    int64_t min_pred, cur;
    explicit Distance() {
        min_pred = INF_LL;
        cur = 0;
    }
    Distance(int64_t a, int64_t b) : min_pred(a), cur(b) {

    }
    const bool operator<(const Distance &other) const {
        if (min_pred < other.min_pred) {
            return true;
        }
        if (min_pred > other.min_pred) {
            return false;
        }
        return cur > other.cur;
    }
    const Distance add_weight(int64_t w, int64_t earn) const {
        int64_t needed = max((int64_t) 0, w - cur);
        int64_t delta = (needed + earn - 1) / earn;
        return Distance(min_pred + delta, cur - w + earn * delta);
    }
};

void solve(Graph &graph, vector<int> &earnings, int p) {
    int n = graph.size();
    vector<int> coords;
    for (int i : earnings) {
        coords.push_back(i);
    }
    sort(coords.begin(), coords.end());
    coords.resize(unique(coords.begin(), coords.end()) - coords.begin());
    unordered_map<int, int> wrap;
    int t = coords.size();
    for (int i = 0; i < t; i++) {
        wrap[coords[i]] = i;
    }
    Graph new_graph(n * n);
    for (int v = 0; v < n; v++) {
        for (auto [u, w] : graph[v]) {
            int z = wrap[earnings[u]];
            for (int i = 0; i < t; i++) {
                if (z > i) {
                    new_graph[v * n + i].emplace_back(u * n + z, w);
                } else {
                    new_graph[v * n + i].emplace_back(u * n + i, w);
                }
            }
        }
    }
    set<pair<Distance, int>> vertices;
    int start = wrap[earnings[0]];
    vector<Distance> dists(n * n, Distance());
    dists[start] = Distance(0, p);
    for (int i = 0; i < n * n; i++) {
        vertices.insert({dists[i], i});
    }
    while (!vertices.empty()) {
        auto tmp = *vertices.begin();
        vertices.erase(tmp);
        Distance d = tmp.first;
        int V = tmp.second;
        int v = V / n, earn = coords[V % n];
        for (auto [u, w] : new_graph[V]) {
            Distance new_dist = d.add_weight(w, earn);
            if (new_dist < dists[u]) {
                vertices.erase({dists[u], u});
                dists[u] = new_dist;
                vertices.insert({dists[u], u});
            }
        }
    }
    Distance min_dist = Distance();
    for (int i = 0; i < t; i++) {
        min_dist = min(min_dist, dists[(n - 1) * n + i]);
    }
    int64_t ans = min_dist.min_pred;
    if (ans >= INF_LL) {
        cout << -1 << endl;
    } else {
        cout << ans << endl;
    }
}

int main() {
#ifdef USE_FAST_IO
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int n, m, p, g; cin >> n >> m >> p >> g;
    vector<int> earnings(n);
    for (int &i : earnings) {
        cin >> i;
    }
    Graph graph(n);
    for (int i = 0; i < m; i++) {
        int a, b, w; cin >> a >> b >> w;
        a--; b--;
        graph[a].emplace_back(b, w);
    }
    solve(graph, earnings, p);
}
