use std::{
    collections::HashSet,
    io::{self, prelude::*},
};

const MOD: i64 = 10i64.pow(9) + 7;
const P: i64 = 239;

const fn mul(a: i64, b: i64) -> i64 {
    (a * b) % MOD
}

const fn add(a: i64, b: i64) -> i64 {
    (a + b) % MOD
}

const fn sub(a: i64, b: i64) -> i64 {
    add(add(a, -b), MOD)
}

fn calc(s: &[u8]) -> i64 {
    s.iter().fold(0, |hash, &c| add(mul(hash, P), c as i64))
}

struct FenwickTree {
    s: Vec<usize>,
}

impl FenwickTree {
    fn new(n: usize) -> Self {
        FenwickTree { s: vec![0; n] }
    }

    fn add(&mut self, mut p: usize, val: usize) {
        while p < self.s.len() {
            self.s[p] += val;
            p |= p + 1;
        }
    }

    fn query(&self, mut p: usize) -> usize {
        let mut res = 0;
        while p > 0 {
            res += self.s[p - 1];
            p &= p - 1;
        }
        res
    }
}

fn main() -> io::Result<()> {
    let mut scan = Scanner::new(io::BufReader::new(io::stdin().lock()));
    let mut w = io::BufWriter::new(io::stdout().lock());

    let (n, m): (usize, usize) = (scan.token(), scan.token());
    let t = scan.token::<String>().bytes().collect::<Vec<_>>();
    let s = (0..n)
        .map(|_| scan.token::<String>().bytes().collect::<Vec<_>>())
        .collect::<Vec<_>>();

    // eprintln!("Read");

    let mut lens: Vec<HashSet<i64>> =
        vec![HashSet::new(); s.iter().map(|s| s.len()).max().unwrap() + 1];
    for s in s {
        lens[s.len()].insert(calc(&s));
    }

    let pow = (0..=lens.len())
        .scan(1, |pow, _| {
            let cpow = *pow;
            *pow = mul(*pow, P);
            Some(cpow)
        })
        .collect::<Vec<_>>();

    let mut found = vec![];
    for (len, s) in lens.into_iter().enumerate().skip(1) {
        let mut hash = None;
        for (i, win) in t.windows(len).enumerate() {
            if let Some(hash) = &mut hash {
                *hash = add(mul(*hash, P), win[len - 1] as i64);
            } else {
                hash = Some(calc(win));
            }
            let hash = hash.as_mut().unwrap();
            if s.contains(hash) {
                found.push((i, i + len));
            }
            *hash = sub(*hash, mul(win[0] as i64, pow[win.len() - 1]));
        }
    }

    found.sort_by_key(|&(l, r)| (r, l));

    let mut q = (0..m)
        .map(|_| (scan.token::<usize>() - 1, scan.token::<usize>()))
        .enumerate()
        .collect::<Vec<_>>();
    q.sort_by_key(|&(_, (l, r))| (r, l));

    let mut tree = FenwickTree::new(t.len() + 1);

    let mut ans = vec![0; q.len()];
    let mut ptr = 0;
    for (i, (l, r)) in q {
        while ptr < found.len() && found[ptr].1 <= r {
            tree.add(found[ptr].0, 1);
            ptr += 1;
        }

        ans[i] = ptr - tree.query(l);
    }

    writeln!(
        w,
        "{}",
        ans.into_iter()
            .map(|x| x.to_string())
            .collect::<Vec<_>>()
            .join(" ")
    )
}

struct Scanner<R> {
    reader: R,
    buf: Vec<u8>,
    buf_iter: std::str::SplitAsciiWhitespace<'static>,
}

impl<R: BufRead> Scanner<R> {
    fn new(reader: R) -> Self {
        Self {
            reader,
            buf: vec![],
            buf_iter: "".split_ascii_whitespace(),
        }
    }

    fn token<T: std::str::FromStr>(&mut self) -> T {
        loop {
            if let Some(token) = self.buf_iter.next() {
                return token.parse().ok().expect("Failed parse");
            }
            self.buf.clear();
            self.reader
                .read_until(b'\n', &mut self.buf)
                .expect("Failed read");
            self.buf_iter = unsafe {
                let slice = std::str::from_utf8_unchecked(&self.buf);
                std::mem::transmute(slice.split_ascii_whitespace())
            };
        }
    }
}
