#include <iostream>
#include <iomanip>
#include <algorithm>
#include <vector>

using namespace std;

int MOD = 1e9 + 7;
int K = 179;

vector<int> power;
vector<int> h;

int mul(int a, int b) {
    return int(1ll * a * b % MOD);
}

int add(int a, int b) {
    if (a + b >= MOD) {
        return a + b - MOD;
    } else {
        return a + b;
    }
}

int sub(int a, int b) {
    if (a - b < 0) {
        return a - b + MOD;
    } else {
        return a - b;
    }
}

void getPower(int n) {
    power.assign(n + 1, 1);
    for (int i = 1; i <= n; ++i) {
        power[i] = mul(power[i - 1], K);
    }
}

void fillHash(int n, const string& s) {
    h.assign(n + 1, 0);
    for (int i = 0; i < n; ++i) {
        h[i + 1] = add(mul(h[i], K), s[i] - 'a' + 1);
    }
}

//[l; r)
int getSubstrHash(int l, int r) {
    return sub(h[r], mul(h[l], power[r - l]));
}

int getHash(const string& s) {
    int H = 0;
    for (char i : s) {
        H = add(mul(H, K), i - 'a' + 1);
    }
    return H;
}

int main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
#endif
    int n, m;
    cin >> n >> m;
    string s;
    cin >> s;
    int len = (int) s.size();
    getPower(len);
    fillHash(len, s);
    vector<pair<int, int>> seg;
    for (int i = 0; i < n; ++i) {
        string t;
        cin >> t;
        int q = (int) t.size();
        int H = getHash(t);
        for (int j = 0; j <= len - q; ++j) {
            int substr = getSubstrHash(j, j + q);
            if (substr == H) {
                seg.emplace_back(j, j + q - 1);
            }
        }
    }
    sort(seg.begin(), seg.end());
//    for (auto [i, j] : seg) {
//        cerr << i << ' ' << j << '\n';
//    }
//    cerr << '\n';
    for (int q = 0; q < m; ++q) {
        int l, r;
        cin >> l >> r;
        l--;
        r--;
        pair<int, int> t = {l, l};
        auto it = lower_bound(seg.begin(), seg.end(), t);
        int cnt = 0;
        while (it != seg.end() && it->first <= r) {
            if (it->second <= r) {
                cnt++;
            }
            it++;
        }
        cout << cnt << ' ';
    }
}
