#define _GLIBCXX_DEBUG

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <cassert>
#include <functional>
#include <set>
#include <algorithm>
#include <map>
#include <random>
#include <chrono>

using namespace std;

struct Req{
    int L, R;
    int id;
};

using hash_t = pair<uint64_t, uint64_t>;

struct PrefixHash{
    int n;
    uint64_t MOD = 1073741723;
    vector<uint64_t> p_power;
    vector<uint64_t> pref;

    PrefixHash(const string& str) : n(str.size()){
        p_power.assign(n + 1, 1);
        for (int i = 1; i <= n; i++){
            p_power[i] = (p_power[i - 1] * 137) % MOD;
        }
        pref.assign(n + 1, 0);
        for (int i = 1; i <= n; i++){
            pref[i] = ((pref[i - 1] * 137) % MOD + (uint64_t)str[i - 1]) % MOD;
        }
    }

    hash_t get_hash(uint64_t L, uint64_t R){
        uint64_t S = R - L + 1;
        uint64_t ALL = pref[R + 1];
        uint64_t MINUS = (pref[L] * p_power[S]) % MOD;
        return {(ALL + MOD - MINUS) % MOD, S};
    }

    hash_t get_all(){
        return {pref[n], n};
    }
};

int main(){
    int n, m;
    cin >> n >> m;
    int K;
    string T;
    cin >> T;
    K = T.size();
    vector<string> s(n);
    for (int i = 0; i < n; i++){
        cin >> s[i];
    }
    vector<Req> requests(m);
    for (int i = 0; i < m; i++){
        requests[i].id = i;
        cin >> requests[i].L >> requests[i].R;
        requests[i].L--;
        requests[i].R--;
    }
    sort(requests.begin(), requests.end(), [](const Req& X, const Req& Y){
        int xblock = X.L / 3000;
        int yblock = Y.L / 3000;
        return pair<int, int>(xblock, X.R) < pair<int, int>(yblock, Y.R);
    });
    // Separate solution
    vector<set<int>> ready(n);
    PrefixHash Th(T);
    vector<hash_t> s_hash(n);
    for (int i = 0; i < n; i++){
        PrefixHash aboba(s[i]);
        s_hash[i] = aboba.get_all();
    }
    int64_t cur_L = 0;
    int64_t cur_R = -1;
    auto just_removed_from_left = [&](){
        for (int i = 0; i < n; i++){
            ready[i].erase(cur_L - 1);
        }
    };
    auto just_removed_from_right = [&](){
        for (int i = 0; i < n; i++){
            ready[i].erase(cur_R + 2 - s[i].size());
        }
    };
    auto just_added_left = [&](){
        for (int i = 0; i < n; i++){
            uint64_t size = s[i].size();
            if (size <= cur_R - cur_L + 1){
                if (Th.get_hash(cur_L, cur_L + size - 1) == s_hash[i]){
                    ready[i].insert(cur_L);
                }
            }
        }
    };
    auto just_added_right = [&](){
        for (int i = 0; i < n; i++){
            uint64_t size = s[i].size();
            if (size <= cur_R - cur_L + 1){
                if (Th.get_hash(cur_R - size + 1, cur_R) == s_hash[i]){
                    ready[i].insert(cur_R - size + 1);
                }
            }
        }
    };
    vector<uint64_t> answer(m);
    for (int i = 0; i < m; i++){
        int64_t rL = requests[i].L;
        int64_t rR = requests[i].R;
        while (rL < cur_L){
//            printf("biba\n");
            cur_L--;
            just_added_left();
        }
        while (cur_R < rR){
//            printf("biba\n");
            cur_R++;
            just_added_right();
        }
        while (cur_L < rL){
//            printf("biba\n");
            cur_L++;
            just_removed_from_left();
        }
        while (rR < cur_R){
//            printf("biba\n");
            cur_R--;
            just_removed_from_right();
        }
        uint64_t& ans_cur = answer[requests[i].id];
        ans_cur = 0;
        for (int j = 0; j < n; j++){
            ans_cur += ready[j].size();
        }
    }
    for (int i = 0; i < m; i++){
        printf("%lu ", answer[i]);
    }
    printf("\n");
}
