#include "bits/stdc++.h"
#ifndef NT1E
//#pragma GCC target("tune=native")
//#pragma GCC optimize("O3,unroll-loops")
#endif

using namespace std;
using ll = long long;

template<typename T>
istream& operator>>(istream& in, vector<T> &v) {
    for (T &now : v) {
        in >> now;
    }
    return in;
}

#ifndef NT1E
constexpr int MAXN = 5000006;
#else
constexpr int MAXN = 503;
#endif

namespace nhash {
    constexpr int HASHES = 2;
    constexpr int BASE[HASHES] = {239, 239017}, HASH_MOD[HASHES] = {998244353, 1000000007};
    int PW[HASHES][MAXN];
    void init() {
        for (int hid = 0; hid < HASHES; ++hid) {
            PW[hid][0] = 1;
            for (int i = 1; i < MAXN; ++i) {
                PW[hid][i] = (PW[hid][i - 1] * 1ll * BASE[hid]) % HASH_MOD[hid];
            }
        }
    }

    struct Hash {
        array<int, HASHES> val;
        int size;
        bool operator==(const Hash &other) const {
            for (int i = 0; i < HASHES; ++i) {
                if (val[i] != other.val[i]) {
                    return false;
                }
            }
            return size == other.size;
        }
        void operator+=(char c) {
            for (int i = 0; i < HASHES; ++i) {
                val[i] = (val[i] * 1ll * BASE[i] + c) % HASH_MOD[i];
            }
            ++size;
        }
        Hash operator+(char c) const {
            Hash res = *this;
            res += c;
            return res;
        }
        Hash operator-(const Hash &other) const {
            // assert(ds >= 0);
            Hash res{{}, size - other.size};
            for (int i = 0; i < HASHES; ++i) {
                res.val[i] = ((this->val[i] - other.val[i] * 1ll * PW[i][size - other.size]) % HASH_MOD[i] + HASH_MOD[i]) % HASH_MOD[i];
            }
            return res;
        }
    };

    vector<Hash> build_hash_v(const string &s) {
        vector<Hash> res(s.size() + 1);
        for (int i = 0; i < (int)s.size(); ++i) {
            res[i + 1] = res[i] + s[i];
        }
        return res;
    }

    Hash build_hash(const string &s) {
        Hash res{{}, 0};
        for (int i = 0; i < (int)s.size(); ++i) {
            res += s[i];
        }
        return res;
    }
}

constexpr int K = 420;

vector<int> vtl[MAXN], vtr[MAXN];

int main() {
#ifdef NT1E
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    cin.tie(nullptr)->sync_with_stdio(false);
    nhash::init();
    int n, m;
    cin >> n >> m;
    string t;
    cin >> t;
    vector<nhash::Hash> ht = nhash::build_hash_v(t);
    for (int i = 0; i < n; ++i) {
        string s;
        cin >> s;
        nhash::Hash hs = nhash::build_hash(s);
        for (int tp = 0; tp <= (int)t.size() - (int)s.size(); ++tp) {
            if (ht[tp + s.size()] - ht[tp] == hs) {
                vtl[tp].push_back(s.size());
                vtr[tp + (int)s.size() - 1].push_back(s.size());
            }
        }
    }
    vector<pair<int, int>> queries(m);
    vector<ll> ans(m);
    for (pair<int, int> &now : queries) {
        cin >> now.first >> now.second;
        --now.first;
    }
    vector<int> rng(m);
    iota(rng.begin(), rng.end(), 0);
    ranges::sort(rng, [&](int lhs, int rhs) {
        if (queries[lhs].first / K != queries[rhs].first / K) {
            return queries[lhs].first / K < queries[rhs].first / K;
        }
        return queries[lhs].second < queries[rhs].second;
    });

    for (int p = 0; p <= (int)t.size(); ++p) {
        std::sort(vtr[p].begin(), vtr[p].end());
        std::sort(vtl[p].begin(), vtl[p].end());
    }

    int curl = 0, curr = 0;
    ll curans = 0;

    for (int qid : rng) {
        while (curl > queries[qid].first) {
            --curl;
            curans += upper_bound(vtl[curl].begin(), vtl[curl].end(), curr - curl) - vtl[curl].begin();
        }
        while (curr < queries[qid].second) {
            curans += upper_bound(vtr[curr].begin(), vtr[curr].end(), curr - curl + 1) - vtr[curr].begin();
            ++curr;
        }

        while (curl < queries[qid].first) {
            curans -= upper_bound(vtl[curl].begin(), vtl[curl].end(), curr - curl) - vtl[curl].begin();
            ++curl;
        }
        while (curr > queries[qid].second) {
            --curr;
            curans -= upper_bound(vtr[curr].begin(), vtr[curr].end(), curr - curl + 1) - vtr[curr].begin();
        }
        ans[qid] = curans;
    }

    for (int i = 0; i < m; ++i) {
        cout << ans[i] << ' ';
    }

    return 0;
}