#include <bits/stdc++.h>

template <typename T>
struct in {
    T x;

    in() {
        std::cin >> x;
    }

    in(const T &_x) : x(_x) {
    }

    operator T&() {
        return x;
    }

    operator const T&() const {
        return x;
    }
};

using namespace std;

struct solve {
    in<int> n, m;
    string t;
    vector<string> s;
    vector<pair<int, int>> q;

    solve() : s(n), q(m) {
        cin >> t;
        for (string &x : s) {
            cin >> x;
        }
        for (auto &[a, b] : q) {
            cin >> a >> b;
            --a;
        }
        solve_segdp();
    }

    const long long MOD1 = 1000000007;
    const long long MOD2 = 999999757;
    const long long P = 29;

    vector<long long> t_hash1, t_hash2;
    vector<long long> powmod1, powmod2;

    void calc_thash1() {
        long long cur = 0;
        t_hash1.push_back(cur);
        powmod1.push_back(1);
        for (char c : t) {
            cur = (cur * P + (c - 'a' + 1)) % MOD1;
            t_hash1.push_back(cur);
            powmod1.push_back(powmod1.back() * P % MOD1);
        }
    }

    void calc_thash2() {
        long long cur = 0;
        t_hash2.push_back(cur);
        powmod2.push_back(1);
        for (char c : t) {
            cur = (cur * P + (c - 'a' + 1)) % MOD2;
            t_hash2.push_back(cur);
            powmod2.push_back(powmod2.back() * P % MOD2);
        }
    }

    long long extract_hash1(int l, int r) {
        return (t_hash1[r] + MOD1 - t_hash1[l] * powmod1[r - l] % MOD1) % MOD1;
    }

    long long extract_hash2(int l, int r) {
        return (t_hash2[r] + MOD2 - t_hash2[l] * powmod2[r - l] % MOD2) % MOD2;
    }

    void solve_segdp() {
        calc_thash1();
        calc_thash2();


        auto calc_hash = [&](const string &x, long long MOD) {
            long long res = 0;
            for (char c : x) {
                res = (res * P + (c - 'a' + 1)) % MOD;
            }
            return res;
        };

        if (t.size() <= 5000) {
            unordered_set<long long> hashes1, hashes2;

            for (const string &x : s) {
                hashes1.insert(calc_hash(x, MOD1));
                hashes2.insert(calc_hash(x, MOD2));
            }

            vector<vector<long long>> dp(t.size() + 1);
            for (int len = 0; len <= t.size(); ++len) {
                dp[len].resize(t.size() - len + 1);
            }
            for (int len = 1; len <= t.size(); ++len) {
                for (int start = 0; start + len <= t.size(); ++start) {
                    dp[len][start] = dp[len - 1][start] + dp[len - 1][start + 1];
                    if (len >= 2) {
                        dp[len][start] -= dp[len - 2][start + 1];
                    }
                    if (hashes1.find(extract_hash1(start, start + len)) != hashes1.end() &&
                        hashes2.find(extract_hash2(start, start + len)) != hashes2.end()) {
                        ++dp[len][start];
                    }
                }
            }

            for (auto [l, r]: q) {
                cout << dp[r - l][l] << " ";
            }
            cout << "\n";
        } else {
            vector<vector<int>> match(n, vector<int>(t.size()));
            vector<vector<int>> match_prefsum(n, vector<int>(t.size() + 1));
            for (int i = 0; i < n; ++i) {
                long long h1 = calc_hash(s[i], MOD1), h2 = calc_hash(s[i], MOD2);
                for (int start = 0; start + (int) s[i].size() <= (int) t.size(); ++start) {
                    if (extract_hash1(start, start + (int) s[i].size()) == h1 &&
                    extract_hash2(start, start + (int) (s[i].size())) == h2) {
                        match[i][start] = 1;
                    }
                }
                for (int x = 0; x < (int) t.size(); ++x) {
                    match_prefsum[i][x + 1] = match_prefsum[i][x] + match[i][x];
                }
            }
            for (auto [l, r] : q) {
                long long res = 0;
                for (int i = 0; i < n; ++i) {
                    if ((int) s[i].size() > r - l) {
                        continue;
                    }
                    res += match_prefsum[i][r - s[i].size() + 1] - match_prefsum[i][l];
                }
                cout << res << " ";
            }
            cout << "\n";
        }
    }
};

int main() {
    ios::sync_with_stdio(false);
#ifndef LOCAL
    cout.tie(nullptr);
#endif

    in<int> t = 1;
    while (t--) {
        solve();
    }
}
