#include <bits/stdc++.h>

using namespace std;
typedef long long ll;
typedef long double ld;

struct Node {
    char pch;
    int parent, suff, deep_suff, count, length;
    vector <int> children;

    Node (char pch, int parent, int length): count(0), length(length), children(26, -1), parent(parent), deep_suff(-1), suff(-1), pch(pch) {}

    bool can_go(char c) {
        return children[c - 'a'] != -1;
    }

    int& get(char c) {
        return children[c - 'a'];
    }
};

vector <Node> nodes;

int new_node(char pch, int parent) {
    int node_id = nodes.size();
    nodes.push_back(Node(pch, parent, parent == -1 ? 0 : nodes[parent].length + 1));
    return node_id;
}

int ROOT = new_node('/', -1);

int go(int node_id, char c) {
    if (!nodes[node_id].can_go(c))
        nodes[node_id].get(c) = new_node(c, node_id);

    return nodes[node_id].get(c);
}

void add_string(string &s) {
    int node_id = ROOT;

    for (char c: s) {
        node_id = go(node_id, c);
    }

    nodes[node_id].count++;
}

int get_suff(int node_id) {
    if (node_id == ROOT || nodes[node_id].parent == ROOT)
        return ROOT;

    if (nodes[node_id].suff == -1) {
        int t = get_suff(nodes[node_id].parent);
        while (!nodes[t].can_go(nodes[node_id].pch) && t != ROOT)
            t = get_suff(t);

        if (nodes[t].can_go(nodes[node_id].pch))
            nodes[node_id].suff = go(t, nodes[node_id].pch);
        else 
            nodes[node_id].suff = ROOT;
    }

    return nodes[node_id].suff;
}

string get_node_string(int node_id) {
    if (node_id == ROOT)
        return "";

    return get_node_string(nodes[node_id].parent) + nodes[node_id].pch;
}

struct Fenwick {
    int n;
    vector <int> tree;

    Fenwick (int n): n(n), tree(n) {}

    void add(int r, int dt) {
        for (; r < n; r |= (r + 1))
            tree[r] += dt;
    }

    int sum(int r) {
        int result = 0;
        for (; r >= 0; r = (r & (r + 1)) - 1)
            result += tree[r];
        return result;
    }
};

struct Event {
    int query_id, x, type;

    Event (int query_id, int x, int type): query_id(query_id), x(x), type(type) {}
};

bool operator<(Event lhs, Event rhs) {
    return lhs.x < rhs.x || (lhs.x == rhs.x && lhs.type < rhs.type);
}

void solve() {
    int n, m;
    cin >> n >> m;

    string t;
    cin >> t;

    string s;
    for (int i = 0; i < n; i++) {
        cin >> s;
        add_string(s);
    }

    int node_id = ROOT;

    int len = t.length();
    vector <vector <int>> nuggets(len);
    for (int i = 0; i < len; i++) {
        while (!nodes[node_id].can_go(t[i]) && node_id != ROOT) {
            node_id = get_suff(node_id);
        }

        if (nodes[node_id].can_go(t[i])) {
            node_id = nodes[node_id].get(t[i]);
        }

        int ni = node_id;
        while (ni != ROOT) {
            if (nodes[ni].count > 0)
                nuggets[i - nodes[ni].length + 1].push_back(i);

            ni = get_suff(ni);
        }
    }

    vector <Event> events;

    int l, r;
    vector <int> query_rs(m);
    for (int i = 0; i < m; i++) {
        cin >> l >> r;
        l--, r--;

        query_rs[i] = r;

        events.emplace_back(i, l, 0);
        events.emplace_back(i, r, 1);
    }

    sort(events.begin(), events.end());

    int i = 0;
    Fenwick tree(len);
    vector <int> answers(m), prev(m);
    for (auto &event: events) {
        if (event.type == 0) {
            while (i < event.x)
                for (int x: nuggets[i++])
                    tree.add(x, 1);

            prev[event.query_id] = tree.sum(query_rs[event.query_id]);
        } else {
            while (i <= event.x)
                for (int x: nuggets[i++])
                    tree.add(x, 1);

            answers[event.query_id] = tree.sum(query_rs[event.query_id]) - prev[event.query_id];
        }
    }
    
    for (int x: answers)
        cout << x << ' ';
    cout << '\n';
}

signed main() {
    cin.tie(NULL);
    ios_base::sync_with_stdio(false);

    solve();
}
