#include <iostream>
#include <vector>
#include <map>

#if !defined LOCAL && !defined DEBUG
#define USE_FAST_IO
#endif

#ifdef USE_FAST_IO
#define endl '\n'
#endif

using namespace std;

const int NULL_NODE = 0;

struct SuffixAutomaton {
    struct Node {
        map<char, int> go;
        int link;

        int length;
        int terminal_cnt;
        int sum_terminal_cnt;

        Node() {
            link = NULL_NODE;
            length = 0;
            terminal_cnt = 0;
            sum_terminal_cnt = -1;
        }
    };

    using vertex_t = int;

    vector<Node> nodes;
    vertex_t root, last;

    vertex_t new_node() {
        nodes.push_back(Node());
        return (int) nodes.size() - 1;
    }

    void add_char(char c) {
        if (nodes[last].go[c] != NULL_NODE) {
            vertex_t q = nodes[last].go[c];
            if (nodes[last].length + 1 == nodes[q].length) {
                last = q;
            } else {
                vertex_t clone = new_node();
                nodes[clone] = nodes[q];
                nodes[q].link = clone;
                nodes[clone].terminal_cnt = 0;
                nodes[clone].length = nodes[last].length + 1;
                while (last != NULL_NODE && nodes[last].go[c] == q) {
                    nodes[last].go[c] = clone;
                    last = nodes[last].link;
                }
                last = clone;
            }
            return;
        }
        vertex_t new_last = new_node();
        nodes[new_last].length = nodes[last].length + 1;
        vertex_t cur = last;
        while (cur != NULL_NODE && nodes[cur].go[c] == NULL_NODE) {
            nodes[cur].go[c] = new_last;
            cur = nodes[cur].link;
        }
        if (cur == NULL_NODE) {
            nodes[new_last].link = root;
            last = new_last;
            return;
        }
        vertex_t p = cur;
        vertex_t q = nodes[p].go[c];
        if (nodes[p].length + 1 == nodes[q].length) {
            nodes[new_last].link = q;
            last = new_last;
            return;
        }
        vertex_t clone = new_node();
        nodes[clone] = nodes[q];
        nodes[q].link = clone;
        nodes[clone].length = nodes[p].length + 1;
        nodes[clone].terminal_cnt = 0;
        while (p != NULL_NODE && nodes[p].go[c] == q) {
            nodes[p].go[c] = clone;
            p = nodes[p].link;
        }
        nodes[new_last].link = clone;
        last = new_last;
    }

    int get_sum_terminal_cnt(vertex_t v) {
        if (v == NULL_NODE) return 0;
        if (nodes[v].sum_terminal_cnt == -1) {
            nodes[v].sum_terminal_cnt = nodes[v].terminal_cnt + get_sum_terminal_cnt(nodes[v].link);
        }
        return nodes[v].sum_terminal_cnt;
    }

    int get_shortest(vertex_t v) {
        if (nodes[v].link == NULL_NODE) {
            return 0;
        }
        return nodes[nodes[v].link].length;
    }

    SuffixAutomaton() {
        nodes = {Node()};
        root = last = new_node();
    }
};

using vertex_t = SuffixAutomaton::vertex_t;

int main() {
#ifdef USE_FAST_IO
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int n, m; cin >> n >> m;
    SuffixAutomaton sa = SuffixAutomaton();
    string t; cin >> t;
    vector<int> t_prefixes;
    // sa.last = sa.root;
    // for (char c : t) {
    //     sa.add_char(c);
    //     t_prefixes.push_back(sa.last);
    // }
    
    for (int i = 0; i < n; i++) {
        string s; cin >> s;
        sa.last = sa.root;
        for (char c : s) {
            sa.add_char(c);
        }
        sa.nodes[sa.last].terminal_cnt++;
    }
    n = t.length();

    vector<vector<int>> pre_answers(n, vector<int>(n));
    for (int i = 0; i < n; i++) {
        vertex_t cur = sa.root;
        for (int j = i; j < n; j++) {
            char c = t[j];
            cur = sa.nodes[cur].go[c];
            if (cur == NULL_NODE) {
                break;
            }
            pre_answers[i][j] += sa.nodes[cur].terminal_cnt * ((j - i + 1) == sa.nodes[cur].length);
        }
    }
    vector<vector<int>> pref_sums(n + 1, vector<int>(n + 1));
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
            pref_sums[i + 1][j + 1] = pref_sums[i + 1][j] + pref_sums[i][j + 1] - pref_sums[i][j] + pre_answers[i][j];
        }
    }
    while (m--) {
        int l, r; cin >> l >> r; l--; r--;
        int ans = pref_sums[n][r + 1] - pref_sums[l][r + 1];
        cout << ans << ' ';
    }
    cout << endl;
}

