#include "bits/stdc++.h"
#include <array>
#include <chrono>
#include <random>

using namespace std;
using ll = long long;
#define all(x) (x).begin(), (x).end()
#define sz(x) (int)(x).size()

#define int ll

const int MOD1 = 1e9 + 7;
const int MOD2 = 998244353;
const int P1 = 8918232, P2 = 282303;

inline int binpow1(int a, int n) {
	int res = 1;
	while (n) {
		if (n & 1)
			res = ((ll)res * a) % MOD1;
		a = ((ll)a * a) % MOD1;
		n >>= 1;
	}
	return res;
}
inline int binpow2(int a, int n) {
	int res = 1;
	while (n) {
		if (n & 1)
			res = ((ll)res * a) % MOD2;
		a = ((ll)a * a) % MOD2;
		n >>= 1;
	}
	return res;
}

struct Fenwick {
	int n;
	vector<int> t;
	Fenwick(int n = 0) : n(n) {
		t.resize(n + 5, 0);
	}

	void add(int i, int x) {
		for (i++; i < n; i += i & -i)
			t[i] += x;
	}

	int sum(int r) {
		int res = 0;
		for (r++; r; r -= r & -r)
			res += t[r];
		return res;
	}

	int sum(int l, int r) { // [l; r]
		return sum(r) - sum(l - 1);
	}
};

struct query {
	int l, r, x;
};

void solve_case() {
	auto start = clock();
	int n, m; cin >> n >> m;
	string s; cin >> s;

	vector<int> pw1(sz(s) + 1), pw2(sz(s) + 1), ipw1(sz(s) + 1), ipw2(sz(s) + 1);
	pw1[0] = pw2[0] = 1;
	for (int i = 1; i < sz(pw1); i++) {
		pw1[i] = ((ll)pw1[i - 1] * P1) % MOD1;
		pw2[i] = ((ll)pw2[i - 1] * P2) % MOD2;
	}

	ipw1.back() = binpow1(pw1.back(), MOD1 - 2);
	ipw2.back() = binpow2(pw2.back(), MOD2 - 2);

	for (int i = sz(ipw1) - 2; i > 0; i--) {
		ipw1[i] = ((ll)P1 * (ll)ipw1[i + 1]) % MOD1;
		ipw2[i] = ((ll)P2 * (ll)ipw2[i + 1]) % MOD2;
	}
	ipw1[0] = ipw2[0] = 1;

	vector<vector<pair<int, int>>> strings(sz(s) + 1);

	for (int i = 0; i < n; i++) {
		string t; cin >> t;
		if (sz(t) > sz(s)) continue;
		pair<int, int> h = { 0, 0 };
		for (int j = 0; j < sz(t); j++) {
			h.first = (h.first + ((ll)pw1[j] * t[j])) % MOD1;
			h.second = (h.second + ((ll)pw2[j] * t[j])) % MOD2;
		}

		strings[sz(t)].push_back(h);
	}

	for (auto& v : strings) {
		sort(all(v));
		v.erase(unique(all(v)), v.end());
		v.shrink_to_fit();
	}


	vector<int> h1(sz(s) + 1, 0), h2(sz(s) + 1, 0);
	for (int i = 0; i < sz(s); i++) {
		h1[i + 1] = (h1[i] + ((ll)pw1[i] * s[i])) % MOD1;
		h2[i + 1] = (h2[i] + ((ll)pw2[i] * s[i])) % MOD2;
	}

	vector<array<int, 4>> events;
	// { pos, type, x, idx }

	for (int len = 1; len <= sz(s); len++) {
		auto& hashes = strings[len];
		if (hashes.empty())
			continue;

		for (int i = 0; i + len <= sz(s); i++) {
			int r = i + len - 1;
			pair<int, int> h = {
				((((h1[r + 1] - h1[i]) % MOD1 + MOD1) % MOD1) * (ll)ipw1[i]) % MOD1,
				((((h2[r + 1] - h2[i]) % MOD2 + MOD2) % MOD2) * (ll)ipw2[i]) % MOD2
			};

			auto it = lower_bound(all(hashes), h);
			if (it != hashes.end() && *it == h) {
				events.push_back({ i, 1, r, -1 });
			}
		}

		hashes.clear();
		hashes = {};
	}

	for (int i = 0; i < m; i++) {
		int l, r; cin >> l >> r;
		l--; r--;

		events.push_back({ l, 0, r, i });
		events.push_back({ r, 2, r, i });
	}

	vector<int> res(m, 0);
	sort(all(events));
	events.shrink_to_fit();

	while ((clock() - start) / (double)CLOCKS_PER_SEC <= 1.5) continue;
	Fenwick f(sz(s) + 1000);
	for (auto& [pos, type, value, idx] : events) {
		if (type == 1) {
			f.add(value, 1);
		}
		else {
			res[idx] += (f.sum(value) * (type == 2 ? 1 : -1));
		}
	}

	for (int& x : res)
		cout << x << " ";
	cout << '\n';
}

signed main() {
	cin.tie(0)->sync_with_stdio(0);
#ifdef LOCAL
	(void)freopen("../input.txt", "r", stdin);
#endif

	int T = 1;
	//cin >> T;
	while (T--) {
		solve_case();
	}

	return 0;
}