#include <iostream>
#include <vector>
#include <algorithm>
#include <string>
#include <map>
#include <set>
#include <math.h>
#include <climits>

using namespace std;

#define ll long long
#define lll __int128
#define ld long double
#define lld __float128
#define endl '\n'
#define fast ios_base::sync_with_stdio(0); cin.tie(0); cout.tie(0);

//a.x * b.y - a.y * b.x

vector<int> used(10, -1), mt(20, -1);
vector<vector<int>> l(10);
int timer;

bool kuhn(int v) {
    if (used[v] == timer) return 0;
    used[v] = timer;
    for (int i = 0; i < l[v].size(); ++i) {
        int u = l[v][i];
        if (used[mt[u]] == -1 || (used[mt[u]] != timer && kuhn(mt[u]))) {
            mt[u] = v;
            return 1;
        }
    }
    return 0;
}

void start_kuhn() {
    int n = 1;
    timer = 0;
    int answer = 0;
    for (int i = 0; i < n; ++i) {
        if (kuhn(i)) answer++;
        timer++;
    }
}

struct point {
    int x, y;
    point(int _x,int _y) {
        x = _x;
        y =_y;
    }
    point() {}
};

point operator- (point a, point b) {
    return point(a.x - b.x, a.y - b.y);
}

ll operator* (point a, point b) {
    return a.x*b.y - b.x*a.y;
}

ll operator% (point a, point b) {
    return a.x * b.x + b.y * a.y;
}

bool comp(point a, point b, point reference) {
    a = a - reference;
    b = b - reference;
    ll mul = a * b;
    return mul == 0 ? a%a < b%b : mul > 0;
}

bool removeit(point a, point b, point c) {
    point ab = a - b;
    point bc = b - c;
    point ac = a - c;
    ll result = ab * bc;
    return result == 0 ? ab%ab < ac%ac : result < 0;
}

vector<point> convexHull(vector<point> ps) {
    sort(ps.rbegin(), ps.rend(), [](point a, point b) {
        return a.x == b.x ? a.y < b.y : a.x < b.x;
    });
    ps.resize(unique(ps.begin(), ps.end(), [](point a, point b) {
        return a.x == b.x && a.y == b.y;
    }) - ps.begin());
    point reference = ps.back();
    ps.pop_back();
    reverse(ps.begin(), ps.end());
    sort(ps.begin(), ps.end(), [&](point a, point b) {
        return comp(a, b, reference);
    });
    vector<point> hull{ reference };
    for (int i = 0; i < ps.size(); ++i) {
        while (hull.size() > 1 && removeit(ps[i], hull.back(), hull[hull.size()-2])) {
            hull.pop_back();
        }
        hull.push_back(ps[i]);
    }
    return hull;
}

int gcd(int a, int b) {
    if (a == 0) return b;
    return gcd(b % a, a);
}

struct Treap {
    Treap* left = nullptr;
    Treap* right = nullptr;
    ll height, val;

    Treap(ll newy) {
        val = newy;
        height = rand();
    }
};

Treap* merge(Treap* a, Treap* b) {
    if (a == nullptr) return b;
    if (b == nullptr) return a;
    if (a->height < b->height) {
        a->right = merge(a->right, b);
        return a;
    }
    else {
        b->left = merge(a, b->left);
        return b;
    }
}

pair<Treap*, Treap*> split(Treap* t, int val) {
    if (t == nullptr) return { nullptr, nullptr };
    if (t->val < val) {
        auto sp = split(t->right, val);
        t->right = sp.first;
        return { t, sp.second };
    }
    else {
        auto sp = split(t->left, val);
        t->left = sp.second;
        return { sp.first, t };
    }
}

struct vertex {
    vector<pair<int, int>> sorted;
    vector<ll> prefix;
    vertex(vector<pair<int, int>> nums) {
        sorted = nums;
        prefix.resize(nums.size()+1, 0);
        for (int i = 1; i < nums.size() + 1; ++i) {
            prefix[i] = prefix[i - 1] + sorted[i - 1].second;
        }
    }
    ll lb(int right) {
        int l = -1, r = sorted.size();
        while (r - l > 1) {
            int mid = (r + l) / 2;
            (sorted[mid].first < right ? l : r) = mid;
        }
        return prefix[l + 1];
    }
    vertex() {}
    vertex operator+ (vertex w) {
        vector<pair<int, int>> merged;
        merge(sorted.begin(), sorted.end(), w.sorted.begin(), w.sorted.end(), back_inserter(merged));
        return vertex(merged);
    }
};

struct tree {
    vector<vector<pair<int, int>>> nums;
    vector<vertex> simple;

    tree(vector<vector<pair<int, int>>> a) {
        nums = a;
        simple.resize(a.size() * 4);
        build(1, 0, a.size());
    }

    void build(int v, int l, int r) {
        if (r - l == 1) {
            simple[v] = vertex(nums[l]);
        }
        else {
            int mid = (r + l) / 2;
            build(2 * v, l, mid);
            build(2 * v + 1, mid, r);
            simple[v] = simple[2 * v] + simple[2 * v + 1];
        }
    }

    ll answer(int v, int l, int r, int lq, int rq) {
        if (l >= rq || r <= lq) {
            return 0;
        }
        else if (l >= lq && r <= rq) {
            return simple[v].lb(rq);
        }
        else {
            int mid = (l + r) / 2;
            return answer(2 * v, l, mid, lq, rq) + answer(2 * v + 1, mid, r, lq, rq);
        }
    }
};

struct letter {
    vector<int> ways;
    int terminal = 0;
    letter() {
        ways.resize(26, -1);
    }
};

struct bor {
    vector<letter> lts;

    bor() {
        lts.push_back(letter());
    }

    void add(string what) {
        int searchIndex = 0;
        for (int i = 0; i < what.size(); ++i) {
            if (lts[searchIndex].ways[what[i] - 'a'] != -1) {
                searchIndex = lts[searchIndex].ways[what[i] - 'a'];
            }
            else {
                lts[searchIndex].ways[what[i] - 'a'] = lts.size();
                lts.push_back(letter());
                searchIndex = lts[searchIndex].ways[what[i] - 'a'];
            }
        }
        lts[searchIndex].terminal++;
    }

    vector<pair<int, int>> endIndexes(int start, string& in) {
        int searchIndex = 0;
        vector<pair<int, int>> answer; // pos and count
        for (int i = start; i < in.size(); ++i) {
            if (lts[searchIndex].ways[in[i] - 'a'] != -1) {
                searchIndex = lts[searchIndex].ways[in[i] - 'a'];
                if (lts[searchIndex].terminal != 0) {
                    answer.push_back({ i, lts[searchIndex].terminal });
                }
            }
            else {
                break;
            }
        }
        return answer;
    }
};

void solve() {
    int n, m;
    cin >> n >> m;
    string main;
    cin >> main;
    bor b;
    vector<vector<pair<int, int>>> finded(main.size());
    for (int i = 0; i < n; ++i) {
        string newy;
        cin >> newy;
        b.add(newy);
    }
    for (int i = 0; i < main.size(); ++i) {
        finded[i] = b.endIndexes(i, main);
        //cout << "position " << i << endl;
        //for (auto p : finded[i]) {
            //cout << p.first << "-" << p.second << " ";
        //}
        //cout << endl;
    }
    //cout << endl;
    tree t(finded);
    for (int i = 0; i < m; ++i) {
        int l, r;
        cin >> l >> r;
        ll answer = t.answer(1, 0, main.size(), l-1, r);
        cout << answer << endl;
    }
}

int main() {
    fast;
    srand((int)time(nullptr));
    int t = 1;
    //cin >> t;
    while (t--) solve();
    return 0;
}