#include <algorithm>
#include <array>
#include <iostream>
#include <string>
#include <vector>

using namespace std;

namespace Aho {
    struct Node {
        array<int, 26> go;
        int suf;
        int p;
        int c;
        int cntterm;
        Node() {
            suf = 0;
            p = -1;
            c = -1;
            go.fill(-1);
            cntterm = 0;
        }
    };

    vector<Node> trie;

    void init() {
        trie.emplace_back();
        trie[0].suf = -1;
    }

    void add(const string& s) {
        int cv = 0;
        for (auto ch : s) {
            int el = ch - 'a';
            if (trie[cv].go[el] == -1) {
                trie[cv].go[el] = (int)trie.size();
                trie.emplace_back();
                trie.back().p = cv;
                trie.back().c = el;
            }
            cv = trie[cv].go[el];
        }
        trie[cv].cntterm++;
    }

    void bfs() {
        vector<int> q;
        q.push_back(0);
        for (int iter = 0; iter < (int)q.size(); iter++) {
            int cv = q[iter];
            for (int i = 0; i < 26; i++) {
                if (trie[cv].go[i] != -1) {
                    q.push_back(trie[cv].go[i]);
                }
            }
            if (cv != 0) {
                int cs = trie[trie[cv].p].suf;
                if (cs == -1) {
                    trie[cv].suf = 0;
                } else {
                    trie[cv].suf = trie[cs].go[trie[cv].c];
                }
                for (int i = 0; i < 26; i++) {
                    if (trie[cv].go[i] == -1) {
                        trie[cv].go[i] = trie[trie[cv].suf].go[i];
                    }
                }
                trie[cv].cntterm += trie[trie[cv].suf].cntterm;
            } else {
                for (int i = 0; i < 26; i++) {
                    if (trie[0].go[i] == -1) {
                        trie[0].go[i] = 0;
                    }
                }
            }
        }
    }

    int curv = 0;

    void getch(char c) {
        curv = trie[curv].go[c - 'a'];
    }

    int countterm() {
        return trie[curv].cntterm;
    }
} // namespace Aho

int main() {
    int n, m;
    cin >> n >> m;
    string t;
    cin >> t;
    Aho::init();
    for (int i = 0; i < n; i++) {
        string cur;
        cin >> cur;
        Aho::add(cur);
    }

    Aho::bfs();

    for (int qr = 0; qr < m; qr++) {
        int l, r;
        cin >> l >> r;
        l--;
        Aho::curv = 0;
        int res = 0;
        for (int i = l; i < r; i++) {
            Aho::getch(t[i]);
            res += Aho::countterm();
        }
        cout << res << " ";
    }
    return 0;
}
