#include <algorithm>
#include <array>
#include <cstdio>
#include <map>
#include <string>
#include <vector>

using namespace std;

namespace Aho {
    struct Node {
        array<int, 26> go;
        int suf;
        int p;
        int c;
        int cntterm;
        int len;
        Node() {
            suf = 0;
            p = -1;
            c = -1;
            go.fill(-1);
            cntterm = 0;
            len = 0;
        }
    };

    vector<Node> trie;

    void init() {
        trie.emplace_back();
        trie[0].suf = -1;
    }

    void add(const string& s) {
        int cv = 0;
        for (auto ch : s) {
            int el = ch - 'a';
            if (trie[cv].go[el] == -1) {
                trie[cv].go[el] = (int)trie.size();
                trie.emplace_back();
                trie.back().p = cv;
                trie.back().c = el;
                trie.back().len = trie[cv].len + 1;
            }
            cv = trie[cv].go[el];
        }
        trie[cv].cntterm++;
    }

    void bfs() {
        vector<int> q;
        q.push_back(0);
        for (int iter = 0; iter < (int)q.size(); iter++) {
            int cv = q[iter];
            for (int i = 0; i < 26; i++) {
                if (trie[cv].go[i] != -1) {
                    q.push_back(trie[cv].go[i]);
                }
            }
            if (cv != 0) {
                int cs = trie[trie[cv].p].suf;
                if (cs == -1) {
                    trie[cv].suf = 0;
                } else {
                    trie[cv].suf = trie[cs].go[trie[cv].c];
                }
                for (int i = 0; i < 26; i++) {
                    if (trie[cv].go[i] == -1) {
                        trie[cv].go[i] = trie[trie[cv].suf].go[i];
                    }
                }
                // trie[cv].cntterm += trie[trie[cv].suf].cntterm;
            } else {
                for (int i = 0; i < 26; i++) {
                    if (trie[0].go[i] == -1) {
                        trie[0].go[i] = 0;
                    }
                }
            }
        }
    }

    int curv = 0;

    void getch(char c) {
        curv = trie[curv].go[c - 'a'];
    }

    int countterm() {
        return trie[curv].cntterm;
    }
} // namespace Aho

constexpr int tsize = 524288;

map<int, int> tree[2 * tsize];

void addy(map<int, int>& ctr, int i, int x) {
    while (i < tsize) {
        ctr[i] += x;
        i |= (i + 1);
    }
}

void add(int v, int vl, int vr, int i, int j, int x) {
    addy(tree[v], j, x);
    if (vr - vl == 1) {
        return;
    }
    if (i < (vl + vr) / 2) {
        add(2 * v + 1, vl, (vl + vr) / 2, i, j, x);
    } else {
        add(2 * v + 2, (vl + vr) / 2, vr, i, j, x);
    }
}

int getprefy(map<int, int>& ctr, int i) {
    int res = 0;
    while (i >= 0) {
        if (ctr.find(i) != ctr.end()) {
            res += ctr[i];
        }
        i = (i & (i + 1)) - 1;
    }
    return res;
}

int getsumy(map<int, int>& ctr, int l, int r) {
    return getprefy(ctr, r - 1) - getprefy(ctr, l - 1);
}

int getsum(int v, int vl, int vr, int qlx, int qrx, int qly, int qry) {
    if (vl >= qrx || vr <= qlx) {
        return 0;
    }
    if (vl >= qlx && vr <= qrx) {
        return getsumy(tree[v], qly, qry);
    }
    return getsum(2 * v + 1, vl, (vl + vr) / 2, qlx, qrx, qly, qry) +
           getsum(2 * v + 2, (vl + vr) / 2, vr, qlx, qrx, qly, qry);
}

int main() {
    int n, m;
    scanf("%d %d\n", &n, &m);
    char* tc = new char[5000010];
    fgets(tc, 5000010, stdin);
    string t(tc);
    t.pop_back();
    Aho::init();
    char* cts = new char[1000010];
    for (int i = 0; i < n; i++) {
        fgets(cts, 1000010, stdin);
        string cur(cts);
        cur.pop_back();
        Aho::add(cur);
    }

    Aho::bfs();

    for (int i = 0; i < (int)t.size(); i++) {
        Aho::getch(t[i]);
        int cv = Aho::curv;
        while (cv != -1) {
            if (Aho::trie[cv].cntterm != 0) {
                int l = i - Aho::trie[cv].len + 1;
                add(0, 0, tsize, l, i, Aho::trie[cv].cntterm);
            }
            cv = Aho::trie[cv].suf;
        }
    }

    for (int i = 0; i < m; i++) {
        int l, r;
        scanf("%d %d\n", &l, &r);
        l--;
        r--;
        // printf("%d ", anss[--l][--r]);
        int cursum = getsum(0, 0, tsize, l, tsize, 0, r + 1);
        printf("%d ", cursum);
    }
    printf("\n");
    return 0;
}
