#ifdef LOCAL
#define _GLIBCXX_DEBUG
#else
#pragma GCC optimize("O3,unroll-loops")
#pragma GCC target("avx,avx2")
#endif

#include <algorithm>
#include <bitset>
#include <complex>
#include <deque>
#include <exception>
#include <fstream>
#include <functional>
#include <iomanip>
#include <ios>
#include <iosfwd>
#include <iostream>
#include <istream>
#include <iterator>
#include <limits>
#include <list>
#include <locale>
#include <map>
#include <memory>
#include <new>
#include <numeric>
#include <ostream>
#include <queue>
#include <set>
#include <sstream>
#include <stack>
#include <stdexcept>
#include <streambuf>
#include <string>
#include <typeinfo>
#include <utility>
#include <valarray>
#include <vector>
#include <array>
#include <atomic>
#include <chrono>
#include <codecvt>
#include <condition_variable>
#include <forward_list>
#include <future>
#include <initializer_list>
#include <mutex>
#include <random>
#include <ratio>
#include <regex>
#include <scoped_allocator>
#include <system_error>
#include <thread>
#include <tuple>
#include <typeindex>
#include <type_traits>
#include <unordered_map>
#include <unordered_set>
#include <cassert>
using namespace std;

template<typename T>
ostream& operator<<(ostream& out, const vector<T>& v) {
    for (T x : v) out << x << " ";
    return out;
}

#define all(v) (v).begin(), (v).end()
#define rall(v) (v).rbegin(), (v).rend()
#define ll long long
#define ld long double

const ll MOD = 1e9+321, P = 31;
vector<ll> pows;
int MX;

struct fenwick {

    fenwick(int n) : n(n) {
        v.resize(n);
    }

    void add(int r, int x) {
        for (; r < n; r = r | (r + 1)) {
            v[r] += x;
        }
    }

    int get(int r) {
        int s = 0;
        for (; r >= 0; r = (r & (r + 1)) - 1) {
            s += v[r];
        }
        return s;
    }

    int n;
    vector<int> v;

};

struct qr {

    qr() {}
    qr(int l, int r, int i) : l(l), r(r), i(i) {}

    int l, r, i;

};

int n, m;
string t;
vector<string> v;
vector<qr> qrs;

ll md(ll a) {
    if (a >= MOD) a -= MOD;
    if (a < 0) a += MOD;
    return a;
}

void solve() {

    cin >> n >> m;
    cin >> t;
    v.resize(n);
    for (auto& x : v) cin >> x;
    qrs.resize(m);
    for (int i = 0; i < m; i++) {
        cin >> qrs[i].l >> qrs[i].r;
        qrs[i].l--, qrs[i].r--;
        qrs[i].i = i;
    }

    MX = t.size();
    for (auto& x : v) MX = max(MX, (int)x.size());

    pows.resize(MX + 1);
    pows[0] = 1;
    for (int i = 1; i <= MX; i++) {
        pows[i] = (pows[i - 1] * P) % MOD;
    }

    sort(all(v), [](const string& a, const string& b) {
        return a.size() < b.size();
    });

    unordered_map<int, int> cnt;
    vector<vector<pair<int, int>>> pos(t.size());
    for (int i = 0; i < n; i++) {
        if (i == 0 || v[i - 1].size() != v[i].size()) cnt.clear();
        ll h = 0;
        for (int j = 0; j < v[i].size(); j++) {
            h = (h + (ll)(v[i][j] - 'a' + 1) * pows[j]) % MOD;
        }
        cnt[h * pows[MX - v[i].size() + 1] % MOD]++;

        if (i == n - 1 || v[i].size() != v[i + 1].size()) {
            vector<int> pref(t.size());
            for (int j = 0; j < t.size(); j++) {
                pref[j] = ((ll)(t[j] - 'a' + 1) * pows[j] + (j == 0 ? 0 : pref[j - 1])) % MOD;
                if (j + 1 >= v[i].size()) {
                    int h = md(pref[j] - (j + 1 == v[i].size() ? 0 : pref[j - v[i].size()])) * pows[MX - j] % MOD;
                    int d = cnt[h]; // potential O(n sqrt(n))
                    if (d != 0) pos[j].emplace_back(j - v[i].size() + 1, d);
                }
            }
        }
    }

//    for (int i = 0; i < t.size(); i++) {
//        cout << i << ": ";
//        for (int j = 0; j < pos[i].size(); j++) {
//            cout << pos[i][j].first << " " << pos[i][j].second << ", ";
//        }
//        cout << "\n";
//    }

    sort(all(qrs), [](qr a, qr b) {
        return a.r < b.r;
    });

    fenwick f(t.size());
    vector<int> ans(m);

    int ptr = 0;
    for (int i = 0; i < t.size(); i++) {
        for (int j = 0; j < pos[i].size(); j++) {
            f.add(pos[i][j].first, pos[i][j].second);
        }
        while (ptr < m && qrs[ptr].r == i) {
            ans[qrs[ptr].i] = f.get(t.size() - 1) - f.get(qrs[ptr].l - 1);
            ptr++;
        }
    }

    cout << ans;

}

int main() {
#ifndef LOCAL
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif

    cout << setprecision(10) << fixed;
    solve();

}