#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <set>
#include <iomanip>
#include <map>
#include <string>
#include <vector>
#include <cassert>
#include <unordered_map>

using namespace std;

struct query {
	int l, r, id;
	query(int l0, int r0, int i) {
		l = l0;
		r = r0;
		id = i;
	}
};

const int MAXC = 1e6 + 7;
vector <string> s[MAXC];

const int base = 179;
const int mod = 1791791791;
int b[MAXC];

void calc_b() {
	b[0] = 1;
	for (int i = 1; i < MAXC; i++)
		b[i] = (1ll * b[i - 1] * base) % mod;
}

int myhash(string& s) {
	int res = 0;
	for (auto e : s)
		res = (res * 1ll * base + (e - 'a' + 1)) % mod;
	return res;
}

vector <int> pref;
void calc_pref(string& s) {
	int n = (int)s.size();
	pref.resize(n);
	pref[0] = (s[0] - 'a' + 1);
	for (int i = 1; i < n; i++)
		pref[i] = (1ll * pref[i - 1] * base + (s[i] - 'a' + 1)) % mod;
}

int get_hash(int l, int r) {
	if (l == 0)
		return pref[r];
	int h = (pref[r] - pref[l - 1] * 1ll * b[r - l + 1]) % mod;
	if (h < 0)
		h += mod;
	return h;
}

bool comp(query v, query u) {
	return (v.r - v.l + 1) < (u.r - u.l + 1);
}

const int MAXN = (1 << 25);

vector <int> st[2 * MAXN];

void build(vector <int>& a, int tl = 0, int tr = MAXN - 1, int v = 1) {
	if (tl > tr)
		return;
	if (tl == tr) {
		if (tl < (int)a.size())
			st[v] = { a[tl] };
		return;
	}
	int mid = (tl + tr) / 2;
	build(a, tl, mid, 2 * v);
	build(a, mid + 1, tr, 2 * v + 1);
	st[v].reserve(st[2 * v].size() + st[2 * v + 1].size());
	merge(st[2 * v].begin(), st[2 * v].end(),
		st[2 * v + 1].begin(), st[2 * v + 1].end(), back_inserter(st[v]));
}

int f(int v, int x) {
	if (st[v].empty() || st[v][0] > x)
		return 0;
	if (st[v].back() <= x)
		return (int)st[v].size();
	int l = 0, r = (int)st[v].size() - 1;
	while (l < r) {
		int mid = (l + r + 1) / 2;
		if (st[v][mid] <= x)
			l = mid;
		else
			r = mid - 1;
	}
	return l + 1;
}

long long ask(int l, int r, int x) {
	if (l > r)
		return 0;
	int vl = l + MAXN;
	int vr = r + MAXN;
	long long res = 0;
	while (vl <= vr) {
		if (vl & 1) {
			res += f(vl, x);
			vl++;
		}
		if ((vr & 1) == 0) {
			res += f(vr, x);
			vr--;
		}
		vl /= 2;
		vr /= 2;
	}
	return res;
}

void make_tl() {
	while (true) {

	}
}

void solve() {
	calc_b();

	int n, m;
	cin >> n >> m;
	string t;
	cin >> t;
	int sz = (int)t.size();
	calc_pref(t);
	string temp;
	for (int i = 0; i < n; i++) {
		cin >> temp;
		s[(int)temp.size()].push_back(temp);
	}
	vector <query> q;
	vector <long long> ans(m, 0);
	for (int i = 0; i < m; i++) {
		int l, r;
		cin >> l >> r;
		l--; r--;
		q.push_back(query(l, r, i));
	}

	//	int id = 0;
	//	sort(q.begin(), q.end(), comp);
	vector <vector <int>> sum(sz);
	for (int len = 1; len < MAXC; len++) {
		if (s[len].empty())
			continue;
		unordered_map <int, bool> good_hash;
		good_hash.reserve((int)s[len].size());
		for (auto e : s[len])
			good_hash[myhash(e)] = true;
		for (int l = 0; l + len - 1 < sz; l++) {
			int r = l + len - 1;
			if (good_hash[get_hash(l, r)])
				sum[l].push_back(r);
		}
	}
	vector <int> sta(sz);
	vector <int> fin(sz);
	vector <int> a;
	for (int i = 0; i < sz; i++) {
		sta[i] = (int)a.size();
		for (auto e : sum[i])
			a.push_back(e);
		fin[i] = (int)a.size() - 1;
	}
	build(a);
	for (auto e : q) {
		int l = sta[e.l];
		int r = fin[e.r];
//		if (r >= MAXN) make_tl();
		ans[e.id] = ask(l, r, e.r);
	}
	for (auto e : ans)
		cout << e << " ";
	cout << "\n";
}

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	cout.tie(0);

	int tests = 1;
	for (int test = 1; test <= tests; test++) {
		solve();
	}
}