#include <iostream>
#include <cmath>
#include <fstream>
#include <algorithm>
#include <vector>
#include <map>
#include <set>
#include <ext/pb_ds/assoc_container.hpp>
#include <numeric>
#include <iomanip>

using namespace __gnu_pbds;
using namespace std;

#define int long long
#define all(x) (x).begin(), (x).end()
#define sz(x) (int) ((x).size())

template<typename T>
using ordered_set = tree<T, null_type, less<T>, rb_tree_tag, tree_order_statistics_node_update>;

#ifdef LOCAL
#include "algo/debug.h"
#else
#define debug(...) 42
#endif

const int K = 1002;
const int P = 179;
const int N = 1e6 + 5;

void solve() {
  int n, m;
  string t;
  cin >> n >> m >> t;

  vector<int> deg(N);
  deg[0] = 1;
  for (int i = 1; i < N; i++) {
    deg[i] = deg[i - 1] * P;
  }

  int st = sz(t);
  vector<int> hash(st + 1);
  for (int i = 0; i < st; i++) {
    hash[i + 1] = hash[i] * P + t[i];
  }

  gp_hash_table<int, int> hashes;
  vector<pair<int, int>> heavy; // [hash, len]
  for (int i = 0; i < n; i++) {
    string s;
    cin >> s;
    if (sz(s) > st) {
      continue;
    }
    int hash = 0;
    for (int j = 0; j < sz(s); j++) {
      hash = hash * P + s[j];
    }
    if (sz(s) <= K) { // light
      hashes[hash] = 1;
//      hashes.push_back(hash);
    } else { // heavy
      heavy.emplace_back(hash, sz(s));
    }
  }

  auto get_hash = [&] (int l, int r) {
    return hash[r + 1] - hash[l] * deg[r - l + 1];
  };

  vector<vector<int>> pref(st, vector<int> (K + 1));
  for (int i = 0; i < st; i++) {
    int cur_hash = 0;
    for (int len = 1; i + len <= st && len <= K; len++) {
      cur_hash = cur_hash * P + t[i + len - 1];
      pref[i][len] = pref[i][len - 1] + (hashes.find(cur_hash) != hashes.end());
    }
    if (i > 0) {
      for (int len = 1; len <= K; len++) {
        pref[i][len] += pref[i - 1][len];
      }
    }
  }

  for (int w = 0; w < m; w++) {
    int l, r;
    cin >> l >> r;
    --l, --r;
    int res = 0;
//    for (auto &[hash, len] : heavy) {
//      for (int i = l; i + len - 1 <= r; i++) {
//        res += get_hash(i, i + len - 1) == hash;
//      }
//    }
//    if (r - K + 1 >= 0) {
//      res += pref[r - K + 1][K];
//      if (l > 0) {
//        res -= pref[l][K];
//      }
//    }

    for (int i = max(l, r - K + 2); i <= r; i++) {
      int cur_hash = 0;
      for (int j = i; j <= r; j++) {
        cur_hash = cur_hash * P + t[j];
        res += hashes.find(cur_hash) != hashes.end();
      }
    }

    cout << res << '\n';
  }
}

int32_t main() {
#ifdef LOCAL
  freopen("input.txt", "r", stdin);
#endif
  ios_base::sync_with_stdio(false);
  cin.tie(nullptr);
  solve();
  return 0;
}