#include <iostream>
#include <cmath>
#include <fstream>
#include <algorithm>
#include <vector>
#include <map>
#include <set>
#include <ext/pb_ds/assoc_container.hpp>
#include <numeric>
#include <iomanip>

using namespace __gnu_pbds;
using namespace std;

//#define int long long
#define all(x) (x).begin(), (x).end()
#define sz(x) (int) ((x).size())

template<typename T>
using ordered_set = tree<T, null_type, less<T>, rb_tree_tag, tree_order_statistics_node_update>;

#ifdef LOCAL
#include "algo/debug.h"
#else
#define debug(...) 42
#endif

typedef long long ll;

const int K = 256;
const int P = 239;
const int N = 1e6 + 5;
const ll mod = 1000000000000000003;

struct Node {
  array<int, 26> go;
  bool term;
};

const int KEK = 2e5 + 5;

Node ar[KEK];
int ptr = 1;

void add(const string &s) {
  int cur = 0;
  for (auto &x : s) {
    int c = x - 'a';
    if (ar[cur].go[c] == 0) {
      ar[cur].go[c] = ptr++;
    }
    cur = ar[cur].go[c];
  }
  ar[cur].term = true;
}

ll mul(ll a, ll b) {
  return (__int128) a * (__int128) b % mod;
}

ll sum(ll a, ll b) {
  a += b;
  if (a >= mod) {
    a -= mod;
  }
  return a;
}

ll sub(ll a, ll b) {
  a -= b;
  if (a < 0) {
    a += mod;
  }
  return a;
}

void solve() {
  int n, m;
  string t;
  cin >> n >> m >> t;

  vector<ll> deg(N);
  deg[0] = 1;
  for (int i = 1; i < N; i++) {
    deg[i] = mul(deg[i - 1], P);
  }

  int st = sz(t);
  vector<ll> hash(st + 1);
  for (int i = 0; i < st; i++) {
    hash[i + 1] = sum(mul(hash[i], P), t[i]);
  }

  gp_hash_table<ll, char> hashes;
  vector<pair<ll, int>> heavy; // [hash, len]
  for (int i = 0; i < n; i++) {
    string s;
    cin >> s;
    if (sz(s) > st) {
      continue;
    }
    if (sz(s) <= K) {
      add(s);
    } else if (sz(s) > K) { // light
      ll hash = 0;
      for (int j = 0; j < sz(s); j++) {
        hash = sum(mul(hash, P), s[j]);
      }
      heavy.emplace_back(hash, sz(s));
    }
  }

  auto get_hash = [&] (int l, int r) {
    return sub(hash[r + 1], mul(hash[l], deg[r - l + 1]));
  };

  vector<int> l(m), r(m);
  vector<ll> ans(m);

  vector<vector<int>> ind(st);
  for (int w = 0; w < m; w++) {
    cin >> l[w] >> r[w];
    --l[w], --r[w];
    ind[r[w]].push_back(w);
  }

  vector<int> full(st);
  vector<vector<int>> pref(K, vector<int> (K + 1));
  for (int i = 0; i < st; i++) {
    for (int j = K - 1; j > 0; j--) {
      pref[j].swap(pref[j - 1]);
    }
    fill(all(pref[0]), 0);
    int cur = 0;
    for (int len = 1; i + len <= st && len <= K; len++) {
      int c = t[i + len - 1] - 'a';
      if (ar[cur].go[c] == 0) {
        break;
      }
      cur = ar[cur].go[c];
      int kok = ar[cur].term;
      full[i] += kok;
      pref[0][len] = kok;
    }
    for (int len = 1; len <= K; len++) {
      pref[0][len] += pref[0][len - 1];
    }
    for (int len = 1; len <= K; len++) {
      pref[0][len] += pref[1][len];
    }
    if (i > 0) {
      full[i] += full[i - 1];
    }

    for (auto &query_id : ind[i]) {
      for (int j = max(l[query_id], r[query_id] - K + 2); j <= r[query_id]; j++) {
        ans[query_id] += pref[i - j][r[query_id] - j + 1] - (i - j + 1 < K ? pref[i - j + 1][r[query_id] - j + 1] : 0);
      }
    }
  }


  for (int i = 0; i < m; i++) {
    if (r[i] - K + 1 >= l[i]) {
      ans[i] += full[r[i] - K + 1];
      if (l[i] > 0) {
        ans[i] -= full[l[i] - 1];
      }
    }
  }

  vector<int> kok(st + 1);
  for (auto &[hash, len] : heavy) {
    for (int j = 0; j + len <= st; j++) {
      kok[j + 1] = kok[j] + (get_hash(j, j + len - 1) == hash);
    }
    for (int i = 0; i < m; i++) {
      int to = r[i] - len + 1;
      if (l[i] <= to) {
        ans[i] += kok[to + 1] - kok[l[i]];
      }
    }
  }

  for (auto &x : ans) {
    cout << x << ' ';
  }
  cout << '\n';
}

int32_t main() {
#ifdef LOCAL
  freopen("input.txt", "r", stdin);
#endif
  ios_base::sync_with_stdio(false);
  cin.tie(nullptr);
  solve();
  return 0;
} // lol