#include <iostream>
#include <vector>
#include <algorithm>
#include <numeric>
#include <cstring>
#include <cmath>
#include <map>
#include <set>
#include <unordered_map>
#include <unordered_set>
#include <random>
#include <iomanip>
#include <queue>
#include <stack>
#include <tuple>
#include <bitset>
#include <cassert>

using namespace std;

//#pragma GCC optimize("O2")

typedef long long ll;
typedef pair<int, int> pii;
typedef double db;

#define X first
#define Y second
#define All(x) (x).begin(), (x).end()
#define rAll(x) (x).rbegin(), (x).rend()
#define sz(x) ((int)(x).size())

mt19937 rng(time(0));

const int LOG = 30;
const int MAXN = 2e5 + 5;

vector<vector<int>> gr, rgr;
pii edges[2 * MAXN];
vector<int> fa, fb, ord, loop, comp;
int cur_cnt = 0;

void find_ord(int v) {
    fa[v] = 0;
    for (int id : gr[v]) {
        int u = edges[id].X;
        if (fa[u] == -1)
            find_ord(u);
    }
    ord.push_back(v);
}

void dfs(int v, int x = 0) {
    fa[v] = x;
    for (int id : gr[v]) {
        int u, w;
        tie(u, w) = edges[id];
        if (fa[u] == -1)
            dfs(u, (x ^ w));
    }
}

void r_dfs(int v, int x = 0) {
    fb[v] = x;
    for (int id : rgr[v]) {
        int u, w;
        tie(u, w) = edges[id];
        if (fb[u] == -1)
            r_dfs(u, (x ^ w));
    }
}

void multi(int v) {
    if (fa[v] == 3) return;
    fa[v] = 3;
    for (int id : gr[v]) {
        int u = edges[id].X;
        if (fa[u] != 3)
            multi(u);
    }
}

void r_multi(int v) {
    if (fb[v] == 3) return;
    fb[v] = 3;
    for (int id : rgr[v]) {
        int u = edges[id].X;
        if (fb[u] != 3)
            r_multi(u);
    }
}

void css(int v, int cmp) {
    comp[v] = cmp;
    cur_cnt++;
    for (int id : rgr[v]) {
        int u = edges[id].X;
        if (comp[u] == -1)
            css(u, cmp);
    }
}

void solve() {
    int n, m, v, u, w, a, b;
    cin >> n >> m;

    gr.resize(n);
    rgr.resize(n);
    loop.resize(n);

    int ecnt = 0;

    vector<int> weight;
    for (int i = 0; i < m; i++) {
        cin >> v >> u >> w;
        v--, u--;
        if (v == u)
            loop[v] = 1;
        gr[v].push_back(ecnt);
        weight.push_back(w);
        edges[ecnt++] = { u, w };

        rgr[u].push_back(ecnt);
        weight.push_back(w);
        edges[ecnt++] = { v, w };
    }
    cin >> a >> b;
    a--, b--;
    int ans = 0;

    int lim_b = LOG;
    if (*max_element(All(weight)) <= 1)
        lim_b = 1;

    fa.assign(n, -1);
    for (int v = 0; v < n; v++) {
        if (fa[v] == -1)
            find_ord(v);
    }
    reverse(All(ord));

    comp.assign(n, -1);
    vector<int> cnt;
    for (int v : ord) {
        cur_cnt = 0;
        if (comp[v] == -1) {
            css(v, sz(cnt));
            cnt.push_back(cur_cnt);
        }
        if (cnt[comp[v]] > 1)
            loop[v] = 1;
    }

    for (int bit = 0; bit < lim_b; bit++) {
        for (int i = 0; i < ecnt; i++) {
            edges[i].Y = ((weight[i] >> bit) & 1);
        }

        fa.assign(n, -1);
        fb.assign(n, -1);

        dfs(a);
        r_dfs(b);

        for (int v = 0; v < n; v++) {
            for (int id : gr[v]) {
                int u, w;
                tie(u, w) = edges[id];
                if (min(fa[v], fa[u]) >= 0 && max(fa[v], fa[u]) < 3 && fa[u] != (fa[v] ^ w))
                    multi(u);
            }
        }
        for (int v = 0; v < n; v++) {
            for (int id : rgr[v]) {
                int u, w;
                tie(u, w) = edges[id];
                if (min(fb[v], fb[u]) >= 0 && max(fb[v], fb[u]) < 3 && fb[u] != (fb[v] ^ w))
                    r_multi(u);
            }
        }
        for (int v = 0; v < n; v++) {
            if (!loop[v]) continue;
            if (min(fa[v], fb[v]) >= 0 && (max(fa[v], fb[v]) == 3 || fa[v] != fb[v])) {
                cout << -1 << '\n';
                return;
            }
        }
        vector<int> dp(n, 0), p(n, 0);
        p[a] = 1;
        for (int v : ord) {
            if (loop[v]) continue;
            for (int id : gr[v]) {
                int u, w;
                tie(u, w) = edges[id];
                if (loop[u]) continue;
                dp[u] ^= dp[v];
                if (p[v]) {
                    p[u] ^= 1;
                    dp[u] ^= w;
                }
            }
        }
        ans ^= (dp[b] * (1 << bit));
    }
    cout << ans << '\n';
}

void init() {
    gr.clear();
    rgr.clear();
    loop.clear();
    ord.clear();
    comp.clear();
}

int main()
{
    ios::sync_with_stdio(0);
    cin.tie(0); cout.tie(0);

    int t;
    cin >> t;
    while (t--) {
        init();
        solve();
    }
}