#include <iostream>
#include <vector>

using namespace std;
using ll = long long;

void solveCycle(const vector<vector<pair<ll, ll>>> &graph, ll start, ll finish) {
    ll n = ll(graph.size());
    ll sub = 0;
    while (start != finish) {
        sub ^= graph[start][0].second;
        start = (start + 1) % n;
    }

    ll all = 0;
    for (const auto &lst: graph) {
        for (const auto &item: lst) {
            all ^= item.second;
        }
    }

    if (sub == 0 && all == 0) {
        cout << sub << '\n';
    } else {
        cout << "-1\n";
    }
}

void dfs(ll from, const vector<vector<pair<ll, ll>>> &graph, vector<bool> &used, vector<ll> &topSort) {
    used[from] = true;
    for (const auto &to: graph[from]) {
        if (used[to.first]) {
            continue;
        }

        dfs(to.first, graph, used, topSort);
    }

    topSort.push_back(from);
}

void resolve() {
    ll n, m;
    cin >> n >> m;
    vector<vector<pair<ll, ll>>> graph(n);
    bool isCycle = n == m;
    for (ll i = 0; i < m; ++i) {
        ll v, u, w;
        cin >> v >> u >> w;
        --v;
        --u;
        graph[v].emplace_back(u, w);
        if (v != i || u != (i + 1) % n) {
            isCycle = false;
        }
    }

    ll start, finish;
    cin >> start >> finish;
    --start;
    --finish;

    if (isCycle) {
        solveCycle(graph, start, finish);
        return;
    }

    vector<ll> topSort;
    vector<bool> used(n);
    dfs(start, graph, used, topSort);
    reverse(topSort.begin(), topSort.end());
    vector<ll> dp(n);
    for (const auto &from: topSort) {
        for (const auto &to: graph[from]) {
            dp[to.first] ^= dp[from] ^ to.second;
        }
    }

    cout << dp[finish] << '\n';
}

int32_t main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    ll testN;
    cin >> testN;
    while (testN--) {
        resolve();
    }

    return 0;
}
