#pragma GCC optimize("O3")
#pragma GCC target("avx,avx2")

#include <iostream>
#include <vector> 
#include <algorithm>
#include <iomanip>
#include <numeric>
#include <functional>
#include <map>

using namespace std;

#define all(v) (v).begin(), (v).end()
#define rall(v) (v).rbegin(), (v).rend()


using int64 = long long;


#define int long long

class SegmentTree {
private:
	vector <int> data;
	int size;

	int get(int ql, int qr, int nl, int nr, int node) {
		if (ql >= nr || nl >= qr) return 0;
		if (ql <= nl && nr <= qr) return data[node];
		return max(
			get(ql, qr, nl, (nl + nr) / 2, node * 2),
			get(ql, qr, (nl + nr) / 2, nr, node * 2 + 1)
		);
	}

public:
	SegmentTree(int n) {
		size = 1;
		while (size < n) size *= 2;
		data.assign(size * 2, 0);
	}

	void update(int pos, int val) {
		pos += size;
		data[pos] = max(data[pos], val);
		while ((pos /= 2) > 0) {
			data[pos] = max(data[pos * 2], data[pos * 2 + 1]);
		}
	}

	int get(int l, int r) {
		return get(l, r, 0, size, 1);
	}
};

void solve() {
	int n, m; cin >> n >> m;
	vector <vector <int>> g(n);
	vector <vector <int>> rg(n);
	vector <tuple <int, int, int>> edge(m);
	bool flag = true;
	for (int i = 0; i < m; ++i) {
		int u, v, w; cin >> u >> v >> w;
		--u; --v;
		if (u >= v) flag = false;

		g[u].push_back(v);
		rg[v].push_back(u);
		edge[i] = { u, v, w };
	}
	int start, finish; cin >> start >> finish; --start; --finish;

	if (flag) {
		vector <int> dp1(n, 0); dp1[start] = 1;
		for (int i = 0; i < n; ++i) {
			for (auto u : g[i]) {
				dp1[u] += dp1[i];
				dp1[u] %= 2;
			}
		}

		vector <int> dp2(n, 0); dp2[finish] = 1;
		for (int i = n - 1; i >= 0; --i) {
			for (auto u : rg[i]) {
				dp2[u] += dp2[i];
				dp2[u] %= 2;
			}
		}

		int ans = 0;
		for (auto [u, v, w] : edge) {
			if (dp1[u] == 1 && dp2[v] == 1) {
				ans ^= w;
			}
		}

		cout << ans;
		return;
	}

	int sm = 0;
	map <pair <int, int>, int> mp;
	for (auto [u, v, w] : edge) {
		sm ^= w;
		mp[{u, v}] = w;
	}
	if (sm != 0) {
		cout << -1;
		return;
	}

	int ans = 0;
	while (start != finish) {
		ans ^= mp[{start, (start + 1) % n}];
		start = (start + 1) % n;
	}

	cout << ans;
}


int32_t main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);
	cout.tie(nullptr);
	cout << fixed << setprecision(15);

	int t = 1;
	cin >> t;
	while (t--) {
		solve();
		cout << '\n';
	}

	return 0;
}