#include <bits/stdc++.h>

#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/tree_policy.hpp>

using namespace std;

#define ALL(a) (a).begin(), (a).end()
#define RALL(a) (a).rbegin(), (a).rend()

using i32 = int32_t;
using i64 = int64_t;
using f64 = double;
using f128 = long double;

template <typename K, typename V, typename Cmp = less<K>>
using ordered_map =
    __gnu_pbds::tree<K, V, Cmp, __gnu_pbds::rb_tree_tag,
                     __gnu_pbds::tree_order_statistics_node_update>;

template <typename T, typename Cmp = less<T>>
using ordered_set = ordered_map<T, __gnu_pbds::null_type, Cmp>;

template <typename T>
istream& operator>>(istream& is, vector<T>& v) {
  for (T& e : v) is >> e;
  return is;
}
template <typename T>
ostream& operator<<(ostream& os, const vector<T>& v) {
  for (const T& e : v) os << e << " ";
  return os;
}
template <typename T1, typename T2>
istream& operator>>(istream& is, pair<T1, T2>& p) {
  return is >> p.first >> p.second;
}
template <typename T1, typename T2>
ostream& operator<<(ostream& os, const pair<T1, T2>& p) {
  return os << p.first << " " << p.second;
}
template <typename T>
void chmax(T& a, const T& b) {
  if (a < b) a = b;
}
template <typename T>
void chmin(T& a, const T& b) {
  if (a > b) a = b;
}

void solve();

signed main() {
  ios_base::sync_with_stdio(false);
  cin.tie(nullptr);
  i32 t;
  cin >> t;
  while (t--) solve();
  return 0;
}

using Graph = vector<vector<pair<i32, i32>>>;

vector<bool> delete_extra(const Graph& gt, i32 b) {
  vector<bool> notvisited(size(gt), true);
  auto dfs = [&](i32 u, auto&& dfs) -> void {
    notvisited[u] = false;
    for (auto [v, _] : gt[u]) {
      if (notvisited[v]) dfs(v, dfs);
    }
  };
  dfs(b, dfs);
  return notvisited;
}

vector<i32> color_vertices(const Graph& g, const Graph& gt,
                           const vector<bool>& deleted, i32 a) {
  vector<bool> visited(size(g));
  vector<i32> topsort;
  auto dfs1 = [&](i32 u, auto&& dfs1) -> void {
    visited[u] = true;
    for (auto [v, _] : g[u]) {
      if (deleted[v]) continue;
      if (!visited[v]) dfs1(v, dfs1);
    }
    topsort.push_back(u);
  };
  dfs1(a, dfs1);

  visited.assign(size(visited), false);
  vector<i32> color(size(g), -1);
  i32 maxc = 0;
  auto dfs2 = [&](i32 u, i32 c, auto&& dfs2) -> void {
    color[u] = c;
    for (auto [v, _] : gt[u]) {
      if (color[v] == -1) dfs2(v, c, dfs2);
    }
  };
  for (int i = 0; i < ssize(g); ++i) {
    if (deleted[i]) continue;
    if (color[i] == -1) dfs2(i, maxc++, dfs2);
  }
  return color;
}

bool find_nonzero_1loops(const Graph& g, const vector<bool>& deleted) {
  for (int u = 0; u < ssize(g); ++u) {
    for (auto [v, w] : g[u]) {
      if (v == u && w != 0) return true;
    }
  }
  return false;
}

bool find_nonzero_loops(const Graph& g, const Graph& gt,
                        const vector<bool>& deleted, i32 a) {
  if (find_nonzero_1loops(g, deleted)) return true;

  vector<i32> color = color_vertices(g, gt, deleted, a);
  vector<i32> color_count(*max_element(ALL(color)) + 1);
  for (i32 c : color)
    if (c != -1) color_count[c]++;

  vector<i32> entry_xor(size(g), -1);
  auto dfs2 = [&](i32 u, i32 x, auto&& dfs2) -> bool {
    entry_xor[u] = x;
    for (auto [v, w] : g[u]) {
      if (deleted[v]) continue;
      i32 new_x = x ^ w;
      if (entry_xor[v] == -1) {
        if (dfs2(v, new_x, dfs2)) return true;
      } else {
        if (color_count[color[v]] > 1 && entry_xor[v] != new_x) return true;
      }
    }
    return false;
  };
  return dfs2(a, 0, dfs2);
}

tuple<Graph, i32, i32> compress_scc(const Graph& g, const Graph& gt,
                              const vector<bool>& deleted, i32 a, i32 b) {
  vector<i32> color = color_vertices(g, gt, deleted, a);
  Graph compressed(*max_element(ALL(color)) + 1);
  for (int i = 0; i < ssize(g); ++i) {
    if (deleted[i]) continue;
    for (auto [j, w] : g[i]) {
      if (deleted[j]) continue;
      compressed[color[i]].emplace_back(color[j], w);
    }
  }
  return {compressed, color[a], color[b]};
}

void solve() {
  i32 n, m;
  cin >> n >> m;
  Graph g(n), gt(n);
  for (int i = 0; i < m; ++i) {
    i32 u, v, w;
    cin >> u >> v >> w;
    --u, --v;
    g[u].emplace_back(v, w);
    gt[v].emplace_back(u, w);
  }
  i32 a, b;
  cin >> a >> b;
  --a, --b;

  vector<bool> deleted = delete_extra(gt, b);
  if (deleted[a]) {
    cout << "0\n";
    return;
  }
  if (find_nonzero_loops(g, gt, deleted, a)) {
    cout << "-1\n";
    return;
  }
  auto [gc, ac, bc] = compress_scc(g, gt, deleted, a, b);
  Graph gct(size(gc));
  for (int u = 0; u < ssize(gc); ++u) {
    for (auto [v, w] : gc[u]) {
      gct[v].emplace_back(u, w);
    }
  }
  vector<i32> cnt_nonvisited_children(size(gc));
  for (int i = 0; i < ssize(gc); ++i) {
    cnt_nonvisited_children[i] = ssize(gc[i]);
  }
  vector<i32> vxor(size(gc));
  vector<bool> odd_xor(size(gc), false);
  odd_xor[bc] = true;
  queue<i32> q;
  q.push(bc);
  while (!q.empty()) {
    i32 u = q.front();
    q.pop();
    for (auto [v, w] : gct[u]) {
      --cnt_nonvisited_children[v];
      if (cnt_nonvisited_children[v] == 0) q.push(v);
      vxor[v] ^= (w * (odd_xor[u])) ^ vxor[u];
      odd_xor[v] = odd_xor[v] ^ odd_xor[u];
    }
  }
  cout << vxor[ac] << "\n";
}
