#include <bits/stdc++.h>

using namespace std;
using ll = long long;

constexpr int N = 2e5 + 2, P = 543103, MOD = 1000696969, M = 1e9 + 7;

int power(int a, int b = M - 2) {
    int res = 1;
    for (; b > 0; b >>= 1, a = 1LL * a * a % M) {
        if (b & 1) {
            res = 1LL * res * a % M;
        }
    }
    return res;
}

void add(int& x, int y) {
    if ((x += y) >= MOD) x -= MOD;
}

int sub(int a, int b) {
    return a >= b ? a - b : a - b + MOD;
}

int powers[N];
int n;

namespace fn {
    int t[N], r[N];

    void mt(int x, int i) {
        i = 1LL * i * powers[x] % MOD;

        for (++x; x < N; x += x & -x) {
            add(t[x], i);
        }
    }

    void mr(int x, int i) {
        i = 1LL * i * powers[x] % MOD;

        for (++x; x < N; x += x & -x) {
            add(r[x], i);
        }
    }

    void modify(int x, int i) {
        if (i < 0) i += MOD;

        mt(x, 1LL * i * powers[x] % MOD);
        mr(n - x - 1, 1LL * i * powers[n - x - 1] % MOD);
    }


    int sumt(int x) {
        int ans = 0;

        for (++x; x > 0; x -= x & -x) {
            add(ans, t[x]);
        }

        return ans;
    }

    int sumr(int x) {
        int ans = 0;

        for (++x; x > 0; x -= x & -x) {
            add(ans, r[x]);
        }

        return ans;
    }


    int sumt(int l, int rr) {
        return sub(sumt(rr - 1), sumt(l - 1));
    }

    int sumr(int l, int rr) {
        return sub(sumr(rr - 1), sumr(l - 1));
    }
}

vector<int> adj[N];
int tin[N], siz[N], head[N], parent[N], T = 0;

void dfs1(int v) {
    siz[v] = 1;

    for (int &to : adj[v]) {
        dfs1(to);

        siz[v] += siz[to];

        if (siz[to] > siz[adj[v][0]]) {
            swap(adj[v][0], to);
        }
    }
}

void dfs2(int v) {
    tin[v] = T++;

    for (int to : adj[v]) {
        if (T == tin[v] + 1) {
            head[to] = head[v];
        } else {
            head[to] = to;
        }

        dfs2(to);
    }
}

vector<pair<int, int>> path(int a, int b) {
    vector<pair<int, int>> res;

    for (; ; b = parent[b]) {
        if (tin[a] > tin[b]) {
            swap(a, b);
        }

        if (tin[head[b]] <= tin[a]) {
            res.emplace_back(tin[a], tin[b] + 1);
            break;
        }

        res.emplace_back(tin[head[b]], tin[b] + 1);
        b = head[b];
    }

    return res;
}

int fa[N], mxL[N], mnR[N], answer = 1;
vector<int> subtree[N];

void init_dsu(int n) {
    iota(fa, fa + n, 0);
    for (int i = 0; i < n; ++i) {
        subtree[i] = {i};
        mnR[i] = M - 1;
    }
}

int leader(int x) {
    return x == fa[x] ? x : fa[x] = leader(fa[x]);
}

bool unite(int x, int y) {
    x = leader(x), y = leader(y);

    if (x == y) {
        return false;
    }

    if (subtree[x].size() < subtree[y].size()) {
        swap(x, y);
    }

    for (int c : subtree[y]) {
        fn::modify(c, x - y);
    }

    if (answer != 0) {
        answer = 1LL * answer * power(mnR[x] - mxL[x] + 1) % M * power(mnR[y] - mxL[y] + 1) % M;
    }

    subtree[x].insert(subtree[x].end(), subtree[y].begin(), subtree[y].end());
    subtree[y].clear();

    fa[y] = x;
    mnR[x] = min(mnR[x], mnR[y]);
    mxL[x] = max(mxL[x], mxL[y]);

    if (mnR[x] < mxL[x]) {
        answer = 0;
    } else {
        answer = 1LL * answer * (mnR[x] - mxL[x] + 1) % M;
    }

    return true;
}

//TODO: make this a bit faster
void unite_segments(int i, int j, int len, bool revi) {
    int sumi = 0, sumj = fn::sumt(j, j + len);
    int pi = 0;

    if (revi) {
        sumi = fn::sumr(n - i - 1, n - (i - len) - 1);
        pi = (n - i - 1);
    } else {
        sumi = fn::sumt(i, i + len);
        pi = i;
    }

    if (pi < j) {
        sumi = 1LL * sumi * powers[j - pi] % MOD;
    } else if (j < pi) {
        sumj = 1LL * sumj * powers[pi - j] % MOD;
    }

    if (sumi == sumj) {
        return;
    }

    if (len == 1) {
        unite(i, j);
    } else {
        int h = len / 2;
        unite_segments(i, j, h, revi);
        unite_segments(revi ? i - h : i + h, j + h, len - h, revi);
    }
}

int main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    cin >> n;

    for (int i = 1; i < n; ++i) {
        cin >> parent[i];
        parent[i] -= 1;

        adj[parent[i]].push_back(i);
    }

    dfs1(0);
    dfs2(0);

    init_dsu(n);

    for (int i = 0; i < n; ++i) {
        int t = tin[i];
        cin >> mxL[t] >> mnR[t];

        answer = 1LL * answer * (mnR[t] - mxL[t] + 1) % M;
    }

    powers[0] = 1;

    for (int i = 0; i < n; ++i) {
        if (i > 0) {
            powers[i] = 1LL * powers[i - 1] * P % MOD;
        }

        fn::modify(i, i);
    }

    int m;
    cin >> m;

    while (m--) {
        int a, b, c, d;
        cin >> a >> b >> c >> d;
        a -= 1, b -= 1, c -= 1, d -= 1;

        auto x = path(a, b);
        auto y = path(c, d);

        bool revx = x[0].first != tin[a];
        bool revy = y[0].first != tin[c];
        bool rev = false;

        if (revx ^ revy) {
            if (revy) {
                swap(x, y);
            }

            rev = true;
        }

        int j = 0, ty = y[0].first;
        int i = rev ? int(size(x)) - 1 : 0;
        int tx = rev ? x.back().second : x.front().first;

        while (j < y.size()) {
            int nxt = y[j].second - ty;

            if (rev) {
                nxt = min(nxt, tx - x[i].first);
            } else {
                nxt = min(nxt, x[i].second - tx);
            }

            unite_segments(rev ? tx - 1 : tx, ty, nxt, rev);

            ty += nxt;

            if (rev) tx -= nxt;
            else tx += nxt;

            if (rev) {
                if (tx == x[i].first) {
                    i -= 1;

                    if (i >= 0) {
                        tx = x[i].second;
                    }
                }
            } else {
                if (tx == x[i].second) {
                    ++i;

                    if (i < x.size()) {
                        tx = x[i].first;
                    }
                }
            }

            if (ty == y[j].second) {
                ++j;

                if (j < y.size()) {
                    ty = y[j].first;
                }
            }
        }

        cout << answer << '\n';
    }

    return 0;
}
