#include <iostream>
#include <algorithm>
#include <cmath>
#include <string>
#include <vector>
#include <iomanip>
#include <map>
#include <cassert>

using namespace std;

typedef long long ll;
typedef long double ld;

struct seg {
	ll l, r;
};

vector<vector<ll>> G;
vector<ll> p;
vector<seg> segs;

const ll M = 1e9 + 7, MAXN = 2e5 + 10, LOG = 20;

ll sz[MAXN], P[MAXN];
ll up[MAXN][LOG];

ll col(ll v) {
	return (P[v] == v ? v : P[v] = col(P[v]));
}

void unique(ll a, ll b) {
	a = col(a);
	b = col(b);
	if (a == b) return;
	if (sz[a] < sz[b]) {
		swap(a, b);
	}
	sz[a] += sz[b];
	P[b] = a;
}

vector<ll> d;
void dfs(ll v, ll p) {
	for (ll u : G[v]) {
		if (u == p) continue;
		d[u] = d[v] + 1;
		dfs(u, v);
	}
}

ll LA(ll a, ll k) {
	for (ll i = LOG - 1; i >= 0; i--) {
		if (k & (1 << i)) {
			a = up[a][i];
		}
	}
	return a;
}

ll LCA(ll a, ll b) {
	if (d[a] < d[b]) {
		swap(a, b);
	}
	a = LA(a, d[a] - d[b]);
	if (a == b) return a;
	for (ll i = LOG - 1; i >= 0; i--) {
		if (up[a][i] != up[b][i]) {
			a = up[a][i];
			b = up[b][i];
		}
	}
	return up[a][0];
}

vector<ll> cur2;
void f(ll u, ll v, ll lca, vector<ll>& cur) {
	cur2 = {};
	ll to = u;
	while (to != lca) {
		cur.push_back(to);
		to = p[to];
	}
	to = v;
	while (to != lca) {
		cur2.push_back(to);
		to = p[to];
	}
	cur2.push_back(lca);
	reverse(cur2.begin(), cur2.end());
	for (ll elem : cur2) {
		cur.push_back(elem);
	}
}

void solve() {
	ll n;
	cin >> n;
	d.assign(n, 0);
	for (ll i = 0; i < n; i++) {
		P[i] = i;
		sz[i] = 1;
	}
	p.assign(n, 0);
	G.assign(n, {});
	for (ll i = 1; i < n; i++) {
		cin >> p[i]; p[i]--;
		up[i][0] = p[i];
		G[p[i]].push_back(i);
		G[i].push_back(p[i]);
	}
	for (ll i = 1; i < LOG; i++) {
		for (ll j = 0; j < n; j++) {
			up[j][i] = up[up[j][i - 1]][i - 1];
		}
	}
	segs.assign(n, {});
	for (ll i = 0; i < n; i++) {
		cin >> segs[i].l >> segs[i].r;
	}
	ll q, a, b, c, d, mx_l, mn_r;
	cin >> q;
	dfs(0, -1);

	vector<ll> cur1, cur2;
	while (q--) {
		cin >> a >> b >> c >> d; a--; b--; c--; d--;
		cur1 = {}; cur2 = {};
		ll lca1 = LCA(a, b);
		f(a, b, lca1, cur1);
		ll lca2 = LCA(c, d);
		f(c, d, lca2, cur2);


		for (ll i = 0; i < cur1.size(); i++) {
			ll to1 = col(cur1[i]), to2 = col(cur2[i]);
			mx_l = max(segs[to1].l, segs[to2].l);
			mn_r = min(segs[to1].r, segs[to2].r);
			segs[to1] = { mx_l, mn_r };
			segs[to2] = { mx_l, mn_r };
			unique(to1, to2);
		}
		ll ans = 1;
		for (ll i = 0; i < n; i++) {
			if (P[i] == i) {
				ans *= max(0ll, segs[i].r - segs[i].l + 1);
				ans %= M;
			}
		}
		cout << ans << "\n";
	}
}

int main() {
	int t = 1;
	//cin >> t;
	while (t--) {
		solve();
	}
}

/*
5
1 1 2 2
2 4
1 3
1 3
2 4
4 4
4
1 1 2 2 
1 2 2 1
3 4 4 3
3 4 3 5

*/