use std::convert::TryInto;
use std::io::{self, prelude::*};

const MOD: i64 = 10i64.pow(9) + 7;
const fn mul(a: i64, b: i64) -> i64 {
    (a * b) % MOD
}

const fn add(a: i64, b: i64) -> i64 {
    (a + b) % MOD
}

const fn sub(a: i64, b: i64) -> i64 {
    add(add(a, -b), MOD)
}

const fn modpow(mut b: i64, mut e: usize) -> i64 {
    let mut ans = 1;
    while e > 0 {
        if e % 2 == 1 {
            ans = mul(ans, b);
        }
        b = mul(b, b);
        e >>= 1;
    }
    ans
}

fn dfs(g: &[Vec<usize>], d: &mut [usize], u: usize) {
    for &v in &g[u] {
        d[v] = d[u] + 1;
        dfs(g, d, v);
    }
}

fn path(p: &[usize], d: &[usize], mut u: usize, mut v: usize) -> Vec<usize> {
    let s = u;
    let (mut path1, mut path2) = (vec![], vec![]);
    while u != v {
        if d[u] < d[v] {
            std::mem::swap(&mut u, &mut v);
            std::mem::swap(&mut path1, &mut path2);
        }
        path1.push(u);
        u = p[u];
    }
    path1.push(u);
    path1.extend(path2.into_iter().rev());
    if path1.first() != Some(&s) {
        path1.reverse();
    }
    path1
}

struct UF {
    e: Vec<usize>,
}

impl UF {
    fn new(n: usize) -> UF {
        UF {
            e: (0..n).collect(),
        }
    }

    fn find(&mut self, x: usize) -> usize {
        if self.e[x] == x {
            x
        } else {
            let p = self.find(self.e[x]);
            self.e[x] = p;
            p
        }
    }

    fn join(&mut self, a: usize, b: usize) {
        let (a, b) = (self.find(a), self.find(b));
        self.e[a] = b;
    }
}

fn main() -> io::Result<()> {
    let mut scan = Scanner::new(io::BufReader::new(io::stdin().lock()));
    let mut w = io::BufWriter::new(io::stdout().lock());

    let n = scan.token();
    let mut g = vec![vec![]; n];
    let p: Vec<_> = std::iter::once(0)
        .chain((1..n).map(|i| {
            let p = scan.token::<usize>() - 1;
            g[p].push(i);
            p
        }))
        .collect();
    let mut depth = vec![0; n];
    dfs(&g, &mut depth, 0);

    let mut lim: Vec<_> = (0..n)
        .map(|_| (scan.token::<i64>() - 1, scan.token::<i64>()))
        .collect();

    let mut uf = UF::new(n);

    let mut ans = lim.iter().map(|&(l, r)| r - l).fold(1, mul);
    for _ in 0..scan.token() {
        let (a, b, c, d) = (
            scan.token::<usize>() - 1,
            scan.token::<usize>() - 1,
            scan.token::<usize>() - 1,
            scan.token::<usize>() - 1,
        );

        let p1 = path(&p, &depth, a, b);
        let p2 = path(&p, &depth, c, d);

        for (v, u) in p1.into_iter().zip(p2.into_iter()) {
            let (u, v) = (uf.find(u), uf.find(v));
            if u == v {
                continue;
            }
            uf.join(u, v);
            ans = mul(
                ans,
                modpow(lim[v].1 - lim[v].0, (MOD - 2).try_into().unwrap()),
            );
            ans = mul(
                ans,
                modpow(lim[u].1 - lim[u].0, (MOD - 2).try_into().unwrap()),
            );

            let (l1, r1) = lim[v];
            let (l2, r2) = lim[u];
            lim[v] = (l1.max(l2), r1.min(r2));
            lim[u] = (l1.max(l2), r1.min(r2));

            ans = mul(ans, (lim[v].1 - lim[v].0).max(0));
        }

        writeln!(w, "{ans}")?;
    }

    Ok(())
}

struct Scanner<R> {
    reader: R,
    buf: Vec<u8>,
    buf_iter: std::str::SplitAsciiWhitespace<'static>,
}

impl<R: BufRead> Scanner<R> {
    fn new(reader: R) -> Self {
        Self {
            reader,
            buf: vec![],
            buf_iter: "".split_ascii_whitespace(),
        }
    }

    fn token<T: std::str::FromStr>(&mut self) -> T {
        loop {
            if let Some(token) = self.buf_iter.next() {
                return token.parse().ok().expect("Failed parse");
            }
            self.buf.clear();
            self.reader
                .read_until(b'\n', &mut self.buf)
                .expect("Failed read");
            self.buf_iter = unsafe {
                let slice = std::str::from_utf8_unchecked(&self.buf);
                std::mem::transmute(slice.split_ascii_whitespace())
            };
        }
    }
}
