#include <bits/stdc++.h>

using namespace std;
#define ll long long
const ll mod = 1e9 + 7;
const ll osn = 5e5 + 1303;
vector <ll> pw;

void build_hash(int n) {
    pw.resize(n + 1);
    pw[0] = 1;
    for (int i = 1; i <= n; ++i) pw[i] = pw[i - 1] * osn % mod;
}

struct Vertex {
    ll hash1, hash2;
    int len;
};

inline Vertex merge(Vertex a, Vertex b) {
    if (a.len == 0) return b;
    if (b.len == 0) return a;
    Vertex ans;
    ans.len = a.len + b.len;
    ans.hash1 = (a.hash1 * pw[b.len] + b.hash1) % mod;
    ans.hash2 = (a.hash2 + pw[a.len] * b.hash2) % mod;
    return ans;
}

struct SegTree {
    int n;
    vector <Vertex> t;

    SegTree(int n) : n(n) {
        build_hash(n);
        t.resize(4 * n);
        build(1, 0, n - 1);
    }

    SegTree() {}

    void build(int v, int l, int r) {
        if (l == r) {
            t[v].len = 1;
            t[v].hash1 = t[v].hash2 = l + 1;
            return;
        }
        int m = (l + r) / 2;
        build(2 * v, l, m);
        build(2 * v + 1, m + 1, r);
        t[v] = merge(t[2 * v], t[2 * v + 1]);
    }

    Vertex get_seg(int v, int l, int r, int A, int B) {
        if (l > B || A > r) return {0, 0, 0};
        if (A <= l && r <= B) return t[v];
        int m = (l + r) / 2;
        return merge(get_seg(2 * v, l, m, A, B), get_seg(2 * v + 1, m + 1, r, A, B));
    }

    inline ll hash1(int l, int r) {
        return get_seg(1, 0, n - 1, l, r).hash1;
    }

    inline ll hash2(int l, int r) {
        return get_seg(1, 0, n - 1, l, r).hash2;
    }

    void update(int v, int l, int r, int i, int x) {
        if (l > i || i > r) return;
        if (l == r) {
            t[v].hash1 = t[v].hash2 = x + 1;
            return;
        }
        int m = (l + r) / 2;
        update(2 * v, l, m, i, x);
        update(2 * v + 1, m + 1, r, i, x);
        t[v] = merge(t[2 * v], t[2 * v + 1]);
    }

    inline void update(int i, int x) {
        update(1, 0, n - 1, i, x);
    }
};

inline ll power(ll x, int num) {
    ll ans = 1;
    while (num) {
        if (num & 1) {
            ans = ans * x % mod;
        }
        num >>= 1;
        x = x * x % mod;
    }
    return ans;
}

struct dsu {
    ll ans;
    vector <pair <int, int>> seg;
    vector <int> p, sz;
    vector <vector <int>> all;
    SegTree t;

    dsu(int n, const vector <pair <int, int>>& seg) : seg(seg) {
        t = SegTree(n);
        ans = 1;
        p.resize(n);
        sz.resize(n);
        all.resize(n);
        for (int i = 0; i < n; ++i) {
            p[i] = i;
            sz[i] = 1;
            all[i].push_back(i);
            ans = ans * (seg[i].second - seg[i].first + 1) % mod;
        }
    }

    int parent(int a) {
        if (a == p[a]) return a;
        p[a] = parent(p[a]);
        return p[a];
    }

    inline void unite(int a, int b) {
        a = parent(a);
        b = parent(b);
        if (a == b) return;
        if (sz[a] > sz[b]) swap(a, b);
        ans = ans * power(seg[a].second - seg[a].first + 1, mod - 2) % mod;
        ans = ans * power(seg[b].second - seg[b].first + 1, mod - 2) % mod;
        p[a] = b;
        sz[b] += sz[a];
        seg[b].second = min(seg[a].second, seg[b].second);
        seg[b].first = max(seg[a].first, seg[b].first);
        if (seg[b].first > seg[b].second) {
            ans = 0;
            return;
        }
        ans = ans * (seg[b].second - seg[b].first + 1) % mod;
        while (!all[a].empty()) {
            t.update(all[a].back(), b);
            all[b].push_back(all[a].back());
            all[a].pop_back();
        }
    }

    inline ll get_path(int a, int b) {
        if (a <= b) return t.hash1(a, b);
        else return t.hash2(b, a);
    }

    inline bool make_flex(int a, int b, int c, int d) {
        if (ans == 0) return false;
        int n1 = 1, n2 = 1;
        if (b < a) n1 = -1;
        if (d < c) n2 = -1;
        int l = -1;
        int r = max(a, b) - min(a, b) + 1;
        while (r - l > 1) {
            int m = (l + r) / 2;
            if (get_path(a, a + n1 * m) == get_path(c, c + n2 * m)) l = m;
            else r = m;
        }
        if (r == max(a, b) - min(a, b) + 1) return false;
        unite(a + n1 * l + n1, c + n2 * l + n2);
        return true;
    }
};

vector <vector <int>> g;
vector <int> p, bp, gl;

void dfs(int v) {
    if (v != 0 && p[v] != 0 && p[p[v]] != 0 && gl[p[v]] - gl[bp[p[v]]] == gl[bp[p[v]]] - gl[bp[bp[p[v]]]]) bp[v] = bp[bp[p[v]]];
    else bp[v] = p[v];
    for (int i : g[v]) {
        gl[i] = gl[v] + 1;
        dfs(i);
    }
}

inline int parent(int v, int k) {
    while (k) {
        int l = gl[v] - gl[bp[v]];
        if (k >= l) {
            k -= l;
            v = bp[v];
        } else {
            k--;
            v = p[v];
        }
    }
    return v;
}

inline int lca(int a, int b) {
    if (gl[a] < gl[b]) swap(a, b);
    a = parent(a, gl[a] - gl[b]);
    if (a == b) return a;
    while (p[a] != p[b]) {
        if (bp[a] != bp[b]) {
            a = bp[a];
            b = bp[b];
        } else {
            a = p[a];
            b = p[b];
        }
    }
    return p[a];
}

inline int get_path(int a, int b, vector <int>& path) {
    int lc = lca(a, b);
    int ln = gl[a] + gl[b] - 2 * gl[lc] + 1;
    for (int i = 0;; ++i) {
        path[i] = a;
        if (a == lc) break;
        a = p[a];
    }
    for (int i = ln - 1;; --i) {
        if (b == lc) break;
        path[i] = b;
        b = p[b];
    }
    return ln;
}

inline void solve() {
    int n;
    cin >> n;
    g.resize(n);
    p.resize(n, -1);
    bp.resize(n, -1);
    gl.resize(n);
    bool bamboo = true;
    for (int i = 1; i < n; ++i) {
        cin >> p[i]; --p[i];
        if (p[i] != i - 1) bamboo = false;
        g[p[i]].push_back(i);
    }
    vector <pair <int, int>> seg(n);
    for (int i = 0; i < n; ++i) cin >> seg[i].first >> seg[i].second;
    dsu now(n, seg);
    int q;
    cin >> q;
    if (bamboo) {
        while (q--) {
            int a, b, c, d;
            cin >> a >> b >> c >> d; --a; --b; --c; --d;
            while (now.make_flex(a, b, c, d)) {}
            cout << now.ans << "\n";
        }
        return;
    }
    dfs(0);

    vector <int> path1(n), path2(n);
    while (q--) {
        int a, b, c, d;
        cin >> a >> b >> c >> d;
        if (now.ans != 0) {
            int ln = get_path(a - 1, b - 1, path1);
            get_path(c - 1, d - 1, path2);
            for (int i = 0; i < ln && now.ans != 0; ++i) {
                now.unite(path1[i], path2[i]);
            }
        }
        cout << now.ans << "\n";
    }
}

signed main() {
    if (1) {
        ios_base::sync_with_stdio(false);
        cin.tie(nullptr);
        cout.tie(nullptr);
    }
    if (1) {
        solve();
    }
    return 0;
}

/*
5
 1 1 2 2
 2 4
 1 3
 1 3
 2 4
 4 4
 4
 1 1 2 2
 1 2 2 1
 3 4 4 3
 3 4 3 5

 8
 1 2 3 4 5 8 6
 3 7
 2 6
 3 8
 5 10
 5 8
 2 9
 3 8
 6 8
 4
 1 3 7 6
 4 1 5 7
 1 7 7 1
 1 8 2 7

 5
 1 2 3 4
 1 3
 2 4
 3 5
 4 6
 5 7
 1
 1 1 2 2

 */