#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <iomanip>
#include <unordered_map>
#include <unordered_set>
#include <map>
#include <set>
#include <deque>
#include <queue>
#include <random>
#include <chrono>

typedef long long ll;
typedef long double ld;

using namespace std;

const int MAXN = 2e5 + 5;
const int logn = 17;
const int INF = 2e9;
ll ans = 1;

vector<int> g[MAXN];
int used[MAXN], up[MAXN][logn];
int tin[MAXN], tout[MAXN];

int timer = 0;
void dfs(int v) {
    used[v] = 1;
    tin[v] = timer++;

    for (int i = 1; i < logn; ++i) {
        up[v][i] = up[up[v][i - 1]][i - 1];
    }

    for (auto to : g[v]) {
        if (!used[to]) {
            up[to][0] = v;
            dfs(to);
        }
    }

    tout[v] = timer++;
}

bool is_ancestor(int u, int v) {
    return tin[u] <= tin[v] && tout[v] <= tout[u];
}

int lca(int u, int v) {
    if (is_ancestor(u, v)) {
        return u;
    }
    if (is_ancestor(v, u)) {
        return v;
    }

    for (int i = logn - 1; i >= 0; --i) {
        if (!is_ancestor(up[u][i], v)) {
            u = up[u][i];
        }
    }

    return up[u][0];
}

int pr[MAXN];
int p[MAXN], sz[MAXN];
pair<int, int> r[MAXN];

void build(int n) {
    for (int i = 0; i < n; ++i) {
        p[i] = i;
        sz[i] = 1;
    }
}

int getRoot(int v) {
    if (p[v] == v) {
        return v;
    }

    return p[v] = getRoot(p[v]);
}

void merge(int v, int u) {
    v = getRoot(v), u = getRoot(u);
    if (v == u) {
        return;
    }

    if (sz[v] > sz[u]) {
        swap(v, u);
    }
    ans /= max(0, (r[v].second - r[v].first + 1));
    ans /= max(0, (r[u].second - r[u].first + 1));
    r[u] = {max(r[v].first, r[u].first), min(r[v].second, r[u].second)};
    ans *= max(0, (r[u].second - r[u].first + 1));
    sz[u] += sz[v];
    p[v] = u;
}

int main() {
    int n;
    cin >> n;
    build(n);
    for (int i = 1; i < n; ++i) {
        int par;
        cin >> par;
        --par;
        g[par].push_back(i);
        g[i].push_back(par);
        pr[i] = par;
    }
    dfs(0);

    for (int i = 0; i < n; ++i) {
        cin >> r[i].first >> r[i].second;
        ans *= (r[i].second - r[i].first + 1);
    }

    int m;
    cin >> m;
    for (int i = 0; i < m; ++i) {
        int a, b, c, d;
        cin >> a >> b >> c >> d;
        --a, --b, --c, --d;
        if (ans <= 0) {
            cout << ans << '\n';
            continue;
        }

        int lca1 = lca(a, b), lca2 = lca(c, d);
        vector<int> tmp1;
        while (a != lca1) {
            tmp1.push_back(a);
            a = pr[a];
        }
        tmp1.push_back(lca1);
        vector<int> tmp3;
        while (b != lca1) {
            tmp3.push_back(b);
            b = pr[b];
        }
        reverse(tmp3.begin(), tmp3.end());
        for (auto x : tmp3) {
            tmp1.push_back(x);
        }

        vector<int> tmp2;
        while (c != lca2) {
            tmp2.push_back(c);
            c = pr[c];
        }
        tmp2.push_back(lca2);
        vector<int> tmp4;
        while (d != lca2) {
            tmp4.push_back(d);
            d = pr[d];
        }
        reverse(tmp4.begin(), tmp4.end());
        for (auto x : tmp4) {
            tmp2.push_back(x);
        }

        for (int j = 0; j < (int)tmp1.size(); ++j) {
            merge(tmp1[j], tmp2[j]);
        }

        cout << ans << '\n';
    }

    return 0;
}
