#include <vector>
#include <iostream>
#include <algorithm>
#include <cmath>
#include <numeric>
using namespace std;
using ll = long long;


#define dbg(x) cerr << #x << ": " << x << endl;

const int MOD = 1e9 + 7;

int mul(ll a, int b) {
    return a * b;
}

const int MAX_N = 2e5 + 5;
const int MAX_K = 20;
vector<int> g[MAX_N];
int binup[MAX_K][MAX_N];
int tin[MAX_N];
int tout[MAX_N];
int timer = 0;

void dfs(int v) {
    tin[v] = timer++;
    for (auto u : g[v]) {
        binup[0][u] = v;
        dfs(u);
    }
    tout[v] = timer;
}

bool is_parent(int p, int v) {
    return tin[p] <= tin[v] && tout[v] <= tout[p];
}

int max_l[MAX_N];
int min_r[MAX_N];
int dsu_parent[MAX_N];
int sz[MAX_N];

int find_parent(int v) {
        if (v == dsu_parent[v]) {
            return v;
        }
        else {
            return dsu_parent[v] = find_parent(dsu_parent[v]);
        }
}

void unite(int u, int v) {
    u = find_parent(u);
    v = find_parent(v);
    if (u == v) return;
    if (sz[u] > sz[v]) swap(u, v);
    dsu_parent[u] = v;
    sz[v] += sz[u];
    max_l[v] = max(max_l[v], max_l[u]);
    min_r[v] = min(min_r[v], min_r[u]);
}

int get_lca(int u, int v) {
    if (is_parent(u, v)) return u;
    if (is_parent(v, u)) return v;

    for (int k = MAX_K - 1; k >= 0; --k) {
        if (!is_parent(binup[k][v], u)) {
            v = binup[k][v];
        }
    }
    return binup[0][v];
}


int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int n;
    cin >> n;
    for (int v = 1; v < n; ++v) {
        int p;
        cin >> p;
        --p;
        g[p].push_back(v);
    }

    for (int v = 0; v < n; ++v) {
        int l, r;
        cin >> l >> r;
        max_l[v] = l;
        min_r[v] = r;
    }

    iota(dsu_parent, dsu_parent + n, 0);
    fill_n(sz, n, 1);

    dfs(0);

    for (int k = 1; k < MAX_K; ++k) {
        for (int v = 0; v < n; ++v) {
            binup[k][v] = binup[k - 1][binup[k - 1][v]];
        }
    }

    int m;
    cin >> m;
    for (int t = 0; t < m; ++t) {
        int u1, v1;
        cin >> u1 >> v1;
        --u1, --v1;
        int u2, v2;
        cin >> u2 >> v2;
        --u2, --v2;

        vector<int> path_up1, path_down1;
        int lca1 = get_lca(u1, v1);
        while (u1 != lca1) {
            path_up1.push_back(u1);
            u1 = binup[0][u1];
        }
        while (v1 != lca1) {
            path_down1.push_back(v1);
            v1 = binup[0][v1];
        }
        reverse(path_down1.begin(), path_down1.end());
        vector<int> path1 = path_up1;
        path1.push_back(lca1);
        for (auto x : path_down1) {
            path1.push_back(x);
        }

        vector<int> path_up2, path_down2;
        int lca2 = get_lca(u2, v2);
        while (u2 != lca2) {
            path_up2.push_back(u2);
            u2 = binup[0][u2];
        }
        while (v2 != lca2) {
            path_down2.push_back(v2);
            v2 = binup[0][v2];
        }
        reverse(path_down2.begin(), path_down2.end());
        vector<int> path2 = path_up2;
        path2.push_back(lca2);
        for (auto x : path_down2) {
            path2.push_back(x);
        }
        for (int i = 0; i < (int)path1.size(); ++i) {
            unite(path1[i], path2[i]);
        }

        int res = 1;
        vector<int> values(n, 1);
        for (int v = 0; v < n; ++v) {
            int p = find_parent(v);
            values[p] = mul(res, max(0, min_r[p] - max_l[p] + 1));
        }
        for (auto x : values) {
            res = mul(res, x);
        }
        cout << res << '\n';
    }
    return 0;
}