#include <iostream>
#include <iomanip>
#include <vector>
#include <set>
#include <unordered_set>
#include <map>
#include <unordered_map>
#include <bitset>
#include <algorithm>
#include <ext/pb_ds/assoc_container.hpp>
#include <random>
#include <chrono>
#include <cassert>
#include <climits>

using namespace std;

using namespace __gnu_pbds;

template <typename T>
using ordered_set = tree <T, null_type, less<T>, rb_tree_tag, tree_order_statistics_node_update>;

template <typename T>
istream& operator>>(istream &in, vector <T> &arr) {
    for (T &i: arr) in >> i;
    return in;
}

template <typename T>
ostream& operator<<(ostream &out, const vector <T> &arr) {
    for (const T &i: arr) out << i << ' ';
    return out;
}

#define all(x) (x).begin(), (x).end()
#define rall(x) (x).rbegin(), (x).rend()

//#define int int64_t
//#define multitest

//#pragma GCC target("fma")
//#pragma GCC optimize("O3")
#pragma GCC optimize("trapv")

vector <vector <int>> g;
vector <pair <int, int>> prices;

const int mod = 1e9 + 7;

struct dsu {
    int _n;
    vector <int> p;
    vector <pair <int, int>> segs;
    explicit dsu(int n, vector <pair <int, int>> &segs) {
        _n = n;
        p.resize(n);
        iota(all(p), 0);
        assert(n == segs.size());
        this->segs = segs;
    }

    int get(int x) {
        return (p[x] == x ? x : p[x] = get(p[x]));
    }

    bool unite(int v, int u) {
        int x = get(v);
        int y = get(u);
        if (x != y) p[x] = y;
        segs[y].first = max(segs[y].first, segs[x].first);
        segs[y].second = min(segs[y].second, segs[x].second);
        return segs[y].second >= segs[y].first;
    }
};

long long mul(long long a, long long b) {
    return (a % mod) * (b % mod) % mod;
}

long long calc(dsu &d) {
    set <int> roots;
    for (int i = 1; i < d._n; ++i) {
        roots.insert(d.get(i));
    }
    long long cur = 1;
    for (int i : roots) {
        if (d.segs[i].second - d.segs[i].first + 1 <= 0) return 0;
        cur = mul(cur, d.segs[i].second - d.segs[i].first + 1);
    }
    return cur;
}

vector <int> path(int v, int prev, int goal, vector <int> &cur_path) {
    cur_path.push_back(v);
    if (v == goal) return cur_path;
    vector <int> ye = {};
    for (int u : g[v]) {
        if (u == prev) continue;
        auto x = path(u, v, goal, cur_path);
        if (x.size() > ye.size()) ye = x;
    }
    cur_path.pop_back();
    return ye;
}

void solve() {
    int n;
    cin >> n;
    g.resize(n + 1);
    prices.resize(n + 1);
    vector <int> p(n - 1);
    cin >> p;
    for (int i = 0; i < n - 1; ++i) {
        g[i + 2].push_back(p[i]);
        g[p[i]].push_back(i + 2);
    }
     for (int i = 1; i <= n; ++i) {
        cin >> prices[i].first >> prices[i].second;
    }
    int m;
    cin >> m;
    dsu snm(n + 1, prices);
    long long ans = -1;
    for (int i = 0; i < m; ++i) {
        int a, b, c, d;
        cin >> a >> b >> c >> d;

        if (ans != 0) {
            vector <int> lol;
            auto path_ab = path(a, -1, b, lol);
            lol = {};
            auto path_cd = path(c, -1, d, lol);
            for (int j = 0; j < path_ab.size(); ++j) {
                snm.unite(path_ab[j], path_cd[j]);
            }
            ans = calc(snm);
        }
        cout << ans << '\n';
    }
}

int32_t main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#ifdef LOCAL
    freopen("in.txt", "rt", stdin);
    freopen("out.txt", "wt", stdout);
    auto start = clock();
#endif
    int t;
#ifdef multitest
    cin >> t;
#else
    t = 1;
#endif
    while (t --> 0) solve();
#ifdef LOCAL
    auto end = clock();
    cerr << "time: " << end - start << endl;
#endif
    return 0;
}