#include <iostream>
#include <vector>
#include <set>

using namespace std;
using ll = long long;

const ll MOD = 1e9 + 7;

const ll maxN = 3000;

ll n;
vector<ll> graph[maxN];
pair<ll, ll> range[maxN];
ll szs[maxN];
ll parents[maxN];
set<ll> roots;

void build() {
    for (ll i = 0; i < n; ++i) {
        szs[i] = 1;
        parents[i] = i;
        roots.insert(i);
    }
}

ll getRoot(ll v) {
    ll root = v;
    while (parents[root] != root) {
        root = parents[root];
    }

    while (v != root) {
        ll tmp = v;
        v = parents[v];
        parents[tmp] = root;
    }

    return root;
}

void unite(ll v, ll u) {
    v = getRoot(v);
    u = getRoot(u);
    if (v == u) {
        return;
    }

    parents[u] = v;
    szs[v] += szs[u];
    roots.erase(u);
    range[v].first = max(range[v].first, range[u].first);
    range[v].second = min(range[v].second, range[u].second);
}

bool detect(vector<ll> &path, ll from, ll target, ll p = -1) {
    path.push_back(from);
    if (from == target) {
        return true;
    }

    for (const auto &to: graph[from]) {
        if (to == p) {
            continue;
        }

        if (detect(path, to, target, from)) {
            return true;
        }
    }

    path.pop_back();
    return false;
}

void resolve() {
    cin >> n;
    for (ll v = 1; v < n; ++v) {
        ll p;
        cin >> p;
        --p;
        graph[v].push_back(p);
        graph[p].push_back(v);
    }

    for (ll i = 0; i < n; ++i) {
        cin >> range[i].first >> range[i].second;
    }

    ll q;
    cin >> q;
    build();
    for (ll tst = 0; tst < q; ++tst) {
        ll a, b, c, d;
        cin >> a >> b >> c >> d;
        vector<ll> path1;
        vector<ll> path2;
        --a;
        --b;
        --c;
        --d;
        detect(path1, a, b);
        detect(path2, c, d);
        for (ll i = 0; i < ll(path1.size()); ++i) {
            unite(path1[i], path2[i]);
        }

        ll res = 1;
        for (const auto &v: roots) {
            res = (res * max(0LL, (range[v].second - range[v].first + 1))) % MOD;
        }

        cout << res << '\n';
    }
}

int32_t main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    resolve();
    return 0;
}
