#include <algorithm>
#include <iostream>
#include <iomanip>
#include <numeric>
#include <cassert>
#include <vector>
#include <stack>
#include <set>

typedef long long llong;
const int MAXN = 200000 + 10;
const int MOD = 1e9 + 7;
const int MAXLOG = 20;

int n, q;
int l[MAXN];
int r[MAXN];
int par[MAXN];
int dep[MAXN];
int cntEqual[MAXN];
int sparse[MAXLOG][MAXN];
std::vector <int> g[MAXN];
llong ans = 1;

llong power(int num, int base)
{
    if (base == 0) return 1;
    if (base & 1) return (num * power(num, base - 1)) % MOD;
    llong res = power(num, base / 2);
    return (res * res) % MOD;
}

struct DSU
{
    int dsuL[MAXN];
    int dsuR[MAXN];
    int par[MAXN];
    int dep[MAXN];
    void build()
    {
        std::fill(dep + 1, dep + 1 + n, 1);
        std::iota(par + 1, par + 1 + n, 1);
        for (int i = 1 ; i <= n ; ++i)
        {
            dsuL[i] = l[i];
            dsuR[i] = r[i];
        }
    }

    int find(int node)
    {
        if (par[node] == node) return node;
        return par[node] = find(par[node]);
    }

    inline void connect(int u, int v)
    {
        u = find(u);
        v = find(v);
        if (u == v)
        {
            return;
        }

        if (dep[u] > dep[v]) std::swap(u, v);
        if (dep[u] == dep[v]) dep[v]++;
        
        par[u] = v;
        int maxL = std::max(dsuL[u], dsuL[v]);
        int minR = std::min(dsuR[u], dsuR[v]);
        if (maxL > minR)
        {
            ans = 0;
            return;
        }

        ans *= power(dsuR[u] - dsuL[u] + 1, MOD - 2); ans %= MOD;
        ans *= power(dsuR[v] - dsuL[v] + 1, MOD - 2); ans %= MOD;

        dsuL[v] = maxL;
        dsuR[v] = minR;
        
        ans *= std::max(0, dsuR[v] - dsuL[v] + 1); 
        ans %= MOD;
    }

    inline bool areConnected(int u, int v)
    {
        return find(u) == find(v);
    }
};

DSU dsu;
void dfs(int node)
{
    for (const int &i : g[node])
    {
        dep[i] = dep[node] + 1;
        dfs(i);
    }
}

void buildSparse()
{
    for (int log = 1 ; (1 << log) <= n ; ++log)
    {
        for (int i = 1 ; i <= n ; ++i)
        {
            sparse[log][i] = sparse[log - 1][sparse[log - 1][i]];
        }
    }
}

inline int findKth(int node, int k)
{
    for (int log = MAXLOG - 1 ; log >= 0 ; --log)
    {
        if (k & (1 << log))
        {
            node = sparse[log][node];
        }
    }

    return node;
}

inline int getLCA(int x, int y)
{
    if (x == y)
    {
        return x;
    }

    for (int log = MAXLOG - 1 ; log >= 0 ; --log)
    {
        if (sparse[log][x] != sparse[log][y])
        {
            x = sparse[log][x];
            y = sparse[log][y];
        }
    }

    return sparse[0][x];
}

inline int findLCA(int x, int y)
{
    if (dep[x] > dep[y]) x = findKth(x, dep[x] - dep[y]);
    if (dep[x] < dep[y]) y = findKth(y, dep[y] - dep[x]);
    return getLCA(x, y);
}

std::stack <int> stackAB, stackCD;
void solve()
{
    dsu.build();
    std::fill(cntEqual + 1, cntEqual + 1 + n, 1);
    for (int i = 1 ; i <= n ; ++i)
    {
        ans *= (r[i] - l[i] + 1);
        ans %= MOD;
    }

    dep[1] = 1;
    dfs(1);
    buildSparse();
    int a, b, c, d;
    for (int i = 1 ; i <= q ; ++i)
    {
        std::cin >> a >> b >> c >> d;
        if (ans == 0)
        {
            std::cout << 0 << '\n';
            continue;
        }

        int lcaAB, lcaCD;
        lcaAB = findLCA(a, b);
        lcaCD = findLCA(c, d);

        int curr = b;
        while (curr != lcaAB)
        {
            stackAB.push(curr);
            curr = sparse[0][curr];
        }

        curr = d;
        while (curr != lcaCD)
        {
            stackCD.push(curr);
            curr = sparse[0][curr];
        }   

        bool whereAB = false;
        bool whereCD = false;
        int currABnode = a;
        int currCDnode = c;
        while (true)
        {
            dsu.connect(currABnode, currCDnode);
            if (ans == 0)
            {
                break;
            }

            if (currABnode == lcaAB)
            {
                whereAB = true;
            }

            if (currCDnode == lcaCD)
            {
                whereCD = true;
            }

            if (whereAB)
            {
                if (stackAB.empty()) break;
                currABnode = stackAB.top();
                stackAB.pop();
            } else
            {
                currABnode = sparse[0][currABnode];
            }

            if (whereCD)
            {
                assert(!stackCD.empty());
                currCDnode = stackCD.top();
                stackCD.pop();
            } else
            {
                currCDnode = sparse[0][currCDnode];
            }
        }

        std::cout << ans << '\n' << std::flush;
    }
}

void input()
{
    std::cin >> n;
    for (int i = 2 ; i <= n ; ++i)
    {
        int p;
        std::cin >> p;
        g[p].push_back(i);
        sparse[0][i] = p;
    }

    for (int i = 1 ; i <= n ; ++i)
    {
        std::cin >> l[i] >> r[i];
    }

    std::cin >> q;
}

void fastIO()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIO();
    input();
    solve();

    return 0;
}