﻿#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>

#include <algorithm>
#include <cmath>

#include <chrono>
#include <random>

#include <unordered_map>
#include <unordered_set>
#include <functional>
#include <vector>
#include <string>
#include <deque>
#include <queue>
#include <stack>
#include <array>
#include <list>
#include <set>
#include <map>

using namespace std;
using namespace chrono;


using ll = long long;
using ull = unsigned long long;
using ld = long double;

constexpr ll INF = static_cast<ll>(1e17);
constexpr ll MOD = static_cast<ll>(1e9 + 7);
constexpr ld EPS = 1e-9L;


#ifndef MSC_VER
ull __lg(ull x) {
	ull res = 0;
	for (; x > 1; x /= 2)
		++res;
	return res;
}
#endif


mt19937 rnd(0);


void solve(istream& cin, ostream& cout);


int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);


	ll _ = 1;
	// cin >> _;
	while (_--)
		solve(cin, cout);
}


template<typename Ty, typename St>
Ty binPow(Ty a, St st) {
	Ty res = 1;
	for (; st > 0; st /= 2) {
		if (st % 2 == 1)
			res = res * a;
		a = a * a;
	}
	return res;
}

struct mint {
public:
	ll x = 0;

	mint() noexcept = default;
	mint(const ll val) :x((val + MOD) % MOD) {}

	const mint operator+(const mint val) const {
		return mint(x + val.x);
	}

	const mint operator-(const mint val) const {
		return mint(x - val.x);
	}

	const mint operator*(const mint val) const {
		return mint(x * val.x);
	}

	const mint operator/(const mint val) const {
		return (*this) * binPow(val, MOD - 2);
	}
};


class DSU {
private:
	vector<ll> p, r;

public:
	DSU() = default;
	DSU(const size_t size) :p(size), r(size) {
		for (ll i = 0; i < size; ++i)
			p[i] = i;
	}

	ll find(const ll v) {
		if (p[v] == v)
			return v;
		return p[v] = find(p[v]);
	}

	bool isMerge(ll a, ll b) {
		return find(a) == find(b);
	}

	bool merge(ll a, ll b) {
		a = find(a);
		b = find(b);
		if (a == b)
			return false;
		if (r[a] < r[b])
			swap(a, b);
		p[b] = a;
		if (r[a] == r[b])
			++r[a];
		return true;
	}
};


vector<vector<ll>> g;
array<vector<ll>, 20> up;
vector<ll> tin, tout;
DSU dsu;
ll n;


vector<pair<ll, ll>> seg;
mint res;


void dfsInit(ll& timer, const ll v, const ll p = 0) {
	tin[v] = timer++;
	up[0][v] = p;
	for (ll i = 1; i < up.size(); ++i)
		up[i][v] = up[i - 1][up[i - 1][v]];

	for (auto& u : g[v]) {
		if (u != p)
			dfsInit(timer, u, v);
	}
	tout[v] = timer++;
}

bool isPred(const ll v, const ll pred) {
	return tin[pred] <= tin[v] && tout[pred] >= tin[v] && tout[pred] >= tout[v];
}

ll lca(ll a, ll b) {
	if (isPred(a, b))
		return b;
	if (isPred(b, a))
		return a;

	for (ll i = up.size() - 1; i >= 0; --i)
		if (!isPred(b, up[i][a]))
			a = up[i][a];
	return up[0][a];
}


void dfs(const ll v, const ll p, vector<ll>& path) {
	path.push_back(v);
	if (v == p)
		return;
	dfs(up[0][v], p, path);
}


vector<ll> getPath(const ll a, const ll b) {
	ll d = lca(a, b);
	vector<ll> res;
	if (b == d) {
		dfs(a, d, res);
		return res;
	}
	if (a == d) {
		dfs(b, d, res);
		reverse(res.begin(), res.end());
		return res;
	}
	dfs(a, d, res);
	ll buf = res.size();
	dfs(b, d, res);
	res.pop_back();
	reverse(res.begin() + buf, res.end());
	return res;
}


void solve(istream& cin, ostream& cout) {
	cin >> n;
	g.resize(n);
	for (ll i = 0; i < up.size(); ++i)
		up[i].resize(n);
	tin.resize(n);
	tout.resize(n);
	seg.resize(n);
	dsu = DSU(n);

	for (ll i = 1; i < n; ++i) {
		ll p;
		cin >> p;
		--p;
		g[p].push_back(i);
	}
	ll timer = 1;
	dfsInit(timer, 0, 0);


	res = 1;
	for (ll i = 0; i < n; ++i) {
		auto& [l, r] = seg[i];
		cin >> l >> r;

		res = res * mint(r - l + 1);
	}

	ll m;
	cin >> m;
	for (ll kkk = 0; kkk < m; ++kkk) {
		ll a, b, c, d;
		cin >> a >> b >> c >> d;
		--a, --b, --c, --d;

		vector<ll> x = move(getPath(a, b));
		vector<ll> y = move(getPath(c, d));

		for (ll i = 0; i < x.size() && res.x != 0; ++i) {
			if (dsu.isMerge(x[i], y[i]))
				continue;
			auto& [l, r] = seg[dsu.find(x[i])];
			auto& [q, w] = seg[dsu.find(y[i])];
			res = res / mint(r - l + 1);
			res = res / mint(w - q + 1);
			dsu.merge(x[i], y[i]);

			l = q = max(l, q);
			r = w = min(r, w);

			if (l > r)
				res = 0;
			else
				res = res * mint(r - l + 1);
		}


		cout << res.x << '\n';
	}
}
