#include <iostream>
#include <vector>
#include <set>
#include <iomanip>
#include <algorithm>

using namespace std;

vector<long long> ranks, parents;
vector<bool> used;
vector<vector<long long>> range;
vector<set<long long>> up;

long long get_parent(long long pos) {
	if (parents[pos] != pos) {
		parents[pos] = get_parent(parents[pos]);
	}
	return parents[pos];
}

pair<long long, long long> merge(long long a, long long b) {
	// cout << a << ' ' << b << " -> ";
	a = get_parent(a);
	b = get_parent(b);
	// cout << a << ' ' << b << endl;

	if (a == b) {
		return {-1, -1};
	}
	pair<long long, long long> res = {max(range[a][1] + 1 - range[a][0], (long long) 0),
									  max(range[b][1] + 1 - range[b][0], (long long) 0)};
	if (ranks[a] < ranks[b]) {
		swap(a, b);
	}
	parents[b] = a;
	if (ranks[a] == ranks[b]) {
		ranks[a]++;
	}
	range[a] = {max(range[b][0], range[a][0]), min(range[b][1], range[a][1])};
	return res;
}

void dfs(long long pos, long long dist, const vector<set<long long>>& ways, vector<long long>& path) {
	used[pos] = 1;
	path.push_back(pos);
	if (pos == dist) {
		return;
	}
	for (int i : ways[pos]) {
		if (!used[i]) {
			dfs(i, dist, ways, path);
			if (path[path.size() - 1] == dist) {
				return;
			}
		}
	}
	path.pop_back();
}

void solve() {
	long long n, m, x, y, c, d;
	cin >> n;
	ranks.resize(n, 1);
	up.resize(n);
	long long inp;

	for (int i = 0; i < n - 1; ++i) {
		cin >> inp;
		up[i + 1].insert(inp - 1);
		up[inp - 1].insert(i + 1);
		parents.push_back(i);
	}
	parents.push_back(n - 1);

	long long all = 1;
	for (int i = 0; i < n; ++i) {
		cin >> x >> y;
		range.push_back({x, y});
		all *= y + 1 - x;
	}

	cin >> m;
	vector<long long> path1, path2;
	pair<long long, long long> pr;
	for (int i = 0; i < m; ++i) {
		cin >> x >> y >> c >> d;
		--x; --y; --c; --d;

		path1 = {};
		path2 = {};
		used = vector<bool>(n, 0);
		dfs(x, y, up, path1);
		used = vector<bool>(n, 0);
		dfs(c, d, up, path2);

		for (int j = 0; j < path1.size(); ++j) {
			pr = merge(path1[j], path2[j]);
			// cout << ">> " << path1[j] << ' ' << path2[j] << endl;
			if (pr.first > -1) {
				if (pr.first == 0) {
					all = 0;
				}
				else {
					all /= pr.first;
				}
				if (pr.second == 0) {
					all = 0;
				}
				else {
					all /= pr.second;
				}
				inp = get_parent(path1[j]);
				all *= max(range[inp][1] - range[inp][0] + 1, (long long) 0);
				// cout << pr.first << ' ' << pr.second << ' ' << range[inp][0]
				// 	 << ' ' << range[inp][1] << ' ' << inp << endl;
			}
		}
		cout << all << endl;
	}
}

signed main() {
	ios::sync_with_stdio(false);
	cin.tie(0);cout.tie(0);

	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);

	solve();

	return 0;
}