#include <algorithm>
#include <iostream>
#include <numeric>
#include <vector>

using namespace std;

constexpr long long mod = 1000000007;

constexpr long long pow(long long a, long long b) {
    long long res = 1;
    while (b > 0) {
        if (b & 1) {
            res = (res * a) % mod;
        }
        a = (a * a) % mod;
        b >>= 1;
    }
    return res;
}

constexpr long long getrev(long long x) {
    return pow(x, mod - 2);
}

vector<vector<int>> gr;
vector<int> pr;
vector<int> tin;
vector<int> tout;
vector<pair<int, int>> lr;
int tm = 0;

void dfs(int v) {
    tin[v] = tm++;
    for (int to : gr[v]) {
        dfs(to);
    }
    tout[v] = tm++;
}

long long res = 1;

void getpath(int a, int b, vector<int>& pt) {
    while (tin[a] > tin[b] || tout[a] < tout[b]) {
        pt.emplace_back(a);
        a = pr[a];
    }
    vector<int> t2;
    while (a != b) {
        t2.emplace_back(b);
        b = pr[b];
    }
    pt.emplace_back(a);
    pt.insert(pt.end(), t2.rbegin(), t2.rend());
}

namespace DSU {
    constexpr int MAXN = 200000;
    int p[MAXN];
    int r[MAXN];
    void init() {
        iota(p, p + MAXN, 0);
        fill(r, r + MAXN, 1);
    }

    int getp(int v) {
        if (v == p[v]) {
            return v;
        }
        return p[v] = getp(p[v]);
    }

    void unite(int a, int b) {
        a = getp(a);
        b = getp(b);
        if (a == b) {
            return;
        }
        if (r[a] < r[b]) {
            swap(a, b);
        }
        r[a] += r[b];
        p[b] = a;
    }
} // namespace DSU

int main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int n;
    cin >> n;
    gr.resize(n);
    pr.resize(n);
    tin.resize(n);
    tout.resize(n);
    lr.resize(n);
    for (int i = 1; i < n; i++) {
        cin >> pr[i];
        pr[i]--;
        gr[pr[i]].emplace_back(i);
    }

    for (int i = 0; i < n; i++) {
        cin >> lr[i].first >> lr[i].second;
        res =  res * (lr[i].second - lr[i].first + 1) % mod;
    }

    dfs(0);

    DSU::init();

    int m;
    cin >> m;
    for (int iter = 0; iter < m; iter++) {
        int a, b, c, d;
        cin >> a >> b >> c >> d;
        a--;
        b--;
        c--;
        d--;
        vector<int> pt1;
        vector<int> pt2;
        getpath(a, b, pt1);
        getpath(c, d, pt2);

        for (int i = 0; i < (int)pt1.size(); i++) {
            int cv1 = DSU::getp(pt1[i]);
            int cv2 = DSU::getp(pt2[i]);
            if (cv1 != cv2) {
                res = res * getrev(lr[cv1].second - lr[cv1].first + 1) % mod;
                res = res * getrev(lr[cv2].second - lr[cv2].first + 1) % mod;

                lr[cv1].first = lr[cv2].first = max(lr[cv1].first, lr[cv2].first);
                lr[cv1].second = lr[cv2].second = min(lr[cv1].second, lr[cv2].second);
                DSU::unite(cv1, cv2);

                cv1 = DSU::getp(cv1);
                res = res * (lr[cv1].second - lr[cv1].first + 1) % mod;
                if (lr[pt1[i]].second < lr[pt1[i]].first) {
                    res = 0;
                }
            }
        }
        cout << res << "\n";
    }
    return 0;
}
