#include <iostream>
#include <vector>
#include <algorithm>
#include <assert.h>
#include <iomanip>
#include <utility>
#include <set>
#include <map>

using namespace std;

typedef long long ll;
typedef unsigned long long ull;
typedef long double ld;
typedef unsigned int uint;

const int MOD = (int)1e9 + 7;
struct mint {
	int x = 0;
	mint() {};
	mint(ll value) {
		value %= MOD;
		if (value < 0) value += MOD;
		x = value;
	}
	mint pow(int n) const {
		mint res(1);
		mint a(x);
		while (n) {
			if (n & 1) {
				res = res * a;
			}
			a = a * a;
			n /= 2;
		}
		return res;
	}

	mint rev() const {
		return (*this).pow(MOD - 2);
	}
	mint operator+(const mint& b) {
		return mint(x + b.x);
	}
	mint operator-(const mint& b) {
		return mint(x - b.x);
	}
	mint operator*(const mint& b) {
		return mint(1ll * x * b.x);
	}
	mint operator/(const mint& b) {
		return mint(x) * b.rev();
	}
};
const int maxn = (int)2e5 + 10;
vector<int> g[maxn];
int tin[maxn], tout[maxn];
int up[20][maxn];
int p[maxn], l[maxn], r[maxn];


int par(int v) {
	if (p[v] == v) return v;
	return p[v] = par(p[v]);
}
int unite(int v, int u) {
	v = par(v);
	u = par(u);
	if (v == u) return 0;
	p[u] = v;
	l[v] = max(l[v], l[u]);
	r[v] = min(r[v], r[u]);
	return 1;
}
int t = 0;
void dfs(int v, int p = 0) {
	tin[v] = t++;
	up[0][v] = p;
	for (int i = 1; i <= 19; i++) {
		up[i][v] = up[i - 1][up[i - 1][v]];
	}
	for (int u : g[v]) {
		if (u != p) {
			dfs(u, v);
		}
	}
	tout[v] = t++;
}

bool ancestor(int v, int u) {
	return tin[v] <= tin[u] && tout[v] >= tout[u];
}

int lca(int v, int u) {
	if (ancestor(v, u)) return v;
	if (ancestor(u, v)) return u;
	for (int p = 19; p >= 0; p--) {
		if (!ancestor(up[p][v], u)) {
			v = up[p][v];
		}
	}
	return up[0][v];
}

vector<int> getPath(int v, int u) {
	vector<int> res1, res2;
	int l = lca(v, u);
	while (v != l) {
		res1.push_back(v);
		v = up[0][v];
	}
	while (u != l) {
		res2.push_back(u);
		u = up[0][u];
	}
	res1.push_back(l);
	for (int i = (int)res2.size() - 1; i >= 0; i--) {
		res1.push_back(res2[i]);
	}
	return res1;
}

mint get(int v) {
	v = par(v);
	int sz = max(r[v] - l[v] + 1, 0);
	return mint(sz);
}

void solve() {
	int n;
	cin >> n;
	for (int i = 1; i < n; i++) {
		int p;
		cin >> p;
		g[i].push_back(p - 1);
		g[p - 1].push_back(i);
	}
	dfs(0, 0);
	mint ans = 1;
	for (int i = 0; i < n; i++) {
		p[i] = i;
		cin >> l[i] >> r[i];
		ans = ans * get(i);
	}
	int m;
	cin >> m;
	for (int _ = 0; _ < m; _++) {
		int a, b, c, d;
		cin >> a >> b >> c >> d;
		a--, b--, c--, d--;
		if (ans.x == 0) {
			cout << 0 << '\n';
			continue;
		}
		vector<int> ab = getPath(a, b);
		vector<int> cd = getPath(c, d);
		if (ab.size() != cd.size()) return;
		for (int i = 0; i < ab.size(); i++) {
			int v = ab[i];
			int u = cd[i];
			if (par(v) == par(u)) continue;
			ans = ans / get(par(v));
			ans = ans / get(par(u));
			unite(v, u);
			ans = ans * get(par(v));
		}
		cout << ans.x << '\n';
	}
}

int main()
{
	ios_base::sync_with_stdio(0);
	cin.tie(0);

	int t = 1;
	//cin >> t;

	while (t--) {
		solve();
	}
}
