#pragma GCC optimize("O3,unroll-loops")

#include <iostream>
#include <algorithm>
#include <vector>
#include <cmath>
#include <iomanip>
#include <set>
#include <map>
#include <unordered_map>
#include <unordered_set>
#include <numeric>
#include <cassert>

using namespace std;
using ll = long long;
using ld = long double;

vector<vector<int>> g;
vector<int> p;

struct SparseTable{
    vector<int> eult;
    vector<int> h;
    vector<int> v;
    vector<vector<int>> sp;
    int n;
    vector<int> log2;

    int merge(int a, int b){
        if (v[a] <= v[b]){
            return a;
        }
        return b;
    }

    void lg(){
        log2.resize(n + 1);
        log2[0] = log2[1] = 0;
        for (int i = 2; i <= n; ++i){
            log2[i] = log2[i / 2] + 1;
        }
    }

    SparseTable(vector<int> &e, vector<int> &h_){
        eult = e;
        h = h_;
        n = e.size();
        v.resize(n);
        for (int i = 0; i < v.size(); ++i) {
            v[i] = h[eult[i]];
        }
        sp.assign(20, vector<int>(n));
        iota(sp[0].begin(), sp[0].end(), 0);
        for (int log = 1; log < 20; ++log) {
            for (int i = 0; i < n; ++i) {
                sp[log][i] = merge(sp[log - 1][i], sp[log - 1][min(n - 1, i + (1 << (log - 1)))]);
            }
        }
        lg();
    }

    int lca(int a, int b){
        int szb = log2[b - a];
        return merge(sp[szb][a], sp[szb][b - (1 << szb)]);
    }
};

vector<int> way(int a, int b, int l){
    vector<int> wa;
    vector<int> wb;
    while (a != l){
        wa.push_back(a);
        a = p[a];
    }
    while (b != l){
        wb.push_back(b);
        b = p[b];
    }
    wa.push_back(l);
    reverse(wb.begin(), wb.end());
    for (auto el : wb){
        wa.push_back(el);
    }
    return wa;
}

struct ET{
    int n;
    vector<int> e;
    vector<int> tin;
    vector<int> tout;
    vector<int> h;

    void dfs(int v, int he){
        h[v] = he;
        tin[v] = e.size();
        e.push_back(v);
        for (auto u : g[v]){
            dfs(u, he + 1);
            e.push_back(v);
        }
        tout[v] = e.size();
        e.push_back(v);
    }

    ET(int n_){
        n = n_;
        e.reserve(2 * n + 1);
        tin.resize(n);
        tout.resize(n);
        h.resize(n);
        dfs(0, 0);
    }
};


int lca(ET &et, SparseTable &sp, int a, int b){
    return et.e[sp.lca(min(et.tin[a], et.tin[b]), max(et.tout[a], et.tout[b]))];
}

long long answer = 1;
long long P = 1e9 + 7;

long long pow2(ll a, ll n){
    if (n == 0){
        return 1;
    }
    if (n == 1){
        return a;
    }
    if (n % 2 == 0){
        ll x = pow2(a, n / 2);
        return (x * x) % P;
    }
    return (a * pow2(a, n - 1)) % P;
}

struct DSU{
    vector<int> p;
    vector<int> sz;
    vector<int> l, r;

    int ans(int i){
        return max(0, r[i] - l[i] + 1);
    }

    DSU(int n, vector<int> &l_, vector<int> &r_){
        p.resize(n);
        iota(p.begin(), p.end(), 0);
        sz.assign(n, 0);
        l = l_;
        r = r_;
        for (int i = 0; i < n; ++i) {
            answer = (answer * ans(i)) % P;
        }
    }


    int h_p(int v){
        if (v == p[v]){
            return v;
        }
        int pp = h_p(p[v]);
        p[v] = pp;
        return pp;
    }

    void unite(int a, int b){
        a = h_p(a);
        b = h_p(b);
        if (a == b){
            return;
        }
        if (sz[a] < sz[b]){
            swap(a, b);
        }
        p[b] = a;
        sz[a] += sz[b];
        answer *= pow2(ans(b), P - 2);
        answer %= P;
        answer *= pow2(ans(a), P - 2);
        answer %= P;
        l[a] = max(l[a], l[b]);
        r[a] = min(r[a], r[b]);
        answer *= ans(a);
        answer %= P;
    }
};


int main(){
#ifdef DEBUG
    freopen("input.txt", "r", stdin);
    auto BEGIN = (ld)clock() / CLOCKS_PER_SEC;
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
#endif
    int n;
    cin >> n;
    p.resize(n);
    g.resize(n);
    for (int i = 1; i < n; ++i) {
        cin >> p[i];
        --p[i];
        g[p[i]].push_back(i);
    }
    ET et(n);
    SparseTable sp(et.e, et.h);
    vector<int> l(n);
    vector<int> r(n);
    for (int i = 0; i < n; ++i) {
        cin >> l[i] >> r[i];
    }
    int m;
    cin >> m;
    DSU dsu(n, l, r);
    for (int i = 0; i < m; ++i) {
        int a, b, c, d;
        cin >> a >> b >> c >> d;
        --a; --b; --c; --d;
        vector<int> way1 = way(a, b, lca(et, sp, a, b));
        vector<int> way2 = way(c, d, lca(et, sp, c, d));
//        for (auto el : way1){
//            cout << el << ' ';
//        }
//        cout << endl;
//        for (auto el : way2){
//            cout << el << ' ';
//        }
//        cout << endl;

        assert(way1.size() == way2.size());
        for (int j = 0; j < way1.size(); ++j){
            dsu.unite(way1[j], way2[j]);
        }
        cout << answer << '\n';
    }
#ifdef DEBUG
    cerr << (ld)clock() / CLOCKS_PER_SEC - BEGIN << endl;
#endif
}