#define _CRT_SECURE_NO_WARNINGS
#include <algorithm>
#include <iostream>
#include <vector>
#include <cmath>
#include <set>
#include <map>
#include <random>
#include <numeric>
#include <unordered_set>
using namespace std;

mt19937 rnd(1000 - 7);

using ll = long long;
using ld = long double;

const ll INF = 1e9;
const ll LLINF = 1e18;
const ll MOD = 1e9 + 7;
const ld PI = 2 * atan2(1, 0);
const ld EPS = 1e-8;

//struct Point {
//	ll x, y;
//	
//	Point() : x(0), y(0) {}
//	Point(ll _x, ll _y) : x(_x), y(_y) {}
//
//	Point operator+(const Point& other) const {
//		return Point(x + other.x, y + other.y);
//	}
//
//	Point operator-(const Point& other) const {
//		return Point(x - other.x, y - other.y);
//	}
//
//	Point operator^(const Point& other) const {
//		return other - *this;
//	}
//
//	ll operator*(const Point& other) const {
//		return x * other.x + y * other.y;
//	}
//
//	ll operator%(const Point& other) const {
//		return x * other.y - y * other.x;
//	}
//};

pair<ll, ll> mergeRanges(pair<ll, ll> a, pair<ll, ll> b) {
	return { max(a.first, b.first), min(a.second, b.second) };
}

struct DSU {
	vector<int> p, r;
	vector<pair<ll, ll>> rng;
	
	void build(int n, vector<pair<ll, ll>>& _rng) {
		p.assign(n, 0);
		for (int i = 0; i < n; ++i)
			p[i] = i;
		r.assign(n, 0);
		rng = _rng;
	}

	int get(int u) {
		if (p[u] == u) {
			return u;
		}
		return p[u] = get(p[u]);
	}

	pair<ll, ll> getRange(int u) {
		return rng[u];
	}

	void add(int a, int b) {
		a = get(a);
		b = get(b);
		if (r[a] < r[b])
			swap(a, b);
		p[b] = a;
		if (r[a] == r[b])
			r[a]++;
		rng[a] = mergeRanges(rng[a], rng[b]);
	}
};

vector<vector<int>> g;
vector<pair<ll, ll>> r;

bool getPath(int u, int p, int s, vector<int>& res) {
	if (u == s) {
		res.push_back(u);
		return true;
	}
	for (auto v : g[u]) {
		if (v == p)
			continue;
		if (getPath(v, u, s, res)) {
			res.push_back(u);
			return true;
		}
	}
	return false;
}

void solve3() {
	int n;
	cin >> n;
	g.resize(n);
	for (int u = 1; u < n; ++u) {
		int v;
		cin >> v;
		v--;
		g[u].push_back(v);
		g[v].push_back(u);
	}
	r.resize(n);
	for (int i = 0; i < n; ++i) {
		cin >> r[i].first >> r[i].second;
	}

	DSU dsu;
	dsu.build(n, r);

	int m;
	cin >> m;
	for (int i = 0; i < m; ++i) {
		int a, b, c, d;
		cin >> a >> b >> c >> d;
		a--, b--, c--, d--;
		vector<int> p1, p2;
		getPath(a, -1, b, p1);
		getPath(c, -1, d, p2);
		for (int i = 0; i < p1.size(); ++i) {
			dsu.add(p1[i], p2[i]);
		}
		ll res = 1;
		for (int i = 0; i < n; ++i) {
			if (dsu.get(i) == i) {
				auto irng = dsu.getRange(i);
				res = (res * max(0ll, irng.second - irng.first + 1)) % MOD;
			}
		}
		cout << res << "\n";
	}
}

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

	// freopen("input.txt", "r", stdin);
	// freopen("output.txt", "w", stdout);

	solve3();
}