#ifndef DEBUG
#pragma GCC optimize("-Ofast")
#pragma GCC target("avx,avx2")
#endif

#include <bits/stdc++.h>
#include <bits/extc++.h>

using namespace std;
using ll = long long;
using vi = vector<ll>;
using vb = vector<bool>;
using ii = pair<ll, ll>;
using vii = vector<ii>;
using vvi = vector<vi>;

template<class T, class Y>
using umap = __gnu_pbds::gp_hash_table<T, Y>;
template<class T>
using uset = __gnu_pbds::gp_hash_table<T, __gnu_pbds::null_type>;

template<class T, class Y>
istream &operator>>(istream &s, pair<T, Y> &p) {
  return s >> p.first >> p.second;
}

template<class T>
istream &operator>>(istream &s, vector<T> &v) {
  for (auto &e : v) s >> e;
  return s;
}

template<class T>
ostream &operator<<(ostream &s, const vector<T> &v) {
  for (const auto &e : v) s << e << ' ';
  return s;
}

void solve();

int main() {
#ifdef DEBUG
  freopen("../in.txt", "r", stdin);
  freopen("../out.txt", "w", stdout);
#endif
  ios_base::sync_with_stdio(false);
  cin.tie(nullptr);
  cout.setf(ios::fixed);
  cout.precision(20);

  solve();
}

ll modpow(ll base, ll exp, ll mod) {
  ll res = 1;
  while (exp) {
    if (exp & 1) {
      res = (res * base) % mod;
    }
    base = (base * base) % mod;
    exp >>= 1;
  }
  return res;
}

// ----------

constexpr ll mod = 1e9 + 7;

ll modinv(ll x) {
  return modpow(x, mod - 2, mod);
}

ll n;
vvi gr;
vi p;
vii range;

bool pathfind(ll v, ll t, vi &path, ll par = -1) {
  if (v == t) return true;
  for (auto u : gr[v]) {
    if (u == par)continue;
    path.emplace_back(u);
    if (pathfind(u, t, path, v)) return true;
    path.pop_back();
  }
  return false;
}

vi pathfind(ll a, ll b) {
  vi path = {a};
  assert(pathfind(a, b, path));
  return path;
}

class dsu_t {
 public:
  dsu_t() {
    p.resize(n);
    for (int i = 0; i < n; ++i) {
      p[i] = i;
    }
    rank.resize(n, 0);
    rng.resize(n, {});
    ans = 1;
    for (int i = 0; i < n; ++i) {
      rng[i] = range[i];
      rng[i].second += 1;
      ans *= rng[i].second - rng[i].first;
      ans %= mod;
    }
  }

  void ansadd(ll& an, const ii& r) {
    an = (an * (r.second - r.first)) % mod;
  }

  void ansadd(ll& an, const ll& r) {
    an = (an * r) % mod;
  }

  void ansrem(ll& an, const ii& r) {
    an = (an * modinv(r.second - r.first)) % mod;
  }

  void ansrem(ll& an, const ll& r) {
    an = (an * modinv(r)) % mod;
  }

  ll unite(ll a, ll b) {
    a = get(a), b = get(b);
    if (a == b) return a;
    if (rank[a] < rank[b]) swap(a, b);

    ansrem(ans, rng[a]);
    ansrem(ans, rng[b]);

    p[b] = a;
    if (rank[a] == rank[b]) rank[a] += 1;

    rng[a] = {max(rng[a].first, rng[b].first), min(rng[a].second, rng[b].second)};
    if (rng[a].first >= rng[a].second) rng[a] = {0, 0};

    ansadd(ans, rng[a]);

    return a;
  }

  ll get(ll v) {
    while (v != p[v]) v = p[v];
    return v;
  }

  vi p;
  vi rank;
  vii rng;
  ll ans;
};

void solve() {
  cin >> n;
  p.assign(n, -1);
  range.assign(n, {});
  gr.assign(n, {});
  for (int i = 1; i < n; ++i) {
    cin >> p[i];
    --p[i];
    gr[i].emplace_back(p[i]);
    gr[p[i]].emplace_back(i);
  }
  cin >> range;

  dsu_t dsu;

  ll m;
  cin >> m;
  for (int qq = 0; qq < m; ++qq) {
    ll a, b, c, d;
    cin >> a >> b >> c >> d;
    --a, --b, --c, --d;
    vi ab = pathfind(a, b);
    vi cd = pathfind(c, d);
    for (int i = 0; i < cd.size(); ++i) {
      if (dsu.ans == 0) break;
      dsu.unite(ab[i], cd[i]);
    }
    cout << dsu.ans << endl;
  }
}