#include <iostream>
#include <vector>
#include <iomanip>
#include <map>
#include <random>
#include <ext/pb_ds/tree_policy.hpp>
#include <ext/pb_ds/assoc_container.hpp>
#include <ext/rope>

using namespace std;
using namespace __gnu_pbds;
using namespace __gnu_cxx;
mt19937 rnd(459);

typedef long long ll;
typedef unsigned int uint;
typedef unsigned long long ull;
typedef long double ld;

#define all(x) x.begin(), x.end()
#define rall(x) x.rbegin(), x.rend()
#ifdef DUDE
#define __GLIBCXX_DEBUG
#define debug(x) cout << #x << ":\t" << x << endl
#define debugb(b, x) cout << b << x << endl
#else
#define debug(x) ;
#define debugb(b, x) ;
#endif

template<typename T>
istream& operator >> (istream& in, vector<T>& vec) {
    for (T& e :vec)
        in >> e;
    return in;
}

template<typename T>
ostream& operator << (ostream& out, const vector<T>& vec) {
    char end_char = '\n';
    if (typeid(T) == typeid(int) || typeid(T) == typeid(ll) || typeid(T) == typeid(char))
        end_char = ' ';
    for (const T& e :vec) {
        out << e << end_char;
    }
    return out;
}

template<typename T>
bool maxi(T& a, const T& b) {
    if (a < b) {
        a = b;
        return true;
    }
    return false;
}

template<typename T>
bool mini(T& a, const T& b) {
    if (a > b) {
        a = b;
        return true;
    }
    return false;
}

/*
 *
 */

const int MOD = 1e9 + 7;

struct ModInt {
    int val;
    ModInt(int v): val(v) {}
    ModInt(ll v): val(v % MOD) {}
    ModInt operator + (const ModInt& other) const {
        int v = val + other.val;
        if (v >= MOD)
            v -= MOD;
        return v;
    }
    ModInt& operator += (const ModInt& other) {
        val += other.val;
        if (val >= MOD)
            val -= MOD;
        return *this;
    }
    ModInt operator - (const ModInt& other) const {
        int v = val - other.val;
        if (v < 0)
            v += MOD;
        return v;
    }
    ModInt& operator -= (const ModInt& other) {
        val -= other.val;
        if (val < 0)
            val += MOD;
        return *this;
    }
    ModInt operator * (const ModInt& other) const {
        return val * (ll) other.val;
    }
    ModInt& operator *= (const ModInt& other) {
        val = (val * (ll) other.val) % MOD;
        return *this;
    }
};

struct DSU {
    vector<int> data;
    vector<pair<int, int>> rng;
    set<int> cmps;

    DSU(const vector<pair<int, int>> a) {
        data.resize(a.size(), -1);
        rng = a;
        for (int i = 0; i < a.size(); ++i)
            cmps.insert(i);
    }

    int get(int v) {
        if (data[v] < 0)
            return v;
        return data[v] = get(data[v]);
    }

    bool unite(int v, int u) {
        v = get(v); u = get(u);
        if (v == u)
            return false;
        if (data[v] > data[u])
            swap(v, u);
        data[v] += data[u];
        data[u] = v;
        maxi(rng[v].first, rng[u].first);
        mini(rng[v].second, rng[u].second);
        cmps.erase(u);
        return true;
    }

    int get_rng(int i) {
        if (rng[i].first > rng[i].second)
            return 0;
        return rng[i].second - rng[i].first + 1;
    }

};

const int N = 3003;
int tin[N], tout[N];
vector<int> g[N];
int p[N];

int t = 0;
void dfs(int v) {
    tin[v] = t++;
    for (int u :g[v]) {
        dfs(u);
    }
    tout[v] = t++;
}

bool is_ancestor(int v, int u) {
    return (tin[v] <= tin[u] && tout[u] <= tout[v]);
}

int nxt(int v, int u) {
    if (is_ancestor(v, u)) {
        for (int w :g[v]) {
            if (is_ancestor(w, u))
                return w;
        }
        return v;
    }
    else {
        return p[v];
    }
}

int main() {
    ios_base::sync_with_stdio(false); cin.tie(nullptr); cout.tie(nullptr);
#ifdef DUDE
    freopen("input.txt", "r", stdin);
#else
    freopen("input.txt", "r", stdin);
#endif
    int n;
    cin >> n;
    for (int i = 1; i < n; ++i) {
        cin >> p[i];
        --p[i];
        g[p[i]].push_back(i);
    }
    vector<pair<int, int>> ra(n);
    for (int i = 0; i < n; ++i) {
        cin >> ra[i].first >> ra[i].second;
    }
    dfs(0);
    DSU dsu(ra);
    int m;
    cin >> m;
    ll ans = -1;
    while (m--) {
       int a, b, c, d;
       cin >> a >> b >> c >> d;
       --a; --b; --c; --d;
        if (ans == 0) {
            cout << "0\n";
            continue;
        }
        while (dsu.cmps.size() > 1) {
            dsu.unite(a, c);
            if (a == b)
                break;
            a = nxt(a, b);
            c = nxt(c, d);
        }
        ans = 1;
        for (int i :dsu.cmps) {
            ans = (ans * dsu.get_rng(i)) % MOD;
        }
        cout << ans << '\n';
    }
}
