#include <iostream>
#include <algorithm>
#include <vector>
#include <string>
#include <queue>
#include <deque>
#include <set>
#include <map>

using namespace std;

typedef long long ll;
typedef long double ld;

#define all(a) a.begin(), a.end()
#define rall(a) a.rbegin(), a.rend()
#define len(a) (int)a.size()
#define endl '\n'
#define debug(a) cout << #a << ": " << a << endl;

const int mod = 1e9 + 7;
const int mxl = 24;

// author: @cot-in

void fin() {
#ifdef cot_in
    freopen("in.txt", "r", stdin);
    freopen("out.txt", "w", stdout);
#endif
}

int t = 0;

int mul(int a, int b) {
    return (1ll * a * 1ll * b) % mod;
}

bool ancestor(int v, int u, vector<int> &tin, vector<int> &tout) {
    return (tin[v] <= tin[u] && tout[v] >= tout[u]);
}

void dfs(int v, vector<vector<int>> &g, vector<bool> &used, vector<int> &p, vector<int> &tin, vector<int> &tout) {
    used[v] = true;
    tin[v] = t++;
    for (auto u: g[v]) {
        if (!used[u]) {
            p[u] = v;
            dfs(u, g, used, p, tin, tout);
        }
    }
    tout[v] = t++;
}

void build(int n, vector<vector<int>> &bup, vector<int> &p) {
    for (int v = 0; v < n; v++)
        bup[v][0] = p[v];
    for (int k = 1; k < mxl; k++) {
        for (int v = 0; v < n; v++) {
            int w = bup[v][k - 1];
            bup[v][k] = bup[w][k - 1];
        }
    }
}

int lca(int v, int u, vector<vector<int>> &bup, vector<int> &p, vector<int> &tin, vector<int> &tout) {
    if (ancestor(v, u, tin, tout))
        return v;
    if (ancestor(u, v, tin, tout))
        return u;
    for (int k = mxl - 1; k >= 0; k--) {
        int w = bup[v][k];
        if (!ancestor(w, u, tin, tout))
            v = w;
    }
    return p[v];
}

vector<int> get_path(int v, int u, vector<vector<int>> &bup, vector<int> &p, vector<int> &tin, vector<int> &tout) {
    int fl = lca(v, u, bup, p, tin, tout);
    vector<int> path;
    if (fl == v) { // v - предок u
        int cur = u;
        while (cur != v) {
            path.push_back(cur);
            cur = p[cur];
        }
        path.push_back(cur);
        reverse(all(path));
    } else if (fl == u) { // u - предок v
        int cur = v;
        while (cur != u) {
            path.push_back(cur);
            cur = p[cur];
        }
        path.push_back(cur);
    } else { // fl - lca v и u
        int cur = v;
        while (cur != fl) {
            path.push_back(cur);
            cur = p[cur];
        }
        cur = u;
        vector<int> add_path;
        while (cur != fl) {
            add_path.push_back(cur);
            cur = p[cur];
        }
        add_path.push_back(cur);
        reverse(all(add_path));
        for (auto el: add_path)
            path.push_back(el);
    }
    return path;
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr), fin();
    int n, m;
    cin >> n;
    vector<vector<int>> g(n), bup(n, vector<int>(mxl, 0));
    vector<bool> used(n, false);
    vector<int> tin(n), tout(n), p(n, 0);
    vector<pair<int, int>> vc(n);
    for (int u = 1; u < n; u++) {
        int v;
        cin >> v;
        v--;
        g[v].push_back(u);
        g[u].push_back(v);
    }
    dfs(0, g, used, p, tin, tout);
    build(n, bup, p);
    for (int i = 0; i < n; i++)
        cin >> vc[i].first >> vc[i].second;
    cin >> m;
    while (m--) {
        int a, b, c, d;
        cin >> a >> b >> c >> d;
        a--;
        b--;
        c--;
        d--;
        vector<int> path1 = get_path(a, b, bup, p, tin, tout);
        vector<int> path2 = get_path(c, d, bup, p, tin, tout);
//        cout << "path 1: " << endl;
//        for (auto el: path1) {
//            cout << el + 1 << ' ';
//        }
//        cout << endl;
//        cout << "path 2: " << endl;
//        for (auto el: path2) {
//            cout << el + 1 << ' ';
//        }
//        cout << endl;
        int length = len(path1);
        int cnt = 1;
        vector<bool> was(n, false);
        for (int i = 0; i < length; i++) {
            int v = path1[i], u = path2[i];
            int l = max(vc[v].first, vc[u].first);
            int r = min(vc[v].second, vc[u].second);
            if (l > r)
                cnt = 0;
            vc[v] = vc[u] = make_pair(l, r);
            if (!was[v] || !was[u])
                cnt = mul(cnt, r - l + 1);
            was[v] = was[u] = true;
        }
//        cout << "current vc: " << endl;
//        for (auto [l, r]: vc) {
//            cout << l << ' ' << r << endl;
//        }
        for (int v = 0; v < n; v++) {
            if (!was[v])
                cnt = mul(cnt, vc[v].second - vc[v].first + 1);
        }
        cout << cnt << endl;
    }
    return 0;
}