#pragma GCC optimize("Ofast")

#include <bits/stdc++.h>
#include <bit>

using namespace std;

#define get_time() ((double)clock() / (double)CLOCKS_PER_SEC)

#define all(_a) _a.begin(), _a.end()
#define rall(_a) _a.rbegin(), _a.rend()

const int MAX_N = 2e5;
const int MAX_LOG = 18;
const int MOD = 1e9 + 7;

int mul(int a, int b) {
    return 1ll * a * b % MOD;
}

int power(int a, int n) {
    if (n == 0) {
        return 1;
    }
    if (n % 2 == 0) {
        return power(mul(a, a), n >> 1);
    }
    return mul(a, power(a, n - 1));
}

int get_reverse(int a) {
    return power(a, MOD - 2);
}

int n;

vector<int> graph[MAX_N];
int parent[MAX_N];

int tin[MAX_N];
int tout[MAX_N];

int timer;

int answer;

void dfs(int v) {
    tin[v] = timer++;
    for (int u : graph[v]) {
        dfs(u);
    }
    tout[v] = timer++;
}

bool is_ancestor(int v, int u) {
    return tin[v] <= tin[u] && tout[v] >= tout[u];
}

int dp[MAX_LOG][MAX_N];

void cacl_dp() {
    for (int i = 0; i < n; ++i) {
        dp[0][i] = parent[i];
    }
    for (int k = 1; k < MAX_LOG; ++k) {
        for (int v = 0; v < n; ++v) {
            dp[k][v] = dp[k - 1][dp[k - 1][v]];
        }
    }
}

int get_lca(int v, int u) {
    if (is_ancestor(v, u)) {
        return v;
    }
    if (is_ancestor(u, v)) {
        return u;
    }
    for (int k = MAX_LOG - 1; k >= 0; --k) {
        if (!is_ancestor(dp[k][v], u)) {
            v = dp[k][v];
        }
    }
    return parent[v];
}

struct Segment {
    int l;
    int r;

    Segment() {}
    Segment(int l, int r) : l(l), r(r) {}
    int get_len() const;
};

int Segment::get_len() const {
    return max(0, r - l + 1);
}

istream& operator >> (istream& in, Segment& seg) {
    in >> seg.l >> seg.r;
    return in;
}

Segment merge(const Segment& a, const Segment& b) {
    return Segment(max(a.l, b.l), min(a.r, b.r));
}

Segment segments[MAX_N];

int parent_dsu[MAX_N];

int get_parent(int v) {
    if (parent_dsu[v] == v) {
        return v;
    }
    return parent_dsu[v] = get_parent(parent_dsu[v]);
}

int num_test;
int m;

void unite(int v, int u) {
    v = get_parent(v);
    u = get_parent(u);
    if (v != u) {
        parent_dsu[v] = u;
        int mul_r = mul(get_reverse(segments[v].get_len()),
                        get_reverse(segments[u].get_len()));
        answer = mul(answer, mul_r);
        segments[u] = merge(segments[u], segments[v]);
        if (segments[u].get_len() == 0) {
            for (int i = 0; i < m - num_test; ++i) {
                cout << 0 << '\n';
            }
            exit(0);
        }
        answer = mul(answer, segments[u].get_len());
    }
}

vector<int> get_vertex_path(int v, int u) {
    vector<int> ans;
    while (v != u) {
        ans.push_back(v);
        v = parent[v];
    }
    return ans;
}

vector<int> get_path(int v, int u) {
    int w = get_lca(v, u);

    vector<int> path_v = get_vertex_path(v, w);
    vector<int> path_u = get_vertex_path(u, w);
    reverse(all(path_u));
    path_v.push_back(w);
    path_v.insert(path_v.end(), all(path_u));
    return path_v;
}

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);

#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif

    cin >> n;
    for (int i = 1; i < n; ++i) {
        cin >> parent[i];
        --parent[i];
    }
    for (int i = 1; i < n; ++i) {
        graph[parent[i]].push_back(i);
    }
    dfs(0);
    cacl_dp();

    for (int i = 0; i < n; ++i) {
        cin >> segments[i];
    }
    cin >> m;
    iota(parent_dsu, parent_dsu + n, 0);

    answer = 1;
    for (int i = 0; i < n; ++i) {
        answer = mul(answer, segments[i].get_len());
    }
    for (num_test = 0; num_test < m; ++num_test) {
        int a, b, c, d;
        cin >> a >> b >> c >> d;
        --a;
        --b;
        --c;
        --d;
        vector<int> path1 = get_path(a, b);
        vector<int> path2 = get_path(c, d);
        assert((int)path1.size() == (int)path2.size());


        int len = (int)path1.size();
        for (int i = 0; i < len; ++i) {
            unite(path1[i], path2[i]);
        }
        cout << answer << '\n';
    }
}