#include <iostream>
#include <cmath>
#include <fstream>
#include <algorithm>
#include <vector>
#include <map>
#include <set>
#include <ext/pb_ds/assoc_container.hpp>
#include <numeric>
//#include <ext/pb_ds/>

using namespace __gnu_pbds;
using namespace std;

#define int long long
#define all(x) (x).begin(), (x).end()
#define sz(x) (int) ((x).size())

template<typename T>
using ordered_set = tree<T, null_type, less<T>, rb_tree_tag, tree_order_statistics_node_update>;

#ifdef LOCAL
#include "algo/debug.h"
#else
#define debug(...) 42
#endif

const int mod = 1e9 + 7;
const int P = 239017;

int mul(int a, int b) {
  return a * b % mod;
}

int binpow(int a, int n) {
  int res = 1;
  while (n) {
    if (n % 2) {
      res = mul(res, a);
    }
    a = mul(a, a);
    n /= 2;
  }

  return res;
}

int rev(int x) {
  return binpow(x, mod - 2);
}

void solve() {
  int n;
  cin >> n;
  vector<int> p(n, -1);
  vector<vector<int>> g(n);
  for (int i = 1; i < n; i++) {
    cin >> p[i];
    --p[i];
    g[p[i]].push_back(i);
  }

  int res = 1;
  vector<int> lb(n), rb(n);
  for (int i = 0; i < n; i++) {
    cin >> lb[i] >> rb[i];
    res = mul(res, rb[i] - lb[i] + 1);
  }

  vector<int> sizeSubtree(n);
  function<void(int)> hldReorder = [&] (int v) {
    sizeSubtree[v] = 1;
    for (auto &u : g[v]) {
      hldReorder(u);
      sizeSubtree[v] += sizeSubtree[u];
    }
    sort(all(g[v]), [&] (const int& lhs, const int& rhs) {
      return sizeSubtree[lhs] > sizeSubtree[rhs];
    });
  };
  hldReorder(0);

  int timer = 0;
  vector<int> tin(n), tout(n), head(n);
  function<void(int)> hld = [&] (int v) {
    tin[v] = timer++;
    for (auto &u : g[v]) {
      head[u] = (u == g[v][0] ? head[v] : u);
      hld(u);
    }
    tout[v] = timer;
  };
  hld(0);

  auto anc = [&] (int u, int v) -> bool {
    return tin[u] <= tin[v] && tin[v] < tout[u];
  };

  vector<int> par(n), size(n);
  iota(all(par), 0);
  fill(all(size), 1);

  function<int(int)> get = [&] (int v) {
    return par[v] == v ? v : par[v] = get(par[v]);
  };

  auto unite = [&] (int a, int b) {
    a = get(a);
    b = get(b);
    if (a == b) {
      return;
    }
    if (size[b] > size[a]) {
      swap(a, b);
    }
    par[b] = a;
    size[a] += size[b];
    res = mul(res, rev(rb[b] - lb[b] + 1));
    res = mul(res, rev(rb[a] - lb[a] + 1));
    lb[a] = max(lb[a], lb[b]);
    rb[a] = min(rb[a], rb[b]);
    res = mul(res, max(0ll, rb[a] - lb[a] + 1));
  };

  vector<int> kok(n);
  for (int i = 0; i < n; i++) {
    kok[tin[i]] = i;
  }

  auto get_segs = [&] (int a, int b) -> vector<pair<int, int>> {
    if (a == b) {
      return vector<pair<int, int>> {make_pair(tin[a], tin[a])};
    }
    vector<pair<int, int>> segs;
    while (!anc(head[a], b)) {
      segs.emplace_back(tin[a], tin[head[a]]);
      a = p[head[a]];
    }
    vector<pair<int, int>> segs_rev;
    while (!anc(head[b], a)) {
      segs_rev.emplace_back(tin[head[b]], tin[b]);
      b = p[head[b]];
    }
    segs.emplace_back(tin[a], tin[b]);
    reverse(all(segs_rev));
    for (auto &x : segs_rev) {
      segs.push_back(x);
    }

    return segs;
  };

  int m;
  cin >> m;
  for (int w = 0; w < m; w++) {
    int a, b, c, d;
    cin >> a >> b >> c >> d;
    --a, --b, --c, --d;
    vector<pair<int, int>> segs1 = get_segs(a, b), segs2 = get_segs(c, d);
    vector<int> path1, path2;
    { // construct path1
      for (auto &[l, r] : segs1) {
        if (l <= r) {
          for (int i = l; i <= r; i++) {
            path1.push_back(kok[i]);
          }
        } else {
          for (int i = l; i >= r; i--) {
            path1.push_back(kok[i]);
          }
        }
      }
    }
    { // construct path2
      for (auto &[l, r] : segs2) {
        if (l <= r) {
          for (int i = l; i <= r; i++) {
            path2.push_back(kok[i]);
          }
        } else {
          for (int i = l; i >= r; i--) {
            path2.push_back(kok[i]);
          }
        }
      }
    }
    assert(sz(path1) == sz(path2));
    for (int i = 0; i < sz(path1); i++) {
      unite(path1[i], path2[i]);
    }

    cout << res << '\n';
  }
}

int32_t main() {
#ifdef LOCAL
  freopen("input.txt", "r", stdin);
#endif
  ios_base::sync_with_stdio(false);
  cin.tie(nullptr);
  solve();
  return 0;
}