#include <iostream>
#include <cmath>
#include <fstream>
#include <algorithm>
#include <vector>
#include <map>
#include <set>
#include <ext/pb_ds/assoc_container.hpp>
#include <numeric>
//#include <ext/pb_ds/>

using namespace __gnu_pbds;
using namespace std;

//#define int long long
#define all(x) (x).begin(), (x).end()
#define sz(x) (int) ((x).size())

template<typename T>
using ordered_set = tree<T, null_type, less<T>, rb_tree_tag, tree_order_statistics_node_update>;

#ifdef LOCAL
#include "algo/debug.h"
#else
#define debug(...) 42
#endif

const int mod = 1e9 + 7;
const int P = 239017;
const int MX = 5e5 + 5;

int deg[MX];

int mul(int a, int b) {
  return 1ll * a * b % mod;
}

int binpow(int a, int n) {
  int res = 1;
  while (n) {
    if (n % 2) {
      res = mul(res, a);
    }
    a = mul(a, a);
    n /= 2;
  }

  return res;
}

int rev(int x) {
  return binpow(x, mod - 2);
}

struct Tree {
  vector<int> tree;
  int s;

  Tree() {}

  Tree(int n) {
    s = 1;
    while (s < n) s *= 2;
    tree.resize(2 * s + 1);
  }

  void set(int i, int val, int x, int lx, int rx) {
    if (lx + 1 == rx) {
      tree[x] = val;
      return;
    }
    int m = (lx + rx) / 2;
    if (i < m) {
      set(i, val, 2 * x + 1, lx, m);
    } else {
      set(i, val, 2 * x + 2, m, rx);
    }
    tree[x] = tree[2 * x + 1] * deg[rx - m] + tree[2 * x + 2];
  }

  void set(int i, int val) {
    set(i, val + 1, 0, 0, s);
  }

  pair<int, int> get(int l, int r, int x, int lx, int rx) {
    if (lx >= r || rx <= l) {
      return make_pair(0, 0);
    }
    if (lx >= l && rx <= r) {
      return make_pair(rx - lx, tree[x]);
    }
    int m = (lx + rx) / 2;
    auto [lenL, hashL] = get(l, r, 2 * x + 1, lx, m);
    auto [lenR, hashR] = get(l, r, 2 * x + 2, m, rx);

    return make_pair(lenL + lenR, hashL * deg[lenR] + hashR);
  }

  int get(int l, int r) {
    return get(l, r, 0, 0, s).second;
  }
};

const int N = 2e5 + 5;

Tree st;
vector<int> g[N];
int sizeSubtree[N], timer = 0, tin[N], tout[N], head[N], kok[N];

void hldReorder(int v) {
  sizeSubtree[v] = 1;
  for (auto &u : g[v]) {
    hldReorder(u);
    sizeSubtree[v] += sizeSubtree[u];
  }
  sort(all(g[v]), [&] (const int& lhs, const int& rhs) {
    return sizeSubtree[lhs] > sizeSubtree[rhs];
  });
}

void hld(int v) {
  tin[v] = timer++;
  for (auto &u : g[v]) {
    head[u] = (u == g[v][0] ? head[v] : u);
    hld(u);
  }
  tout[v] = timer;
}

bool anc(int u, int v) {
  return tin[u] <= tin[v] && tin[v] < tout[u];
}

int par[N], sz[N], n, p[N];

int get(int v) {
  if (par[v] == v) {
    return v;
  }
  int np = get(par[v]);
  par[v] = np;
  st.set(tin[v], np);
  st.set(2 * n - 1 - tin[v], np);
  return np;
}


void solve() {
  deg[0] = 1;
  for (int i = 1; i < MX; i++) {
    deg[i] = deg[i - 1] * P;
  }

  cin >> n;
  for (int i = 1; i < n; i++) {
    cin >> p[i];
    --p[i];
    g[p[i]].push_back(i);
  }

  int res = 1;
  vector<int> lb(n), rb(n);
  for (int i = 0; i < n; i++) {
    cin >> lb[i] >> rb[i];
    res = mul(res, rb[i] - lb[i] + 1);
  }

  hldReorder(0);
  hld(0);

  for (int i = 0; i < n; i++) {
    kok[tin[i]] = i;
  }

  st = Tree(2 * n);
  for (int i = 0; i < n; i++) {
    st.set(i, kok[i]);
    st.set(2 * n - 1 - i, kok[i]);
  }

  iota(par, par + n, 0);
  fill(sz, sz + n, 1);

  auto unite = [&] (int a, int b) {
    a = get(a);
    b = get(b);
    if (a == b) {
      return;
    }
    if (sz[b] > sz[a]) {
      swap(a, b);
    }
    st.set(tin[b], a);
    st.set(2 * n - 1 - tin[b], a);
    par[b] = a;
    sz[a] += sz[b];
    res = mul(res, rev(rb[b] - lb[b] + 1));
    res = mul(res, rev(rb[a] - lb[a] + 1));
    lb[a] = max(lb[a], lb[b]);
    rb[a] = min(rb[a], rb[b]);
    res = mul(res, max(0, rb[a] - lb[a] + 1));
  };

  auto get_borders = [&] (int l, int r, int id_from, int id_to) {
    if (l <= r) {
      return make_pair(l + id_from, l + id_to);
    }
    return make_pair(l - id_from, l - id_to);
  };

  auto getHash = [&] (int l, int r, int len) -> int {
    int id_beg = (l <= r ? l : 2 * n - 1 - l);
    return st.get(id_beg, id_beg + len);
  };

  auto get_id = [&] (int l, int r, int pos) {
    if (l <= r) {
      return l + pos;
    }
    return l - pos;
  };

  int m;
  cin >> m;
  for (int w = 0; w < m; w++) {
    int a, b, c, d;
    cin >> a >> b >> c >> d;
    --a, --b, --c, --d;
    vector<pair<int, int>> segs1, segs2;
    if (a == b) {
      segs1.emplace_back(tin[a], tin[a]);
    } else {
      while (!anc(head[a], b)) {
        segs1.emplace_back(tin[a], tin[head[a]]);
        a = p[head[a]];
      }
      vector<pair<int, int>> segs_rev;
      while (!anc(head[b], a)) {
        segs_rev.emplace_back(tin[head[b]], tin[b]);
        b = p[head[b]];
      }
      segs1.emplace_back(tin[a], tin[b]);
      reverse(all(segs_rev));
      for (auto &x : segs_rev) {
        segs1.push_back(x);
      }
    }
    if (c == d) {
      segs2.emplace_back(tin[c], tin[c]);
    } else {
      while (!anc(head[c], d)) {
        segs2.emplace_back(tin[c], tin[head[c]]);
        c = p[head[c]];
      }
      vector<pair<int, int>> segs_rev;
      while (!anc(head[d], c)) {
        segs_rev.emplace_back(tin[head[d]], tin[c]);
        d = p[head[d]];
      }
      segs2.emplace_back(tin[c], tin[d]);
      reverse(all(segs_rev));
      for (auto &x : segs_rev) {
        segs2.push_back(x);
      }
    }
    vector<pair<int, int>> s1, s2;
    int ptr1 = 0, ptr2 = 0, used1 = 0, used2 = 0;
    while (ptr1 < sz(segs1) && ptr2 < sz(segs2)) {
      int len1 = abs(segs1[ptr1].second - segs1[ptr1].first) + 1;
      int len2 = abs(segs2[ptr2].second - segs2[ptr2].first) + 1;
      int can1 = len1 - used1;
      int can2 = len2 - used2;
      int add = min(can1, can2);
      s1.push_back(get_borders(segs1[ptr1].first, segs1[ptr1].second, used1, used1 + add - 1));
      s2.push_back(get_borders(segs2[ptr2].first, segs2[ptr2].second, used2, used2 + add - 1));
      used1 += add;
      used2 += add;
      if (used1 == len1) {
        ptr1++;
        used1 = 0;
      }
      if (used2 == len2) {
        ptr2++;
        used2 = 0;
      }
    }

    assert(sz(s1) == sz(s2));
    for (int i = 0; i < sz(s1); i++) {
      auto [l1, r1] = s1[i];
      auto [l2, r2] = s2[i];

      int len = abs(l1 - r1) + 1;
      while (getHash(l1, r1, len) != getHash(l2, r2, len)) {
        int l = 0, r = len + 1;
        while (l + 1 < r) {
          int mid = (l + r) / 2;
          if (getHash(l1, r1, mid) == getHash(l2, r2, mid)) {
            l = mid;
          } else {
            r = mid;
          }
        }
        unite(kok[get_id(l1, r1, l)], kok[get_id(l2, r2, l)]);
      }
    }

    cout << res << '\n';
  }
}

int32_t main() {
#ifdef LOCAL
  freopen("input.txt", "r", stdin);
#endif
  ios_base::sync_with_stdio(false);
  cin.tie(nullptr);
  solve();
  return 0;
}